#nullable enable
using Canon.Core.Abstractions;
using Canon.Core.GrammarParser;
using Canon.Core.Enums;
namespace Canon.Core.GrammarParser;

public class GeneratedTransformer : ITransformer
{
    private IDictionary<TerminatorBase, string> _shiftPointers;

    public string Name { get; }

    public IDictionary<Terminator, ReduceInformation> ReduceTable { get; }

    public IDictionary<TerminatorBase, ITransformer> ShiftTable { get; }

    public GeneratedTransformer(Dictionary<TerminatorBase, string> shiftTable,
        Dictionary<Terminator, ReduceInformation> reduceTable, string name)
    {
        ReduceTable = reduceTable;
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = shiftTable;
        Name = name;
    }

    public GeneratedTransformer()
    {
        ReduceTable = new Dictionary<Terminator, ReduceInformation>();
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = new Dictionary<TerminatorBase, string>();
        Name = Guid.NewGuid().ToString();
    }

    public void ConstructShiftTable(Dictionary<string, GeneratedTransformer> transformers)
    {
        foreach (KeyValuePair<TerminatorBase,string> pair in _shiftPointers)
        {
            ShiftTable.Add(pair.Key, transformers[pair.Value]);
        }
    }

    public override bool Equals(object? obj)
    {
        if (obj is not GeneratedTransformer other)
        {
            return false;
        }

        return Name == other.Name;
    }

    public override int GetHashCode() => Name.GetHashCode();
}
public class GeneratedGrammarParser : IGrammarParser
{
	private static readonly Dictionary<string, GeneratedTransformer> s_transformers = new()
	{
		{ "0f0898d1-ab59-485e-8cb8-e1f9f76c9fed", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramStruct), "67c788e2-4042-422d-be8b-ec85c023aeec"}, { new NonTerminator(NonTerminatorType.ProgramHead), "26733119-00e8-4fc2-9070-8d72644a2036"}, { new Terminator(KeywordType.Program), "1c000c6c-81b6-409d-9fc2-a20d186645a1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0f0898d1-ab59-485e-8cb8-e1f9f76c9fed") },
		{ "67c788e2-4042-422d-be8b-ec85c023aeec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.StartNonTerminator))}, }, "67c788e2-4042-422d-be8b-ec85c023aeec") },
		{ "26733119-00e8-4fc2-9070-8d72644a2036", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "d2adc71a-e1c9-4502-9e80-4bad232ca7a4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "26733119-00e8-4fc2-9070-8d72644a2036") },
		{ "1c000c6c-81b6-409d-9fc2-a20d186645a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "70746689-8bee-4e9c-83d0-db89487e50dc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1c000c6c-81b6-409d-9fc2-a20d186645a1") },
		{ "d2adc71a-e1c9-4502-9e80-4bad232ca7a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramBody), "35e53ba3-93fa-40de-ad8e-db98d5117cf4"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "7fce735e-7c0d-4618-8852-8a3c801f141a"}, { new Terminator(KeywordType.Const), "285a7f38-cf69-4374-b14f-d4c49bf8001e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "d2adc71a-e1c9-4502-9e80-4bad232ca7a4") },
		{ "70746689-8bee-4e9c-83d0-db89487e50dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "4a46b621-096a-44fa-94ac-d420ce5798cf"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "70746689-8bee-4e9c-83d0-db89487e50dc") },
		{ "35e53ba3-93fa-40de-ad8e-db98d5117cf4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Period), "87efe697-1215-4d97-80ee-b9aaa6af994e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "35e53ba3-93fa-40de-ad8e-db98d5117cf4") },
		{ "7fce735e-7c0d-4618-8852-8a3c801f141a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "b2310a48-1e22-4fb0-8d7a-0c256a1fe579"}, { new Terminator(KeywordType.Var), "27dc734c-f84e-4740-a097-d3b020443e46"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "7fce735e-7c0d-4618-8852-8a3c801f141a") },
		{ "285a7f38-cf69-4374-b14f-d4c49bf8001e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "45e3d63c-a839-42b5-b28e-70a37f34b943"}, { Terminator.IdentifierTerminator, "02c5d167-0f27-4e2e-882f-6c3e787f5048"},}, new Dictionary<Terminator, ReduceInformation>{ }, "285a7f38-cf69-4374-b14f-d4c49bf8001e") },
		{ "4a46b621-096a-44fa-94ac-d420ce5798cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "6ae9de67-4dc4-4c0e-aa78-7674c24c11dd"}, { new Terminator(DelimiterType.Comma), "57ed9f24-34a1-4158-8d6d-1bc6afb23cee"}, { new Terminator(DelimiterType.Colon), "76c923f3-0e75-4685-acb0-5578ccdd7974"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4a46b621-096a-44fa-94ac-d420ce5798cf") },
		{ "87efe697-1215-4d97-80ee-b9aaa6af994e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramStruct))}, }, "87efe697-1215-4d97-80ee-b9aaa6af994e") },
		{ "b2310a48-1e22-4fb0-8d7a-0c256a1fe579", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramDeclarations), "3c9ec8c2-fc2c-4863-beff-9e2aa43440fb"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "b2310a48-1e22-4fb0-8d7a-0c256a1fe579") },
		{ "27dc734c-f84e-4740-a097-d3b020443e46", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "4205ef1a-77b6-437d-8530-e439292c4a1c"}, { Terminator.IdentifierTerminator, "68100ff0-d251-41c2-a498-ad3e6bf36631"},}, new Dictionary<Terminator, ReduceInformation>{ }, "27dc734c-f84e-4740-a097-d3b020443e46") },
		{ "45e3d63c-a839-42b5-b28e-70a37f34b943", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "3930f2d5-c781-44f5-81ba-9331e03d9449"},}, new Dictionary<Terminator, ReduceInformation>{ }, "45e3d63c-a839-42b5-b28e-70a37f34b943") },
		{ "02c5d167-0f27-4e2e-882f-6c3e787f5048", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "29030cf9-7882-497e-a850-2176d8d1ac9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "02c5d167-0f27-4e2e-882f-6c3e787f5048") },
		{ "6ae9de67-4dc4-4c0e-aa78-7674c24c11dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "433895ab-fe04-47a8-a830-dfaabf8e600a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6ae9de67-4dc4-4c0e-aa78-7674c24c11dd") },
		{ "57ed9f24-34a1-4158-8d6d-1bc6afb23cee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ea693d7a-5c36-46e1-a9e8-4ab50bb008a5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "57ed9f24-34a1-4158-8d6d-1bc6afb23cee") },
		{ "76c923f3-0e75-4685-acb0-5578ccdd7974", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "23910d0c-c8e2-4ffb-9c41-1bec801a7e52"}, { new NonTerminator(NonTerminatorType.BasicType), "92158c89-e004-4a0a-990d-733ca1814b3b"}, { new Terminator(KeywordType.Array), "02aabdd7-2b85-4cb3-8652-04e31e84bb46"}, { new Terminator(KeywordType.Integer), "4e4cfe2e-0f81-4126-9641-569669e59526"}, { new Terminator(KeywordType.Real), "478d5073-98fb-4782-838f-b791bcad8110"}, { new Terminator(KeywordType.Boolean), "2b873f3e-64fa-493b-9e04-a4f7cce82410"}, { new Terminator(KeywordType.Character), "308ca356-663b-428e-a883-c7a002b346dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "76c923f3-0e75-4685-acb0-5578ccdd7974") },
		{ "3c9ec8c2-fc2c-4863-beff-9e2aa43440fb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "4c331433-3970-4998-b091-96c87b5664ae"}, { new Terminator(KeywordType.Begin), "6c3e788f-4161-4424-9d98-864f500ace62"}, { new NonTerminator(NonTerminatorType.Subprogram), "96468d8a-8c99-4418-933e-83f7dd596832"}, { new NonTerminator(NonTerminatorType.SubprogramHead), "e001ef07-76f2-4133-aa17-e3aaa3d207a4"}, { new Terminator(KeywordType.Procedure), "8f3e8204-7b37-4303-a226-1e5a7b8cfb8f"}, { new Terminator(KeywordType.Function), "bc9576fa-15d4-47cb-81d5-d4dffb05706b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3c9ec8c2-fc2c-4863-beff-9e2aa43440fb") },
		{ "4205ef1a-77b6-437d-8530-e439292c4a1c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "d95177d7-db03-4595-9f26-4403e0cd0b55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4205ef1a-77b6-437d-8530-e439292c4a1c") },
		{ "68100ff0-d251-41c2-a498-ad3e6bf36631", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "3815143f-fdc6-414e-bc46-9029e579b712"}, { new Terminator(DelimiterType.Comma), "f6f203e8-f94b-4154-8c4e-e745a722afd9"}, { new Terminator(DelimiterType.Colon), "0c8375ba-f970-4b74-a1ae-deeb17528fce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "68100ff0-d251-41c2-a498-ad3e6bf36631") },
		{ "3930f2d5-c781-44f5-81ba-9331e03d9449", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "699372ae-253a-4afd-b887-7fcb4b74f3fe"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "3930f2d5-c781-44f5-81ba-9331e03d9449") },
		{ "29030cf9-7882-497e-a850-2176d8d1ac9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "50fd9c66-3791-4be6-a59e-904c57c61cd3"}, { new Terminator(OperatorType.Plus), "ae04afcb-56a3-4aaf-b0d4-255b18179e5d"}, { new Terminator(OperatorType.Minus), "d1c13361-fda9-47ff-8f88-406609964c37"}, { Terminator.NumberTerminator, "abdf3dc7-c517-4bf0-8e5c-dd0ddca36723"}, { Terminator.CharacterTerminator, "1821133f-4457-45fa-a40c-508e02da142e"}, { new Terminator(KeywordType.True), "20cb6577-3347-43d0-b52c-369bedee73f4"}, { new Terminator(KeywordType.False), "f3322a0b-6888-4424-90c3-ffa5d46f7f2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "29030cf9-7882-497e-a850-2176d8d1ac9c") },
		{ "433895ab-fe04-47a8-a830-dfaabf8e600a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "433895ab-fe04-47a8-a830-dfaabf8e600a") },
		{ "ea693d7a-5c36-46e1-a9e8-4ab50bb008a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "7238be54-d35b-4416-adb4-06d94d016884"}, { new Terminator(DelimiterType.Comma), "57ed9f24-34a1-4158-8d6d-1bc6afb23cee"}, { new Terminator(DelimiterType.Colon), "76c923f3-0e75-4685-acb0-5578ccdd7974"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ea693d7a-5c36-46e1-a9e8-4ab50bb008a5") },
		{ "23910d0c-c8e2-4ffb-9c41-1bec801a7e52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "23910d0c-c8e2-4ffb-9c41-1bec801a7e52") },
		{ "92158c89-e004-4a0a-990d-733ca1814b3b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "92158c89-e004-4a0a-990d-733ca1814b3b") },
		{ "02aabdd7-2b85-4cb3-8652-04e31e84bb46", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "3ca94c27-c5f3-414d-b575-03ac7208943a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "02aabdd7-2b85-4cb3-8652-04e31e84bb46") },
		{ "4e4cfe2e-0f81-4126-9641-569669e59526", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "4e4cfe2e-0f81-4126-9641-569669e59526") },
		{ "478d5073-98fb-4782-838f-b791bcad8110", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "478d5073-98fb-4782-838f-b791bcad8110") },
		{ "2b873f3e-64fa-493b-9e04-a4f7cce82410", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "2b873f3e-64fa-493b-9e04-a4f7cce82410") },
		{ "308ca356-663b-428e-a883-c7a002b346dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "308ca356-663b-428e-a883-c7a002b346dd") },
		{ "4c331433-3970-4998-b091-96c87b5664ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramBody))}, }, "4c331433-3970-4998-b091-96c87b5664ae") },
		{ "6c3e788f-4161-4424-9d98-864f500ace62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "fb26ba9f-fb0e-4632-a4db-60b5fea7420b"}, { new NonTerminator(NonTerminatorType.Statement), "f7d7f42c-bf10-43d4-8e97-409785b892ff"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "6c3e788f-4161-4424-9d98-864f500ace62") },
		{ "96468d8a-8c99-4418-933e-83f7dd596832", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "9cee2227-5764-4f89-b377-3036cab2899b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "96468d8a-8c99-4418-933e-83f7dd596832") },
		{ "e001ef07-76f2-4133-aa17-e3aaa3d207a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "ae90058b-e8f5-4ea1-b294-bf4e706cbceb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e001ef07-76f2-4133-aa17-e3aaa3d207a4") },
		{ "8f3e8204-7b37-4303-a226-1e5a7b8cfb8f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "e8d72994-ef15-4062-be4e-abe193d1c9ec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8f3e8204-7b37-4303-a226-1e5a7b8cfb8f") },
		{ "bc9576fa-15d4-47cb-81d5-d4dffb05706b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "b7ee1011-30c8-4896-9e8d-3cb90712d2d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc9576fa-15d4-47cb-81d5-d4dffb05706b") },
		{ "d95177d7-db03-4595-9f26-4403e0cd0b55", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "03d461ff-96eb-4d00-8179-71306652b9ee"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "d95177d7-db03-4595-9f26-4403e0cd0b55") },
		{ "3815143f-fdc6-414e-bc46-9029e579b712", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "3815143f-fdc6-414e-bc46-9029e579b712") },
		{ "f6f203e8-f94b-4154-8c4e-e745a722afd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "a3d6e6c8-475b-4cfd-9b50-239b188d6ba6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f6f203e8-f94b-4154-8c4e-e745a722afd9") },
		{ "0c8375ba-f970-4b74-a1ae-deeb17528fce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "93cc15c4-42cb-40e0-9ae7-2484ef1b6160"}, { new NonTerminator(NonTerminatorType.BasicType), "d9fa30d0-d284-48b0-a708-9a3d5f06d5e6"}, { new Terminator(KeywordType.Array), "2636e88a-2bf6-41c8-8228-abfcd9420351"}, { new Terminator(KeywordType.Integer), "06bbef92-0d08-4369-a6e1-07308f373494"}, { new Terminator(KeywordType.Real), "141114e5-17fa-4ef5-8833-b7cf274019dc"}, { new Terminator(KeywordType.Boolean), "cfb5a6ab-7bb3-4d92-afae-7235a569a96f"}, { new Terminator(KeywordType.Character), "9ff8a64c-96bc-45f6-8b24-56e3e3dffd24"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0c8375ba-f970-4b74-a1ae-deeb17528fce") },
		{ "699372ae-253a-4afd-b887-7fcb4b74f3fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "edadbd73-30d1-4881-b148-f74b458633b5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "699372ae-253a-4afd-b887-7fcb4b74f3fe") },
		{ "50fd9c66-3791-4be6-a59e-904c57c61cd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "50fd9c66-3791-4be6-a59e-904c57c61cd3") },
		{ "ae04afcb-56a3-4aaf-b0d4-255b18179e5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "e6f1a00f-e50e-439a-a5c5-dc1fd79fca90"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae04afcb-56a3-4aaf-b0d4-255b18179e5d") },
		{ "d1c13361-fda9-47ff-8f88-406609964c37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "c665e0df-54e8-4b90-bfb6-024bb0bb7d55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d1c13361-fda9-47ff-8f88-406609964c37") },
		{ "abdf3dc7-c517-4bf0-8e5c-dd0ddca36723", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "abdf3dc7-c517-4bf0-8e5c-dd0ddca36723") },
		{ "1821133f-4457-45fa-a40c-508e02da142e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "1821133f-4457-45fa-a40c-508e02da142e") },
		{ "20cb6577-3347-43d0-b52c-369bedee73f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "20cb6577-3347-43d0-b52c-369bedee73f4") },
		{ "f3322a0b-6888-4424-90c3-ffa5d46f7f2a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "f3322a0b-6888-4424-90c3-ffa5d46f7f2a") },
		{ "7238be54-d35b-4416-adb4-06d94d016884", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "7238be54-d35b-4416-adb4-06d94d016884") },
		{ "3ca94c27-c5f3-414d-b575-03ac7208943a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "ad6f3d10-49fa-4d34-87d5-3d64d1e6ff0d"}, { Terminator.NumberTerminator, "07caf411-dce0-4707-a4d0-d40f3fe89cae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3ca94c27-c5f3-414d-b575-03ac7208943a") },
		{ "fb26ba9f-fb0e-4632-a4db-60b5fea7420b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "99ea311b-982d-43a4-a72b-203c016790f6"}, { new Terminator(DelimiterType.Semicolon), "e05f016b-b74f-46b6-a24f-f01a847e7d36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fb26ba9f-fb0e-4632-a4db-60b5fea7420b") },
		{ "f7d7f42c-bf10-43d4-8e97-409785b892ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, }, "f7d7f42c-bf10-43d4-8e97-409785b892ff") },
		{ "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "44f7909e-2841-40d4-9455-b497135ad871"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb") },
		{ "70a9a068-00ac-4fda-afd9-8420a497c944", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "70a9a068-00ac-4fda-afd9-8420a497c944") },
		{ "5e1696fb-becd-4945-a48e-278473b63380", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "5e1696fb-becd-4945-a48e-278473b63380") },
		{ "5f5cbf98-1028-45dd-99b3-fb92a2c3558f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c0db7335-cb6f-4550-8952-4c5a1c4a03aa"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b5011c14-e23b-4357-8e8f-09be3bdf7723"}, { new NonTerminator(NonTerminatorType.Term), "03a31cb3-b2c6-40d1-94ed-7254abf45605"}, { new NonTerminator(NonTerminatorType.Factor), "1937a590-6e09-4012-a71e-88f3f4dd738a"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5f5cbf98-1028-45dd-99b3-fb92a2c3558f") },
		{ "8d0836c1-fa55-4101-9a86-33aa16e2810b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "6191cfee-c879-45b8-a2bd-630d9f4d9513"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8d0836c1-fa55-4101-9a86-33aa16e2810b") },
		{ "b4e2291a-278c-4d06-a8db-b307d1fa6694", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdVarPart), "0df18bdb-69d7-4476-b9a3-1518a64f9c5d"}, { new Terminator(DelimiterType.LeftSquareBracket), "4b810105-5afb-4423-8349-fb69ac8898eb"}, { new Terminator(DelimiterType.LeftParenthesis), "66e1762b-7511-4013-98d1-2a67b9471380"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "b4e2291a-278c-4d06-a8db-b307d1fa6694") },
		{ "090bd9b1-fe24-47e8-a6a0-ba484add5b2d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "0c8556a6-fc62-4090-9a38-1a6180222516"}, { new NonTerminator(NonTerminatorType.Statement), "f7d7f42c-bf10-43d4-8e97-409785b892ff"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "090bd9b1-fe24-47e8-a6a0-ba484add5b2d") },
		{ "9cee2227-5764-4f89-b377-3036cab2899b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "9cee2227-5764-4f89-b377-3036cab2899b") },
		{ "ae90058b-e8f5-4ea1-b294-bf4e706cbceb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramBody), "94398520-756d-4e80-98dc-b0b0ada31327"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "5698b854-6cc8-426c-8e0a-15df3fdaae74"}, { new Terminator(KeywordType.Const), "c517db61-6413-415e-908e-3c7d905a16e5"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "ae90058b-e8f5-4ea1-b294-bf4e706cbceb") },
		{ "e8d72994-ef15-4062-be4e-abe193d1c9ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "548b7ae4-a55c-4488-a6e4-f4f0cc9f48ca"}, { new Terminator(DelimiterType.LeftParenthesis), "5aea1f37-fe95-462c-83de-fc3f266c473c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "e8d72994-ef15-4062-be4e-abe193d1c9ec") },
		{ "b7ee1011-30c8-4896-9e8d-3cb90712d2d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "4edbe980-4da2-48c9-905e-fbac861d9b82"}, { new Terminator(DelimiterType.LeftParenthesis), "fed7ec14-0384-4a37-8e58-9fb94d6f0a01"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "b7ee1011-30c8-4896-9e8d-3cb90712d2d1") },
		{ "03d461ff-96eb-4d00-8179-71306652b9ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "988e27b6-ffcf-4471-83cc-127f8860def6"}, { new Terminator(DelimiterType.Comma), "f6f203e8-f94b-4154-8c4e-e745a722afd9"}, { new Terminator(DelimiterType.Colon), "0c8375ba-f970-4b74-a1ae-deeb17528fce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "03d461ff-96eb-4d00-8179-71306652b9ee") },
		{ "a3d6e6c8-475b-4cfd-9b50-239b188d6ba6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "a98b06dd-241a-4b21-aff6-736e4446d82d"}, { new Terminator(DelimiterType.Comma), "f6f203e8-f94b-4154-8c4e-e745a722afd9"}, { new Terminator(DelimiterType.Colon), "0c8375ba-f970-4b74-a1ae-deeb17528fce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a3d6e6c8-475b-4cfd-9b50-239b188d6ba6") },
		{ "93cc15c4-42cb-40e0-9ae7-2484ef1b6160", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "93cc15c4-42cb-40e0-9ae7-2484ef1b6160") },
		{ "d9fa30d0-d284-48b0-a708-9a3d5f06d5e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "d9fa30d0-d284-48b0-a708-9a3d5f06d5e6") },
		{ "2636e88a-2bf6-41c8-8228-abfcd9420351", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "b01f7e65-1cb7-4bb6-9042-028fd4ffe5a5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2636e88a-2bf6-41c8-8228-abfcd9420351") },
		{ "06bbef92-0d08-4369-a6e1-07308f373494", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "06bbef92-0d08-4369-a6e1-07308f373494") },
		{ "141114e5-17fa-4ef5-8833-b7cf274019dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "141114e5-17fa-4ef5-8833-b7cf274019dc") },
		{ "cfb5a6ab-7bb3-4d92-afae-7235a569a96f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "cfb5a6ab-7bb3-4d92-afae-7235a569a96f") },
		{ "9ff8a64c-96bc-45f6-8b24-56e3e3dffd24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "9ff8a64c-96bc-45f6-8b24-56e3e3dffd24") },
		{ "edadbd73-30d1-4881-b148-f74b458633b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "88926413-9c08-4c39-9839-4bb6134c161c"}, { new Terminator(OperatorType.Plus), "ae04afcb-56a3-4aaf-b0d4-255b18179e5d"}, { new Terminator(OperatorType.Minus), "d1c13361-fda9-47ff-8f88-406609964c37"}, { Terminator.NumberTerminator, "abdf3dc7-c517-4bf0-8e5c-dd0ddca36723"}, { Terminator.CharacterTerminator, "1821133f-4457-45fa-a40c-508e02da142e"}, { new Terminator(KeywordType.True), "20cb6577-3347-43d0-b52c-369bedee73f4"}, { new Terminator(KeywordType.False), "f3322a0b-6888-4424-90c3-ffa5d46f7f2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "edadbd73-30d1-4881-b148-f74b458633b5") },
		{ "e6f1a00f-e50e-439a-a5c5-dc1fd79fca90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "e6f1a00f-e50e-439a-a5c5-dc1fd79fca90") },
		{ "c665e0df-54e8-4b90-bfb6-024bb0bb7d55", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "c665e0df-54e8-4b90-bfb6-024bb0bb7d55") },
		{ "ad6f3d10-49fa-4d34-87d5-3d64d1e6ff0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "72fc3a18-0c7e-4792-a02d-8183d0a0fe91"}, { new Terminator(DelimiterType.Comma), "9a62e1cc-7cdb-4336-94b3-720faa11196b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad6f3d10-49fa-4d34-87d5-3d64d1e6ff0d") },
		{ "07caf411-dce0-4707-a4d0-d40f3fe89cae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "c692111c-6167-4810-9113-b4d5693cccba"},}, new Dictionary<Terminator, ReduceInformation>{ }, "07caf411-dce0-4707-a4d0-d40f3fe89cae") },
		{ "99ea311b-982d-43a4-a72b-203c016790f6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "99ea311b-982d-43a4-a72b-203c016790f6") },
		{ "e05f016b-b74f-46b6-a24f-f01a847e7d36", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "677b577e-8689-41e0-a226-752b83f7d654"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "e05f016b-b74f-46b6-a24f-f01a847e7d36") },
		{ "44f7909e-2841-40d4-9455-b497135ad871", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f3238ebc-6b60-45c8-88d0-ff7c697017b0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "0c36ed43-e093-41b4-b3e0-ba81ac2cbc43"}, { new NonTerminator(NonTerminatorType.Term), "01eb5023-f7b5-4dfb-bf95-bd2fc156f3c5"}, { new NonTerminator(NonTerminatorType.Factor), "c162558f-ea41-4672-a1b6-a1c65cb84cfa"}, { Terminator.NumberTerminator, "cef53325-1b8c-4f08-a412-8b565a6ab7de"}, { new NonTerminator(NonTerminatorType.Variable), "d6d4980d-2f84-4091-8473-8a415cca0e85"}, { new Terminator(DelimiterType.LeftParenthesis), "f15bddef-d6f6-406c-9808-3fc153276ec9"}, { Terminator.IdentifierTerminator, "dea3950f-e624-454a-8580-81bd7c414142"}, { new Terminator(KeywordType.Not), "0eba9572-881e-4bb6-8b12-7cfb903af0fc"}, { new Terminator(OperatorType.Minus), "1278eb6a-3f98-4429-a858-7a7d9401db13"}, { new Terminator(OperatorType.Plus), "95843f41-66a5-4a63-9431-0b0fa2bfd9f2"}, { new Terminator(KeywordType.True), "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae"}, { new Terminator(KeywordType.False), "3a664267-532b-4bec-9dd8-37acd5ca4367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "44f7909e-2841-40d4-9455-b497135ad871") },
		{ "c0db7335-cb6f-4550-8952-4c5a1c4a03aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "a1cc474f-4114-4fcf-8761-fa3bf7af1111"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c0db7335-cb6f-4550-8952-4c5a1c4a03aa") },
		{ "b5011c14-e23b-4357-8e8f-09be3bdf7723", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "b1c91fcc-f7ee-4417-a4df-380b22b6c315"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "f4b20ae9-3e21-45ee-a940-4dcab928f9e4"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "b5011c14-e23b-4357-8e8f-09be3bdf7723") },
		{ "03a31cb3-b2c6-40d1-94ed-7254abf45605", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ddaf6ecb-88df-4574-9cc6-fc819989259f"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "03a31cb3-b2c6-40d1-94ed-7254abf45605") },
		{ "1937a590-6e09-4012-a71e-88f3f4dd738a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "1937a590-6e09-4012-a71e-88f3f4dd738a") },
		{ "5b1f2840-09aa-443e-bea9-3a12822d02c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "5b1f2840-09aa-443e-bea9-3a12822d02c1") },
		{ "d0c4e4b7-e350-48d1-aa20-0d76f65055d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d0c4e4b7-e350-48d1-aa20-0d76f65055d7") },
		{ "739d8a67-203c-4a82-83f2-6cef03928db7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "b288449a-1a6e-4a63-af68-892fd028a360"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "739d8a67-203c-4a82-83f2-6cef03928db7") },
		{ "07c9c6be-521a-40a7-ac31-85030d9e477e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "471f7efa-f893-45bf-8d96-544706e12bbc"}, { new NonTerminator(NonTerminatorType.IdVarPart), "8e80f472-0127-4d83-ad8f-980de7fee70b"}, { new Terminator(DelimiterType.LeftSquareBracket), "4b02d152-a939-4aa2-acf2-ed72f81900a5"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "07c9c6be-521a-40a7-ac31-85030d9e477e") },
		{ "c8ee831d-e142-46cb-8bb2-24b96f753c90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2e7ad752-6c24-4e00-947b-2004ddd97446"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c8ee831d-e142-46cb-8bb2-24b96f753c90") },
		{ "1c4c4fb4-28de-4325-a7de-79745c718e33", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "5fd76afe-6611-4938-862b-dd41afcd78d1"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1c4c4fb4-28de-4325-a7de-79745c718e33") },
		{ "cfb994ea-e037-45a7-aca3-e2682ed87464", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e6875378-4952-42ed-9946-9b45db9de896"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cfb994ea-e037-45a7-aca3-e2682ed87464") },
		{ "e61a0272-7657-4704-b8d2-1d348ebe0464", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e61a0272-7657-4704-b8d2-1d348ebe0464") },
		{ "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5") },
		{ "6191cfee-c879-45b8-a2bd-630d9f4d9513", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "2972b2e7-55b0-47ef-973d-eb51b98ff12f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6191cfee-c879-45b8-a2bd-630d9f4d9513") },
		{ "0df18bdb-69d7-4476-b9a3-1518a64f9c5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "0df18bdb-69d7-4476-b9a3-1518a64f9c5d") },
		{ "4b810105-5afb-4423-8349-fb69ac8898eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "615ce50c-31ca-499d-b10c-26fa4b579b09"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4b810105-5afb-4423-8349-fb69ac8898eb") },
		{ "66e1762b-7511-4013-98d1-2a67b9471380", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "31061387-a3d5-4c4b-975f-b12acce18ed3"}, { new NonTerminator(NonTerminatorType.ExpressionList), "51a25622-d03c-45eb-98bc-dfe8bb6fa7dc"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "66e1762b-7511-4013-98d1-2a67b9471380") },
		{ "0c8556a6-fc62-4090-9a38-1a6180222516", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "63bfeb19-e3a8-4cf9-8066-98ec790cf087"}, { new Terminator(DelimiterType.Semicolon), "e05f016b-b74f-46b6-a24f-f01a847e7d36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0c8556a6-fc62-4090-9a38-1a6180222516") },
		{ "94398520-756d-4e80-98dc-b0b0ada31327", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Subprogram))}, }, "94398520-756d-4e80-98dc-b0b0ada31327") },
		{ "5698b854-6cc8-426c-8e0a-15df3fdaae74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "098159a2-c899-4672-99d1-2be33e406320"}, { new Terminator(KeywordType.Var), "a9df0d14-abca-4771-835f-771caf8e2718"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "5698b854-6cc8-426c-8e0a-15df3fdaae74") },
		{ "c517db61-6413-415e-908e-3c7d905a16e5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "f637590f-1e50-4672-864a-0e634b2edcfa"}, { Terminator.IdentifierTerminator, "02c5d167-0f27-4e2e-882f-6c3e787f5048"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c517db61-6413-415e-908e-3c7d905a16e5") },
		{ "548b7ae4-a55c-4488-a6e4-f4f0cc9f48ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "548b7ae4-a55c-4488-a6e4-f4f0cc9f48ca") },
		{ "5aea1f37-fe95-462c-83de-fc3f266c473c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "09baf792-ff91-4537-a4d0-87a1a5739aad"}, { new NonTerminator(NonTerminatorType.ParameterList), "ddce692c-6fe7-4e16-a66e-55c4367a8210"}, { new NonTerminator(NonTerminatorType.Parameter), "51ea5cc9-4def-4153-a579-67d58494d8ad"}, { new NonTerminator(NonTerminatorType.VarParameter), "0378960f-af7e-418d-acb8-28aade479117"}, { new NonTerminator(NonTerminatorType.ValueParameter), "930bfe26-2081-464f-9231-cc2c2f2bbdd9"}, { new Terminator(KeywordType.Var), "7fd499b5-c4cf-4680-b56d-1a66d7d3eb8e"}, { Terminator.IdentifierTerminator, "2fd99590-86fb-4722-aa41-2f96e1a0a41c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5aea1f37-fe95-462c-83de-fc3f266c473c") },
		{ "4edbe980-4da2-48c9-905e-fbac861d9b82", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "644fe0f2-580d-4666-b83f-6d6aaed00933"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4edbe980-4da2-48c9-905e-fbac861d9b82") },
		{ "fed7ec14-0384-4a37-8e58-9fb94d6f0a01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "cc1bc2bc-2270-43e8-b4ed-eefdc63e28ea"}, { new NonTerminator(NonTerminatorType.ParameterList), "b3a6a2fc-4e39-420a-b355-12cc36fcf9db"}, { new NonTerminator(NonTerminatorType.Parameter), "51ea5cc9-4def-4153-a579-67d58494d8ad"}, { new NonTerminator(NonTerminatorType.VarParameter), "0378960f-af7e-418d-acb8-28aade479117"}, { new NonTerminator(NonTerminatorType.ValueParameter), "930bfe26-2081-464f-9231-cc2c2f2bbdd9"}, { new Terminator(KeywordType.Var), "7fd499b5-c4cf-4680-b56d-1a66d7d3eb8e"}, { Terminator.IdentifierTerminator, "2fd99590-86fb-4722-aa41-2f96e1a0a41c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fed7ec14-0384-4a37-8e58-9fb94d6f0a01") },
		{ "988e27b6-ffcf-4471-83cc-127f8860def6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "988e27b6-ffcf-4471-83cc-127f8860def6") },
		{ "a98b06dd-241a-4b21-aff6-736e4446d82d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "a98b06dd-241a-4b21-aff6-736e4446d82d") },
		{ "b01f7e65-1cb7-4bb6-9042-028fd4ffe5a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "41b075f6-a95c-4c5f-bb81-ff9fc4d10310"}, { Terminator.NumberTerminator, "07caf411-dce0-4707-a4d0-d40f3fe89cae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b01f7e65-1cb7-4bb6-9042-028fd4ffe5a5") },
		{ "88926413-9c08-4c39-9839-4bb6134c161c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "88926413-9c08-4c39-9839-4bb6134c161c") },
		{ "72fc3a18-0c7e-4792-a02d-8183d0a0fe91", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "873e33f9-efb1-4a25-a413-05b1b869736f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "72fc3a18-0c7e-4792-a02d-8183d0a0fe91") },
		{ "9a62e1cc-7cdb-4336-94b3-720faa11196b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "bbdbc044-bff2-4318-a607-ce316adf7cb4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9a62e1cc-7cdb-4336-94b3-720faa11196b") },
		{ "c692111c-6167-4810-9113-b4d5693cccba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "54fa0bfd-83b9-4c6e-84d5-2115296510e4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c692111c-6167-4810-9113-b4d5693cccba") },
		{ "677b577e-8689-41e0-a226-752b83f7d654", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, }, "677b577e-8689-41e0-a226-752b83f7d654") },
		{ "f3238ebc-6b60-45c8-88d0-ff7c697017b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "f3238ebc-6b60-45c8-88d0-ff7c697017b0") },
		{ "0c36ed43-e093-41b4-b3e0-ba81ac2cbc43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "98c61d88-ea3e-4827-9f7d-11621362d201"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "caaa2149-e3eb-4c06-adc7-f352f32a872d"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "0c36ed43-e093-41b4-b3e0-ba81ac2cbc43") },
		{ "01eb5023-f7b5-4dfb-bf95-bd2fc156f3c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "d5e8d0dc-d894-40b9-aaca-e1660241c41b"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "01eb5023-f7b5-4dfb-bf95-bd2fc156f3c5") },
		{ "c162558f-ea41-4672-a1b6-a1c65cb84cfa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "c162558f-ea41-4672-a1b6-a1c65cb84cfa") },
		{ "cef53325-1b8c-4f08-a412-8b565a6ab7de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cef53325-1b8c-4f08-a412-8b565a6ab7de") },
		{ "d6d4980d-2f84-4091-8473-8a415cca0e85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d6d4980d-2f84-4091-8473-8a415cca0e85") },
		{ "f15bddef-d6f6-406c-9808-3fc153276ec9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6df24978-46f4-49bf-996d-b96bd6f29ed4"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f15bddef-d6f6-406c-9808-3fc153276ec9") },
		{ "dea3950f-e624-454a-8580-81bd7c414142", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "a9f07cea-7761-4b32-9b86-6ccd2ceefaa5"}, { new NonTerminator(NonTerminatorType.IdVarPart), "794c07ab-ee69-4a83-90b6-fb46f739f874"}, { new Terminator(DelimiterType.LeftSquareBracket), "6bb0e158-2ad1-499d-acdd-a33aae90932d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "dea3950f-e624-454a-8580-81bd7c414142") },
		{ "0eba9572-881e-4bb6-8b12-7cfb903af0fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4773721b-535b-4dfc-a0fe-a59de1785242"}, { Terminator.NumberTerminator, "cef53325-1b8c-4f08-a412-8b565a6ab7de"}, { new NonTerminator(NonTerminatorType.Variable), "d6d4980d-2f84-4091-8473-8a415cca0e85"}, { new Terminator(DelimiterType.LeftParenthesis), "f15bddef-d6f6-406c-9808-3fc153276ec9"}, { Terminator.IdentifierTerminator, "dea3950f-e624-454a-8580-81bd7c414142"}, { new Terminator(KeywordType.Not), "0eba9572-881e-4bb6-8b12-7cfb903af0fc"}, { new Terminator(OperatorType.Minus), "1278eb6a-3f98-4429-a858-7a7d9401db13"}, { new Terminator(OperatorType.Plus), "95843f41-66a5-4a63-9431-0b0fa2bfd9f2"}, { new Terminator(KeywordType.True), "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae"}, { new Terminator(KeywordType.False), "3a664267-532b-4bec-9dd8-37acd5ca4367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0eba9572-881e-4bb6-8b12-7cfb903af0fc") },
		{ "1278eb6a-3f98-4429-a858-7a7d9401db13", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f2b2a71c-b801-40ec-bf59-79ae9f51e6a0"}, { Terminator.NumberTerminator, "cef53325-1b8c-4f08-a412-8b565a6ab7de"}, { new NonTerminator(NonTerminatorType.Variable), "d6d4980d-2f84-4091-8473-8a415cca0e85"}, { new Terminator(DelimiterType.LeftParenthesis), "f15bddef-d6f6-406c-9808-3fc153276ec9"}, { Terminator.IdentifierTerminator, "dea3950f-e624-454a-8580-81bd7c414142"}, { new Terminator(KeywordType.Not), "0eba9572-881e-4bb6-8b12-7cfb903af0fc"}, { new Terminator(OperatorType.Minus), "1278eb6a-3f98-4429-a858-7a7d9401db13"}, { new Terminator(OperatorType.Plus), "95843f41-66a5-4a63-9431-0b0fa2bfd9f2"}, { new Terminator(KeywordType.True), "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae"}, { new Terminator(KeywordType.False), "3a664267-532b-4bec-9dd8-37acd5ca4367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1278eb6a-3f98-4429-a858-7a7d9401db13") },
		{ "95843f41-66a5-4a63-9431-0b0fa2bfd9f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "902c56d4-07b6-403e-bc87-a9371ae7c2ee"}, { Terminator.NumberTerminator, "cef53325-1b8c-4f08-a412-8b565a6ab7de"}, { new NonTerminator(NonTerminatorType.Variable), "d6d4980d-2f84-4091-8473-8a415cca0e85"}, { new Terminator(DelimiterType.LeftParenthesis), "f15bddef-d6f6-406c-9808-3fc153276ec9"}, { Terminator.IdentifierTerminator, "dea3950f-e624-454a-8580-81bd7c414142"}, { new Terminator(KeywordType.Not), "0eba9572-881e-4bb6-8b12-7cfb903af0fc"}, { new Terminator(OperatorType.Minus), "1278eb6a-3f98-4429-a858-7a7d9401db13"}, { new Terminator(OperatorType.Plus), "95843f41-66a5-4a63-9431-0b0fa2bfd9f2"}, { new Terminator(KeywordType.True), "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae"}, { new Terminator(KeywordType.False), "3a664267-532b-4bec-9dd8-37acd5ca4367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "95843f41-66a5-4a63-9431-0b0fa2bfd9f2") },
		{ "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae") },
		{ "3a664267-532b-4bec-9dd8-37acd5ca4367", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3a664267-532b-4bec-9dd8-37acd5ca4367") },
		{ "a1cc474f-4114-4fcf-8761-fa3bf7af1111", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "ac17d13e-a4a8-4315-96db-960d4dd7725b"}, { new NonTerminator(NonTerminatorType.Variable), "02da0c34-6cbd-42b8-a890-cdd4e7f62670"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "10b72ef2-2d50-4e2e-ad5e-9777f87581fc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "4abb7227-aa91-422b-8c07-5cefec0f972c"}, { new Terminator(KeywordType.If), "c9b058f7-972c-4a63-a2f6-4dd0fbda04e4"}, { new Terminator(KeywordType.For), "05a3f0e8-28df-4577-943e-299acda150d7"}, { Terminator.IdentifierTerminator, "c57c40e0-158e-40bf-ba1d-793d854fc4d2"}, { new Terminator(KeywordType.Begin), "6101ca1c-a6a9-41b4-a112-c00138796288"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "a1cc474f-4114-4fcf-8761-fa3bf7af1111") },
		{ "b1c91fcc-f7ee-4417-a4df-380b22b6c315", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "70328748-bf1a-421a-81c1-8b32e0ea0862"}, { new NonTerminator(NonTerminatorType.Term), "0e22d8fb-b57f-4627-87e1-844ac79ff101"}, { new NonTerminator(NonTerminatorType.Factor), "06f68d47-af3e-48cb-ad03-28be91344b7c"}, { Terminator.NumberTerminator, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74"}, { new NonTerminator(NonTerminatorType.Variable), "ee1f29a9-06b3-45fd-824c-a319a2988aa9"}, { new Terminator(DelimiterType.LeftParenthesis), "8abb7440-8069-44c2-ab58-94c4486624d2"}, { Terminator.IdentifierTerminator, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a"}, { new Terminator(KeywordType.Not), "511d04d4-c513-4863-8a48-ab5258a8c6a5"}, { new Terminator(OperatorType.Minus), "71d87b84-da4f-4b6e-b472-04e6910f77d3"}, { new Terminator(OperatorType.Plus), "eaa10f35-d92c-4913-9d3e-66152dc1592e"}, { new Terminator(KeywordType.True), "9067f1a1-acd5-4ef6-b59a-006235d313ba"}, { new Terminator(KeywordType.False), "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b1c91fcc-f7ee-4417-a4df-380b22b6c315") },
		{ "00f9e6b4-d835-4d58-b32a-08c692f2bfd1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "00f9e6b4-d835-4d58-b32a-08c692f2bfd1") },
		{ "53511e62-9a4d-468d-89f2-6dcc50713b8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "53511e62-9a4d-468d-89f2-6dcc50713b8d") },
		{ "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2") },
		{ "e81e43c0-1fea-4c0d-84e7-9a11a11023a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "e81e43c0-1fea-4c0d-84e7-9a11a11023a6") },
		{ "690c772a-40e6-4ba1-8c96-e78b459a3f8c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "690c772a-40e6-4ba1-8c96-e78b459a3f8c") },
		{ "50760e65-8acd-473f-928e-e9ecce50bc7e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "50760e65-8acd-473f-928e-e9ecce50bc7e") },
		{ "f4b20ae9-3e21-45ee-a940-4dcab928f9e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "d0156878-e7fb-4b61-beee-069eeaff65dc"}, { new NonTerminator(NonTerminatorType.Factor), "1937a590-6e09-4012-a71e-88f3f4dd738a"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f4b20ae9-3e21-45ee-a940-4dcab928f9e4") },
		{ "ec492a38-9608-4c47-b622-384820e1e7b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "ec492a38-9608-4c47-b622-384820e1e7b8") },
		{ "8aee6cb0-9499-46fa-bd1d-354d7710722d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "8aee6cb0-9499-46fa-bd1d-354d7710722d") },
		{ "f2449077-15a8-451d-8e69-f734c89a16ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "f2449077-15a8-451d-8e69-f734c89a16ba") },
		{ "ddaf6ecb-88df-4574-9cc6-fc819989259f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fd01b345-fc7a-4165-8a10-24c465ded82b"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ddaf6ecb-88df-4574-9cc6-fc819989259f") },
		{ "b478e88c-8ab6-4b13-a8bb-59838af033b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "b478e88c-8ab6-4b13-a8bb-59838af033b0") },
		{ "eade5b89-478e-431f-8b2c-a336366766ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "eade5b89-478e-431f-8b2c-a336366766ef") },
		{ "cac88815-70df-444a-a247-aa4328ff8b44", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "cac88815-70df-444a-a247-aa4328ff8b44") },
		{ "fe820fb4-bb24-498c-bbbb-a204e3bf646e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "fe820fb4-bb24-498c-bbbb-a204e3bf646e") },
		{ "c143f543-403a-456f-8173-33073fd62586", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "c143f543-403a-456f-8173-33073fd62586") },
		{ "b288449a-1a6e-4a63-af68-892fd028a360", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2b58c414-795a-4bd2-a236-df92e23a4f45"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b288449a-1a6e-4a63-af68-892fd028a360") },
		{ "ab682e97-3b78-4d58-a255-118650e13d96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "751e9a07-dd61-4a9b-98f0-05eb191499f1"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "667a8bb6-99de-40ef-8946-6c36412df4d9"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "ab682e97-3b78-4d58-a255-118650e13d96") },
		{ "611cfdfb-5856-4399-80d2-095291ebb774", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "6511841e-e643-4c22-9273-ce5966dc3912"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "611cfdfb-5856-4399-80d2-095291ebb774") },
		{ "95b52b9a-255c-47d1-b6b0-636ef908452e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "95b52b9a-255c-47d1-b6b0-636ef908452e") },
		{ "3d50648a-761a-4c45-add9-026f9631c92d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3d50648a-761a-4c45-add9-026f9631c92d") },
		{ "50eb06a8-d7a4-4d5c-9a01-8841b61df175", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "50eb06a8-d7a4-4d5c-9a01-8841b61df175") },
		{ "7929fa15-03d5-4eb6-84c2-eba74372cf0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6d1e4ca0-faeb-4543-9fcb-57e047f64fb1"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7929fa15-03d5-4eb6-84c2-eba74372cf0f") },
		{ "e8e3cd92-00c6-4331-b572-7d900e34d3fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "fddf6099-ee08-40ec-926a-16684241ce48"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d02712ea-5f52-412c-b2c6-a0c65d7b4ad3"}, { new Terminator(DelimiterType.LeftSquareBracket), "9136ec11-c2d0-4cd4-903b-47ce7d0acd07"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e8e3cd92-00c6-4331-b572-7d900e34d3fe") },
		{ "e77e5574-028a-4eb0-9cfd-23b040ac8e58", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "81b8e2c2-7233-4801-8899-1722b8505488"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e77e5574-028a-4eb0-9cfd-23b040ac8e58") },
		{ "322c45a4-8a7b-4f0c-b684-3c705ea1166f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "53577af0-a34b-49b7-a069-1f75bb6d1db2"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "322c45a4-8a7b-4f0c-b684-3c705ea1166f") },
		{ "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "aaf70ad5-d1a1-4886-a2bc-1e06339d04af"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc") },
		{ "8a5210d5-c688-431b-a671-85ba652c83b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "8a5210d5-c688-431b-a671-85ba652c83b5") },
		{ "0a01f9a5-406b-4299-92c7-88a7d2ece198", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0a01f9a5-406b-4299-92c7-88a7d2ece198") },
		{ "471f7efa-f893-45bf-8d96-544706e12bbc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9eb2842a-1572-4918-9844-2feaa5cc39f6"}, { new NonTerminator(NonTerminatorType.ExpressionList), "3dbfa90a-5d7c-4ab1-b22d-280ec2c29df7"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "471f7efa-f893-45bf-8d96-544706e12bbc") },
		{ "8e80f472-0127-4d83-ad8f-980de7fee70b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "8e80f472-0127-4d83-ad8f-980de7fee70b") },
		{ "4b02d152-a939-4aa2-acf2-ed72f81900a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "11a20fc9-fb59-4428-b0c1-f6efa412ba5f"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4b02d152-a939-4aa2-acf2-ed72f81900a5") },
		{ "2e7ad752-6c24-4e00-947b-2004ddd97446", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2e7ad752-6c24-4e00-947b-2004ddd97446") },
		{ "5fd76afe-6611-4938-862b-dd41afcd78d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "5fd76afe-6611-4938-862b-dd41afcd78d1") },
		{ "e6875378-4952-42ed-9946-9b45db9de896", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e6875378-4952-42ed-9946-9b45db9de896") },
		{ "2972b2e7-55b0-47ef-973d-eb51b98ff12f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c9c73c71-8d29-49bc-9678-3f330995cb91"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "fdd4eff1-f432-44c3-95cc-04854e0aa40c"}, { new NonTerminator(NonTerminatorType.Term), "0d02e1d4-d78b-4019-a7bd-6b8c6fe08e37"}, { new NonTerminator(NonTerminatorType.Factor), "26bcc0d0-0146-41ca-a19e-298fccaefbd3"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2972b2e7-55b0-47ef-973d-eb51b98ff12f") },
		{ "615ce50c-31ca-499d-b10c-26fa4b579b09", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "e543b165-d65b-4169-ba4d-e1f92a6d69b2"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "615ce50c-31ca-499d-b10c-26fa4b579b09") },
		{ "448c905c-5cc0-4646-85aa-112043d970e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "448c905c-5cc0-4646-85aa-112043d970e9") },
		{ "b190a4d5-1ed9-4785-a10c-7dcab76321a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "1af699b8-c50a-497a-850f-c7be2de94269"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "7e62569f-9cd9-4b5d-9906-60c4afb167f4"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "b190a4d5-1ed9-4785-a10c-7dcab76321a6") },
		{ "fe52cbda-3e9c-491e-b578-970cd3929ef0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ae62e89b-0b5f-4c6f-be21-981e9671786e"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fe52cbda-3e9c-491e-b578-970cd3929ef0") },
		{ "2dc1ec98-133e-4375-8295-8aa394bf8492", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "2dc1ec98-133e-4375-8295-8aa394bf8492") },
		{ "fd7f3201-4476-4775-b6f0-65eeaa91478e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fd7f3201-4476-4775-b6f0-65eeaa91478e") },
		{ "967d95dd-847f-420a-b578-41cebd118238", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "967d95dd-847f-420a-b578-41cebd118238") },
		{ "189a808f-b459-4a2d-b40a-9a3ee1d80ec4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "57f5b409-20e9-4cc4-9421-f4b03d18b459"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "189a808f-b459-4a2d-b40a-9a3ee1d80ec4") },
		{ "afba1d4b-61d5-499f-8c1d-b0ad46898948", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "0a477d08-fed1-4fcb-badf-466388e868f9"}, { new NonTerminator(NonTerminatorType.IdVarPart), "7530f538-3ae0-42ba-b543-de55bc02da6c"}, { new Terminator(DelimiterType.LeftSquareBracket), "09d0ed01-10ef-4dd0-839e-0f6cff8f68e3"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "afba1d4b-61d5-499f-8c1d-b0ad46898948") },
		{ "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7f7a48dc-9adb-4beb-9218-c7fda97823ef"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0") },
		{ "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "882bfa6a-fb20-4088-83cd-1a03204deabf"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50") },
		{ "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "38fa7d9e-c576-4f4c-b1b8-2609aa3e74e8"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef") },
		{ "9ca05cd7-eb2e-4720-a786-80e30d2d2331", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "9ca05cd7-eb2e-4720-a786-80e30d2d2331") },
		{ "2cd9c8ee-2538-4d77-9adf-f3d3172025cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2cd9c8ee-2538-4d77-9adf-f3d3172025cd") },
		{ "31061387-a3d5-4c4b-975f-b12acce18ed3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "31061387-a3d5-4c4b-975f-b12acce18ed3") },
		{ "51a25622-d03c-45eb-98bc-dfe8bb6fa7dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0b5bdcae-f6c9-4808-ab65-11517f66c6fb"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "51a25622-d03c-45eb-98bc-dfe8bb6fa7dc") },
		{ "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb") },
		{ "76f0d1ad-c34a-466d-8253-559dda913cad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "e678709b-de6d-4b47-ac05-358101c1743a"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "f83f4997-f455-4025-bd96-cc799435f14c"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "76f0d1ad-c34a-466d-8253-559dda913cad") },
		{ "e43ff673-e432-4ca6-a673-1610a617a04c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "35226c26-fd83-42d7-853c-d29d48133038"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e43ff673-e432-4ca6-a673-1610a617a04c") },
		{ "c533182a-88ca-4990-ba69-8cad415533c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "c533182a-88ca-4990-ba69-8cad415533c8") },
		{ "5dff7871-fd3e-4474-a5d1-6999b6b73dcd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd") },
		{ "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b") },
		{ "8ad03c91-d8a2-43f3-a9b9-785c90530517", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "1f6c5f36-1d5c-4d99-8a21-c158ae35830d"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8ad03c91-d8a2-43f3-a9b9-785c90530517") },
		{ "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "728fc123-57e4-4af8-ab75-fa670f046f9d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "2ad07047-be21-455c-b607-4e2c1672b85a"}, { new Terminator(DelimiterType.LeftSquareBracket), "dd850ba2-1a8c-4a4b-9a95-d28a95d0824b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50") },
		{ "713922ed-8e68-4d95-83db-f8544e1587eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4f66c1d6-c004-488e-a441-6e57c5deae80"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "713922ed-8e68-4d95-83db-f8544e1587eb") },
		{ "cb641776-84bd-40c8-91e7-ed34b5560acf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "5814a2e7-7245-4f01-a511-99ab0fc4d7d7"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cb641776-84bd-40c8-91e7-ed34b5560acf") },
		{ "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "69971c62-2a0c-4604-988e-2b6ef4e59be8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a") },
		{ "6db98640-0b70-4753-92a5-b25fad75bb99", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6db98640-0b70-4753-92a5-b25fad75bb99") },
		{ "85c68c56-8c61-4219-969e-b531eba59f49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "85c68c56-8c61-4219-969e-b531eba59f49") },
		{ "63bfeb19-e3a8-4cf9-8066-98ec790cf087", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "63bfeb19-e3a8-4cf9-8066-98ec790cf087") },
		{ "098159a2-c899-4672-99d1-2be33e406320", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "de3fb9f6-3a55-42a3-8e3e-bf03c453b822"}, { new Terminator(KeywordType.Begin), "9b21f326-b9a4-4cef-93ad-4ac795053f7a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "098159a2-c899-4672-99d1-2be33e406320") },
		{ "a9df0d14-abca-4771-835f-771caf8e2718", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "25483c72-4b02-41b4-8bc5-b1f864a15187"}, { Terminator.IdentifierTerminator, "68100ff0-d251-41c2-a498-ad3e6bf36631"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a9df0d14-abca-4771-835f-771caf8e2718") },
		{ "f637590f-1e50-4672-864a-0e634b2edcfa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "0e921e41-05f1-4dce-8ffb-85338e8621c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f637590f-1e50-4672-864a-0e634b2edcfa") },
		{ "09baf792-ff91-4537-a4d0-87a1a5739aad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "09baf792-ff91-4537-a4d0-87a1a5739aad") },
		{ "ddce692c-6fe7-4e16-a66e-55c4367a8210", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ca947fdb-cde8-4752-97a0-d1a13c5fd744"}, { new Terminator(DelimiterType.Semicolon), "40b5d737-b43a-42ac-8238-d4f197167db8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ddce692c-6fe7-4e16-a66e-55c4367a8210") },
		{ "51ea5cc9-4def-4153-a579-67d58494d8ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, }, "51ea5cc9-4def-4153-a579-67d58494d8ad") },
		{ "0378960f-af7e-418d-acb8-28aade479117", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "0378960f-af7e-418d-acb8-28aade479117") },
		{ "930bfe26-2081-464f-9231-cc2c2f2bbdd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "930bfe26-2081-464f-9231-cc2c2f2bbdd9") },
		{ "7fd499b5-c4cf-4680-b56d-1a66d7d3eb8e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ValueParameter), "b2563e77-2a1a-4b0c-ae97-20e0694e707c"}, { Terminator.IdentifierTerminator, "2fd99590-86fb-4722-aa41-2f96e1a0a41c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7fd499b5-c4cf-4680-b56d-1a66d7d3eb8e") },
		{ "2fd99590-86fb-4722-aa41-2f96e1a0a41c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "6680defb-1123-4c05-9626-13adfcbb646d"}, { new Terminator(DelimiterType.Comma), "1a933aae-c301-4ce5-9e03-9e7ba76073fe"}, { new Terminator(DelimiterType.Colon), "1823fe2a-60d4-4212-8fd4-bf96ffe71d19"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2fd99590-86fb-4722-aa41-2f96e1a0a41c") },
		{ "644fe0f2-580d-4666-b83f-6d6aaed00933", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "fc9b09a6-f614-486f-a23d-797719a0912b"}, { new Terminator(KeywordType.Integer), "06bbef92-0d08-4369-a6e1-07308f373494"}, { new Terminator(KeywordType.Real), "141114e5-17fa-4ef5-8833-b7cf274019dc"}, { new Terminator(KeywordType.Boolean), "cfb5a6ab-7bb3-4d92-afae-7235a569a96f"}, { new Terminator(KeywordType.Character), "9ff8a64c-96bc-45f6-8b24-56e3e3dffd24"},}, new Dictionary<Terminator, ReduceInformation>{ }, "644fe0f2-580d-4666-b83f-6d6aaed00933") },
		{ "cc1bc2bc-2270-43e8-b4ed-eefdc63e28ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "cc1bc2bc-2270-43e8-b4ed-eefdc63e28ea") },
		{ "b3a6a2fc-4e39-420a-b355-12cc36fcf9db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "854d7dc0-d01d-4135-8244-b43a6d9e9902"}, { new Terminator(DelimiterType.Semicolon), "40b5d737-b43a-42ac-8238-d4f197167db8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b3a6a2fc-4e39-420a-b355-12cc36fcf9db") },
		{ "41b075f6-a95c-4c5f-bb81-ff9fc4d10310", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "49c7d99b-1dd9-403f-8b39-1999c20f67dd"}, { new Terminator(DelimiterType.Comma), "9a62e1cc-7cdb-4336-94b3-720faa11196b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "41b075f6-a95c-4c5f-bb81-ff9fc4d10310") },
		{ "873e33f9-efb1-4a25-a413-05b1b869736f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "eeb042f2-1f51-4b85-a221-dd158bb21b06"}, { new Terminator(KeywordType.Integer), "4e4cfe2e-0f81-4126-9641-569669e59526"}, { new Terminator(KeywordType.Real), "478d5073-98fb-4782-838f-b791bcad8110"}, { new Terminator(KeywordType.Boolean), "2b873f3e-64fa-493b-9e04-a4f7cce82410"}, { new Terminator(KeywordType.Character), "308ca356-663b-428e-a883-c7a002b346dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "873e33f9-efb1-4a25-a413-05b1b869736f") },
		{ "bbdbc044-bff2-4318-a607-ce316adf7cb4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "7901b2aa-d796-49ca-ac12-9832bba31f76"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bbdbc044-bff2-4318-a607-ce316adf7cb4") },
		{ "54fa0bfd-83b9-4c6e-84d5-2115296510e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, }, "54fa0bfd-83b9-4c6e-84d5-2115296510e4") },
		{ "98c61d88-ea3e-4827-9f7d-11621362d201", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "ba617890-7ac5-448f-a25d-44d047183652"}, { new NonTerminator(NonTerminatorType.Term), "0569958f-44ec-4e2e-ae51-95102a293e53"}, { new NonTerminator(NonTerminatorType.Factor), "2e01c973-a9fc-40ce-8dc1-df110cdb0e61"}, { Terminator.NumberTerminator, "118d5952-60b4-42d8-9773-dd7522338535"}, { new NonTerminator(NonTerminatorType.Variable), "9451c76a-01e8-4dbf-8a16-f7db028b7adb"}, { new Terminator(DelimiterType.LeftParenthesis), "df9b2921-601d-43d6-8168-546a4d5597fc"}, { Terminator.IdentifierTerminator, "5ddf6fe6-d704-41a4-b552-e817ef6f729f"}, { new Terminator(KeywordType.Not), "5332de79-32c9-4e62-a7de-15ee234934bf"}, { new Terminator(OperatorType.Minus), "39df86b4-3963-4ba3-a04e-047cd10c6a62"}, { new Terminator(OperatorType.Plus), "21d97a2b-d0ff-4cf4-b483-728ef20a436f"}, { new Terminator(KeywordType.True), "9dc09043-74f0-4711-8c01-db46e7a63643"}, { new Terminator(KeywordType.False), "7d2d2ab3-6074-49b5-a491-1114292132cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "98c61d88-ea3e-4827-9f7d-11621362d201") },
		{ "caaa2149-e3eb-4c06-adc7-f352f32a872d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "4156b65f-321a-4e96-ae31-0330777bbc33"}, { new NonTerminator(NonTerminatorType.Factor), "c162558f-ea41-4672-a1b6-a1c65cb84cfa"}, { Terminator.NumberTerminator, "cef53325-1b8c-4f08-a412-8b565a6ab7de"}, { new NonTerminator(NonTerminatorType.Variable), "d6d4980d-2f84-4091-8473-8a415cca0e85"}, { new Terminator(DelimiterType.LeftParenthesis), "f15bddef-d6f6-406c-9808-3fc153276ec9"}, { Terminator.IdentifierTerminator, "dea3950f-e624-454a-8580-81bd7c414142"}, { new Terminator(KeywordType.Not), "0eba9572-881e-4bb6-8b12-7cfb903af0fc"}, { new Terminator(OperatorType.Minus), "1278eb6a-3f98-4429-a858-7a7d9401db13"}, { new Terminator(OperatorType.Plus), "95843f41-66a5-4a63-9431-0b0fa2bfd9f2"}, { new Terminator(KeywordType.True), "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae"}, { new Terminator(KeywordType.False), "3a664267-532b-4bec-9dd8-37acd5ca4367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "caaa2149-e3eb-4c06-adc7-f352f32a872d") },
		{ "d5e8d0dc-d894-40b9-aaca-e1660241c41b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c1854fed-3d7f-4668-811a-4b82176d4965"}, { Terminator.NumberTerminator, "cef53325-1b8c-4f08-a412-8b565a6ab7de"}, { new NonTerminator(NonTerminatorType.Variable), "d6d4980d-2f84-4091-8473-8a415cca0e85"}, { new Terminator(DelimiterType.LeftParenthesis), "f15bddef-d6f6-406c-9808-3fc153276ec9"}, { Terminator.IdentifierTerminator, "dea3950f-e624-454a-8580-81bd7c414142"}, { new Terminator(KeywordType.Not), "0eba9572-881e-4bb6-8b12-7cfb903af0fc"}, { new Terminator(OperatorType.Minus), "1278eb6a-3f98-4429-a858-7a7d9401db13"}, { new Terminator(OperatorType.Plus), "95843f41-66a5-4a63-9431-0b0fa2bfd9f2"}, { new Terminator(KeywordType.True), "1bf0e5e7-1176-4cbe-a902-07a060f4f4ae"}, { new Terminator(KeywordType.False), "3a664267-532b-4bec-9dd8-37acd5ca4367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d5e8d0dc-d894-40b9-aaca-e1660241c41b") },
		{ "6df24978-46f4-49bf-996d-b96bd6f29ed4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7162215e-0834-43ae-94f2-801c43cb7dfd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6df24978-46f4-49bf-996d-b96bd6f29ed4") },
		{ "a9f07cea-7761-4b32-9b86-6ccd2ceefaa5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c0ce220a-c936-4e08-9a81-08b7147ba8b9"}, { new NonTerminator(NonTerminatorType.ExpressionList), "d82c5fc2-22c2-41df-8317-d8a6ac3eba10"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a9f07cea-7761-4b32-9b86-6ccd2ceefaa5") },
		{ "794c07ab-ee69-4a83-90b6-fb46f739f874", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "794c07ab-ee69-4a83-90b6-fb46f739f874") },
		{ "6bb0e158-2ad1-499d-acdd-a33aae90932d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "aba957b3-15ec-4dbb-976f-6e8e5f2520b4"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6bb0e158-2ad1-499d-acdd-a33aae90932d") },
		{ "4773721b-535b-4dfc-a0fe-a59de1785242", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "4773721b-535b-4dfc-a0fe-a59de1785242") },
		{ "f2b2a71c-b801-40ec-bf59-79ae9f51e6a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f2b2a71c-b801-40ec-bf59-79ae9f51e6a0") },
		{ "902c56d4-07b6-403e-bc87-a9371ae7c2ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "902c56d4-07b6-403e-bc87-a9371ae7c2ee") },
		{ "ac17d13e-a4a8-4315-96db-960d4dd7725b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "90363a8f-7f4a-401f-a5ac-878a034e4453"}, { new Terminator(KeywordType.Else), "469dd03b-8892-47dc-b743-c98e497d66fa"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "ac17d13e-a4a8-4315-96db-960d4dd7725b") },
		{ "02da0c34-6cbd-42b8-a890-cdd4e7f62670", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "14d2a496-27ff-4ba0-b9bf-e5a503cebe29"},}, new Dictionary<Terminator, ReduceInformation>{ }, "02da0c34-6cbd-42b8-a890-cdd4e7f62670") },
		{ "10b72ef2-2d50-4e2e-ad5e-9777f87581fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "10b72ef2-2d50-4e2e-ad5e-9777f87581fc") },
		{ "4abb7227-aa91-422b-8c07-5cefec0f972c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "4abb7227-aa91-422b-8c07-5cefec0f972c") },
		{ "c9b058f7-972c-4a63-a2f6-4dd0fbda04e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "b45eb976-9df0-4f3e-9dbf-bb9dd8fd3c23"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b5011c14-e23b-4357-8e8f-09be3bdf7723"}, { new NonTerminator(NonTerminatorType.Term), "03a31cb3-b2c6-40d1-94ed-7254abf45605"}, { new NonTerminator(NonTerminatorType.Factor), "1937a590-6e09-4012-a71e-88f3f4dd738a"}, { Terminator.NumberTerminator, "5b1f2840-09aa-443e-bea9-3a12822d02c1"}, { new NonTerminator(NonTerminatorType.Variable), "d0c4e4b7-e350-48d1-aa20-0d76f65055d7"}, { new Terminator(DelimiterType.LeftParenthesis), "739d8a67-203c-4a82-83f2-6cef03928db7"}, { Terminator.IdentifierTerminator, "07c9c6be-521a-40a7-ac31-85030d9e477e"}, { new Terminator(KeywordType.Not), "c8ee831d-e142-46cb-8bb2-24b96f753c90"}, { new Terminator(OperatorType.Minus), "1c4c4fb4-28de-4325-a7de-79745c718e33"}, { new Terminator(OperatorType.Plus), "cfb994ea-e037-45a7-aca3-e2682ed87464"}, { new Terminator(KeywordType.True), "e61a0272-7657-4704-b8d2-1d348ebe0464"}, { new Terminator(KeywordType.False), "0c4a0f13-abd3-419a-bb79-6e8b9c78d1d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c9b058f7-972c-4a63-a2f6-4dd0fbda04e4") },
		{ "05a3f0e8-28df-4577-943e-299acda150d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "9afb6c66-5cc3-4ddc-8049-27381dafb426"},}, new Dictionary<Terminator, ReduceInformation>{ }, "05a3f0e8-28df-4577-943e-299acda150d7") },
		{ "c57c40e0-158e-40bf-ba1d-793d854fc4d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdVarPart), "0df18bdb-69d7-4476-b9a3-1518a64f9c5d"}, { new Terminator(DelimiterType.LeftSquareBracket), "4b810105-5afb-4423-8349-fb69ac8898eb"}, { new Terminator(DelimiterType.LeftParenthesis), "77f0e0d9-ee75-4e79-a257-4404d92e8bd5"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "c57c40e0-158e-40bf-ba1d-793d854fc4d2") },
		{ "6101ca1c-a6a9-41b4-a112-c00138796288", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "ce6da976-60ba-4844-baab-0e3e1b6b397e"}, { new NonTerminator(NonTerminatorType.Statement), "f7d7f42c-bf10-43d4-8e97-409785b892ff"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "6101ca1c-a6a9-41b4-a112-c00138796288") },
		{ "70328748-bf1a-421a-81c1-8b32e0ea0862", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "bc837e3b-b5f7-4c23-90cc-ef990a5a98a7"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "70328748-bf1a-421a-81c1-8b32e0ea0862") },
		{ "0e22d8fb-b57f-4627-87e1-844ac79ff101", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "50c1a67d-c4d7-446f-a505-a5dfb43f5988"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0e22d8fb-b57f-4627-87e1-844ac79ff101") },
		{ "06f68d47-af3e-48cb-ad03-28be91344b7c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "06f68d47-af3e-48cb-ad03-28be91344b7c") },
		{ "5c064a71-4cd3-434b-b60c-ac24fe1a2c74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74") },
		{ "ee1f29a9-06b3-45fd-824c-a319a2988aa9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ee1f29a9-06b3-45fd-824c-a319a2988aa9") },
		{ "8abb7440-8069-44c2-ab58-94c4486624d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8b060542-f56f-4a7f-ac56-5c1e5ed27462"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8abb7440-8069-44c2-ab58-94c4486624d2") },
		{ "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "b66337d9-9864-4821-b361-93996d20046d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "b5edb048-e007-46d4-be13-b41b1b9bb78e"}, { new Terminator(DelimiterType.LeftSquareBracket), "953656c4-d3dd-4480-9225-b70bd0ab69f0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a") },
		{ "511d04d4-c513-4863-8a48-ab5258a8c6a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a1415324-197b-43af-959c-6f94ec59d928"}, { Terminator.NumberTerminator, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74"}, { new NonTerminator(NonTerminatorType.Variable), "ee1f29a9-06b3-45fd-824c-a319a2988aa9"}, { new Terminator(DelimiterType.LeftParenthesis), "8abb7440-8069-44c2-ab58-94c4486624d2"}, { Terminator.IdentifierTerminator, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a"}, { new Terminator(KeywordType.Not), "511d04d4-c513-4863-8a48-ab5258a8c6a5"}, { new Terminator(OperatorType.Minus), "71d87b84-da4f-4b6e-b472-04e6910f77d3"}, { new Terminator(OperatorType.Plus), "eaa10f35-d92c-4913-9d3e-66152dc1592e"}, { new Terminator(KeywordType.True), "9067f1a1-acd5-4ef6-b59a-006235d313ba"}, { new Terminator(KeywordType.False), "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511"},}, new Dictionary<Terminator, ReduceInformation>{ }, "511d04d4-c513-4863-8a48-ab5258a8c6a5") },
		{ "71d87b84-da4f-4b6e-b472-04e6910f77d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "40d75275-c36d-49a3-876e-dd1d26db7c5e"}, { Terminator.NumberTerminator, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74"}, { new NonTerminator(NonTerminatorType.Variable), "ee1f29a9-06b3-45fd-824c-a319a2988aa9"}, { new Terminator(DelimiterType.LeftParenthesis), "8abb7440-8069-44c2-ab58-94c4486624d2"}, { Terminator.IdentifierTerminator, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a"}, { new Terminator(KeywordType.Not), "511d04d4-c513-4863-8a48-ab5258a8c6a5"}, { new Terminator(OperatorType.Minus), "71d87b84-da4f-4b6e-b472-04e6910f77d3"}, { new Terminator(OperatorType.Plus), "eaa10f35-d92c-4913-9d3e-66152dc1592e"}, { new Terminator(KeywordType.True), "9067f1a1-acd5-4ef6-b59a-006235d313ba"}, { new Terminator(KeywordType.False), "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511"},}, new Dictionary<Terminator, ReduceInformation>{ }, "71d87b84-da4f-4b6e-b472-04e6910f77d3") },
		{ "eaa10f35-d92c-4913-9d3e-66152dc1592e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7b0f787a-85bb-4963-af18-6a9619df90ae"}, { Terminator.NumberTerminator, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74"}, { new NonTerminator(NonTerminatorType.Variable), "ee1f29a9-06b3-45fd-824c-a319a2988aa9"}, { new Terminator(DelimiterType.LeftParenthesis), "8abb7440-8069-44c2-ab58-94c4486624d2"}, { Terminator.IdentifierTerminator, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a"}, { new Terminator(KeywordType.Not), "511d04d4-c513-4863-8a48-ab5258a8c6a5"}, { new Terminator(OperatorType.Minus), "71d87b84-da4f-4b6e-b472-04e6910f77d3"}, { new Terminator(OperatorType.Plus), "eaa10f35-d92c-4913-9d3e-66152dc1592e"}, { new Terminator(KeywordType.True), "9067f1a1-acd5-4ef6-b59a-006235d313ba"}, { new Terminator(KeywordType.False), "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eaa10f35-d92c-4913-9d3e-66152dc1592e") },
		{ "9067f1a1-acd5-4ef6-b59a-006235d313ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "9067f1a1-acd5-4ef6-b59a-006235d313ba") },
		{ "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511") },
		{ "d0156878-e7fb-4b61-beee-069eeaff65dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ddaf6ecb-88df-4574-9cc6-fc819989259f"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d0156878-e7fb-4b61-beee-069eeaff65dc") },
		{ "fd01b345-fc7a-4165-8a10-24c465ded82b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "fd01b345-fc7a-4165-8a10-24c465ded82b") },
		{ "2b58c414-795a-4bd2-a236-df92e23a4f45", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "2b58c414-795a-4bd2-a236-df92e23a4f45") },
		{ "751e9a07-dd61-4a9b-98f0-05eb191499f1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "9535c43f-c365-4de8-9c84-4239752f1bb0"}, { new NonTerminator(NonTerminatorType.Term), "fe0e57e8-6cb8-4f07-a75a-690c2cfb2a0f"}, { new NonTerminator(NonTerminatorType.Factor), "caed1736-e1a9-4a08-a996-38057dc9ef38"}, { Terminator.NumberTerminator, "2d7a0ca8-d47b-47af-96b1-79a244a8162b"}, { new NonTerminator(NonTerminatorType.Variable), "0203b66a-97cf-49e4-b4a5-44a9240daee1"}, { new Terminator(DelimiterType.LeftParenthesis), "72ae4a09-129b-4e98-a17f-98fd3df8f433"}, { Terminator.IdentifierTerminator, "b3d5f8de-1810-4638-a42c-ed6b741212fe"}, { new Terminator(KeywordType.Not), "baa9ab37-d944-4c90-83a1-2d243c2b3aec"}, { new Terminator(OperatorType.Minus), "075b0a32-180c-440c-9b7b-6cb315a3b4b8"}, { new Terminator(OperatorType.Plus), "7236e6ff-062d-4114-a692-c106386308ab"}, { new Terminator(KeywordType.True), "bf93e069-f9ae-4e60-9b14-743260b945e8"}, { new Terminator(KeywordType.False), "1cf66947-0ecc-4413-8902-7bfcfdcf86fd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "751e9a07-dd61-4a9b-98f0-05eb191499f1") },
		{ "667a8bb6-99de-40ef-8946-6c36412df4d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "7c788ce8-7b50-4152-a830-e1177138f469"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "667a8bb6-99de-40ef-8946-6c36412df4d9") },
		{ "6511841e-e643-4c22-9273-ce5966dc3912", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3456692d-f93d-496e-b951-a675d4900536"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6511841e-e643-4c22-9273-ce5966dc3912") },
		{ "6d1e4ca0-faeb-4543-9fcb-57e047f64fb1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a99c25e5-08b5-4068-9161-4ab4b9da429f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6d1e4ca0-faeb-4543-9fcb-57e047f64fb1") },
		{ "fddf6099-ee08-40ec-926a-16684241ce48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "bb72f9c6-09bb-422a-b5d4-9a97ce6faec7"}, { new NonTerminator(NonTerminatorType.ExpressionList), "69da751b-6a02-422e-84dd-e60952edd5b3"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fddf6099-ee08-40ec-926a-16684241ce48") },
		{ "d02712ea-5f52-412c-b2c6-a0c65d7b4ad3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d02712ea-5f52-412c-b2c6-a0c65d7b4ad3") },
		{ "9136ec11-c2d0-4cd4-903b-47ce7d0acd07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "cb9b53a1-220a-4cab-9860-11875ffe0efd"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9136ec11-c2d0-4cd4-903b-47ce7d0acd07") },
		{ "81b8e2c2-7233-4801-8899-1722b8505488", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "81b8e2c2-7233-4801-8899-1722b8505488") },
		{ "53577af0-a34b-49b7-a069-1f75bb6d1db2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "53577af0-a34b-49b7-a069-1f75bb6d1db2") },
		{ "aaf70ad5-d1a1-4886-a2bc-1e06339d04af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "aaf70ad5-d1a1-4886-a2bc-1e06339d04af") },
		{ "9eb2842a-1572-4918-9844-2feaa5cc39f6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "9eb2842a-1572-4918-9844-2feaa5cc39f6") },
		{ "3dbfa90a-5d7c-4ab1-b22d-280ec2c29df7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ac98e717-18c4-4d33-ab3d-829a849c15f3"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3dbfa90a-5d7c-4ab1-b22d-280ec2c29df7") },
		{ "11a20fc9-fb59-4428-b0c1-f6efa412ba5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "6d854d70-c549-4c54-b4b4-be5115c12be8"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "11a20fc9-fb59-4428-b0c1-f6efa412ba5f") },
		{ "c9c73c71-8d29-49bc-9678-3f330995cb91", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "31583a25-4eeb-42ff-9dca-52b99e96bca3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c9c73c71-8d29-49bc-9678-3f330995cb91") },
		{ "fdd4eff1-f432-44c3-95cc-04854e0aa40c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "51c015e8-a8c2-47da-b993-f196bb938d7e"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "f928a75d-a4db-487e-a0cd-52d9ca904a8d"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "fdd4eff1-f432-44c3-95cc-04854e0aa40c") },
		{ "0d02e1d4-d78b-4019-a7bd-6b8c6fe08e37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "67e63b46-842a-4cc9-b27f-11b09466b910"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0d02e1d4-d78b-4019-a7bd-6b8c6fe08e37") },
		{ "26bcc0d0-0146-41ca-a19e-298fccaefbd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "26bcc0d0-0146-41ca-a19e-298fccaefbd3") },
		{ "6f9ef057-1227-46ff-9bc5-a80ced1c1c52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52") },
		{ "420e78b1-4a11-4138-bea6-40e52080d87a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "420e78b1-4a11-4138-bea6-40e52080d87a") },
		{ "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "bc43a728-0e45-4b02-9683-da0cc3fad18b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c") },
		{ "0d89aa43-37a4-4968-978a-0803faae50e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "c4037f40-b27c-42f9-b0e9-c6ccedb11233"}, { new NonTerminator(NonTerminatorType.IdVarPart), "bba10811-fc61-4a40-9f95-ec7cf18ff557"}, { new Terminator(DelimiterType.LeftSquareBracket), "3bd290db-5e45-4d9d-b8e4-634d46d260ec"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "0d89aa43-37a4-4968-978a-0803faae50e8") },
		{ "bf131321-b3af-49c5-8027-86396b6872c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1f649678-08aa-4a8c-949c-2efd89844328"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf131321-b3af-49c5-8027-86396b6872c4") },
		{ "16515a1b-acea-49a9-b7ba-a123773961bc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "431fa655-343c-43a0-a4e7-9e1b0f26514e"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "16515a1b-acea-49a9-b7ba-a123773961bc") },
		{ "ace0221f-53fa-4b1d-935b-5e47e84c6067", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "155be5d7-2f4b-4467-b422-87c477a88d29"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ace0221f-53fa-4b1d-935b-5e47e84c6067") },
		{ "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce") },
		{ "074d0326-2f3c-4044-a9d2-816242c2d63d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "074d0326-2f3c-4044-a9d2-816242c2d63d") },
		{ "e543b165-d65b-4169-ba4d-e1f92a6d69b2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e543b165-d65b-4169-ba4d-e1f92a6d69b2") },
		{ "5beec4cf-85ae-4849-9f2e-842ea74d7200", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "7c8c601d-2a17-4974-8df4-2958b0123787"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5beec4cf-85ae-4849-9f2e-842ea74d7200") },
		{ "1af699b8-c50a-497a-850f-c7be2de94269", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "4c3ffb7d-d412-48bb-862b-6c4c3cc93ca3"}, { new NonTerminator(NonTerminatorType.Term), "0247a51d-336d-413b-9b5c-c1e7e2bcb3e2"}, { new NonTerminator(NonTerminatorType.Factor), "60f20762-8c4c-41dd-bc93-dc8c84db0682"}, { Terminator.NumberTerminator, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2"}, { new NonTerminator(NonTerminatorType.Variable), "f87282db-e5a1-4b5b-b673-d0b76c972837"}, { new Terminator(DelimiterType.LeftParenthesis), "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e"}, { Terminator.IdentifierTerminator, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2"}, { new Terminator(KeywordType.Not), "c79c62fa-c519-4ece-b1bb-eec179b25cb3"}, { new Terminator(OperatorType.Minus), "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87"}, { new Terminator(OperatorType.Plus), "c5db1619-b97d-4ddb-ac28-08eefe9b20ca"}, { new Terminator(KeywordType.True), "c37942f8-2eb1-4cba-8b4a-b60d627d577c"}, { new Terminator(KeywordType.False), "29a323e8-0116-4be6-a550-317cc5e48e9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1af699b8-c50a-497a-850f-c7be2de94269") },
		{ "7e62569f-9cd9-4b5d-9906-60c4afb167f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b13146e4-c202-440c-9493-63dd6ad0c5cb"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7e62569f-9cd9-4b5d-9906-60c4afb167f4") },
		{ "ae62e89b-0b5f-4c6f-be21-981e9671786e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1f9412ee-dc90-4d46-80fc-9e842389f9cd"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae62e89b-0b5f-4c6f-be21-981e9671786e") },
		{ "57f5b409-20e9-4cc4-9421-f4b03d18b459", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "31dc6f79-88bb-47fa-b947-2d3cca5a052a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "57f5b409-20e9-4cc4-9421-f4b03d18b459") },
		{ "0a477d08-fed1-4fcb-badf-466388e868f9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b9eef3c1-e869-48cb-925a-84cc484bb62b"}, { new NonTerminator(NonTerminatorType.ExpressionList), "684f23c1-cf3f-4c42-bb65-91a1b2c1cc68"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0a477d08-fed1-4fcb-badf-466388e868f9") },
		{ "7530f538-3ae0-42ba-b543-de55bc02da6c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "7530f538-3ae0-42ba-b543-de55bc02da6c") },
		{ "09d0ed01-10ef-4dd0-839e-0f6cff8f68e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "bddabbd1-7bf0-4d45-a29a-42be2f867b43"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "09d0ed01-10ef-4dd0-839e-0f6cff8f68e3") },
		{ "7f7a48dc-9adb-4beb-9218-c7fda97823ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7f7a48dc-9adb-4beb-9218-c7fda97823ef") },
		{ "882bfa6a-fb20-4088-83cd-1a03204deabf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "882bfa6a-fb20-4088-83cd-1a03204deabf") },
		{ "38fa7d9e-c576-4f4c-b1b8-2609aa3e74e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "38fa7d9e-c576-4f4c-b1b8-2609aa3e74e8") },
		{ "0b5bdcae-f6c9-4808-ab65-11517f66c6fb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "0b5bdcae-f6c9-4808-ab65-11517f66c6fb") },
		{ "93da2c69-ca85-444c-8720-a81732bebb2a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "1e4b0846-09a6-4f32-92b3-6894718eb4e2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "93da2c69-ca85-444c-8720-a81732bebb2a") },
		{ "e678709b-de6d-4b47-ac05-358101c1743a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "f4fb9eb5-e0fd-49f6-b2cd-d8e71aca20cf"}, { new NonTerminator(NonTerminatorType.Term), "9d74abdd-6566-4fce-844c-b4070177df3b"}, { new NonTerminator(NonTerminatorType.Factor), "2267b72e-e179-4cec-853d-630a52b9d22b"}, { Terminator.NumberTerminator, "0d4d8133-eb53-49f2-9341-489db84d615b"}, { new NonTerminator(NonTerminatorType.Variable), "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d"}, { new Terminator(DelimiterType.LeftParenthesis), "785f53e2-1ff3-414e-a54b-d53f74bb08ad"}, { Terminator.IdentifierTerminator, "a7b75ed5-af36-43c7-bbde-c5f49995d060"}, { new Terminator(KeywordType.Not), "31658312-8d73-4d47-b03e-c3716a9e250e"}, { new Terminator(OperatorType.Minus), "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db"}, { new Terminator(OperatorType.Plus), "b46278a2-0b18-40f7-9b32-adb4bd69cb78"}, { new Terminator(KeywordType.True), "58ad9ef1-afb6-48e6-9c6b-9695512df704"}, { new Terminator(KeywordType.False), "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e678709b-de6d-4b47-ac05-358101c1743a") },
		{ "f83f4997-f455-4025-bd96-cc799435f14c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b7c1b096-5479-4e49-a673-24cee28834a8"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f83f4997-f455-4025-bd96-cc799435f14c") },
		{ "35226c26-fd83-42d7-853c-d29d48133038", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "af5a3457-dde6-4196-9319-59126d6640f7"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "35226c26-fd83-42d7-853c-d29d48133038") },
		{ "1f6c5f36-1d5c-4d99-8a21-c158ae35830d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f06ef383-6f25-4da5-a3c1-322c45213768"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1f6c5f36-1d5c-4d99-8a21-c158ae35830d") },
		{ "728fc123-57e4-4af8-ab75-fa670f046f9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f1d69dc3-602b-4da2-810e-9bf78def0058"}, { new NonTerminator(NonTerminatorType.ExpressionList), "d97bdf66-358a-41ba-8e8d-d188bad3e6e4"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "728fc123-57e4-4af8-ab75-fa670f046f9d") },
		{ "2ad07047-be21-455c-b607-4e2c1672b85a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "2ad07047-be21-455c-b607-4e2c1672b85a") },
		{ "dd850ba2-1a8c-4a4b-9a95-d28a95d0824b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ebb19376-2bb4-4c80-a6a2-e99ac4f6f626"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dd850ba2-1a8c-4a4b-9a95-d28a95d0824b") },
		{ "4f66c1d6-c004-488e-a441-6e57c5deae80", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "4f66c1d6-c004-488e-a441-6e57c5deae80") },
		{ "5814a2e7-7245-4f01-a511-99ab0fc4d7d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "5814a2e7-7245-4f01-a511-99ab0fc4d7d7") },
		{ "69971c62-2a0c-4604-988e-2b6ef4e59be8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "69971c62-2a0c-4604-988e-2b6ef4e59be8") },
		{ "de3fb9f6-3a55-42a3-8e3e-bf03c453b822", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramBody))}, }, "de3fb9f6-3a55-42a3-8e3e-bf03c453b822") },
		{ "9b21f326-b9a4-4cef-93ad-4ac795053f7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "9b1ff833-4e6d-470b-80ad-1b5a41902b83"}, { new NonTerminator(NonTerminatorType.Statement), "f7d7f42c-bf10-43d4-8e97-409785b892ff"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "9b21f326-b9a4-4cef-93ad-4ac795053f7a") },
		{ "25483c72-4b02-41b4-8bc5-b1f864a15187", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "63885661-a232-41c3-9b86-8f3acd369238"},}, new Dictionary<Terminator, ReduceInformation>{ }, "25483c72-4b02-41b4-8bc5-b1f864a15187") },
		{ "0e921e41-05f1-4dce-8ffb-85338e8621c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "699372ae-253a-4afd-b887-7fcb4b74f3fe"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "0e921e41-05f1-4dce-8ffb-85338e8621c0") },
		{ "ca947fdb-cde8-4752-97a0-d1a13c5fd744", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "ca947fdb-cde8-4752-97a0-d1a13c5fd744") },
		{ "40b5d737-b43a-42ac-8238-d4f197167db8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Parameter), "b725044e-275e-421c-b156-1deeb8682521"}, { new NonTerminator(NonTerminatorType.VarParameter), "0378960f-af7e-418d-acb8-28aade479117"}, { new NonTerminator(NonTerminatorType.ValueParameter), "930bfe26-2081-464f-9231-cc2c2f2bbdd9"}, { new Terminator(KeywordType.Var), "7fd499b5-c4cf-4680-b56d-1a66d7d3eb8e"}, { Terminator.IdentifierTerminator, "2fd99590-86fb-4722-aa41-2f96e1a0a41c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "40b5d737-b43a-42ac-8238-d4f197167db8") },
		{ "b2563e77-2a1a-4b0c-ae97-20e0694e707c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, }, "b2563e77-2a1a-4b0c-ae97-20e0694e707c") },
		{ "6680defb-1123-4c05-9626-13adfcbb646d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ValueParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ValueParameter))}, }, "6680defb-1123-4c05-9626-13adfcbb646d") },
		{ "1a933aae-c301-4ce5-9e03-9e7ba76073fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "fc5789b9-9d19-44c3-9862-182730b83237"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1a933aae-c301-4ce5-9e03-9e7ba76073fe") },
		{ "1823fe2a-60d4-4212-8fd4-bf96ffe71d19", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "fd8a3538-6475-41b4-a8e1-6311f4af42d7"}, { new NonTerminator(NonTerminatorType.BasicType), "2a811ee0-fa89-43e1-866d-5551afa06384"}, { new Terminator(KeywordType.Array), "d99a28a1-3151-450e-ba2b-7f4aefd351e6"}, { new Terminator(KeywordType.Integer), "8e00c7b1-2698-40a2-9d75-c6c231ca214d"}, { new Terminator(KeywordType.Real), "1af623d2-70f9-4436-8ef1-6a6f38eb66ce"}, { new Terminator(KeywordType.Boolean), "c111394e-c196-4e91-9e1f-29f2ca60aae1"}, { new Terminator(KeywordType.Character), "154e611b-b0ac-4ff7-a430-1b750a012404"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1823fe2a-60d4-4212-8fd4-bf96ffe71d19") },
		{ "fc9b09a6-f614-486f-a23d-797719a0912b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "fc9b09a6-f614-486f-a23d-797719a0912b") },
		{ "854d7dc0-d01d-4135-8244-b43a6d9e9902", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "854d7dc0-d01d-4135-8244-b43a6d9e9902") },
		{ "49c7d99b-1dd9-403f-8b39-1999c20f67dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "4aa4f335-86c1-4f4c-9cf4-88313d4247ba"},}, new Dictionary<Terminator, ReduceInformation>{ }, "49c7d99b-1dd9-403f-8b39-1999c20f67dd") },
		{ "eeb042f2-1f51-4b85-a221-dd158bb21b06", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "eeb042f2-1f51-4b85-a221-dd158bb21b06") },
		{ "7901b2aa-d796-49ca-ac12-9832bba31f76", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "53af4ff8-86ea-490a-9209-9b45ccf78a2c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7901b2aa-d796-49ca-ac12-9832bba31f76") },
		{ "ba617890-7ac5-448f-a25d-44d047183652", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "44f6ccb9-9dfa-46ba-aea9-a510a85c7f9a"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "ba617890-7ac5-448f-a25d-44d047183652") },
		{ "0569958f-44ec-4e2e-ae51-95102a293e53", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fdf2caf6-3dd1-4f8b-941a-e19e7580eb23"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0569958f-44ec-4e2e-ae51-95102a293e53") },
		{ "2e01c973-a9fc-40ce-8dc1-df110cdb0e61", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "2e01c973-a9fc-40ce-8dc1-df110cdb0e61") },
		{ "118d5952-60b4-42d8-9773-dd7522338535", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "118d5952-60b4-42d8-9773-dd7522338535") },
		{ "9451c76a-01e8-4dbf-8a16-f7db028b7adb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "9451c76a-01e8-4dbf-8a16-f7db028b7adb") },
		{ "df9b2921-601d-43d6-8168-546a4d5597fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "62577db7-9b09-4ebb-aa6a-005a5c41d199"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df9b2921-601d-43d6-8168-546a4d5597fc") },
		{ "5ddf6fe6-d704-41a4-b552-e817ef6f729f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "051b4a7e-be7f-4037-980e-a9cf7376e8dc"}, { new NonTerminator(NonTerminatorType.IdVarPart), "aaa54aa2-8d3a-4d1f-b79e-a1fd06b06a84"}, { new Terminator(DelimiterType.LeftSquareBracket), "a08db743-71f6-40db-a4c9-6dca4dde579e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5ddf6fe6-d704-41a4-b552-e817ef6f729f") },
		{ "5332de79-32c9-4e62-a7de-15ee234934bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e124b53f-21e8-4548-97f7-7ffa3342a588"}, { Terminator.NumberTerminator, "118d5952-60b4-42d8-9773-dd7522338535"}, { new NonTerminator(NonTerminatorType.Variable), "9451c76a-01e8-4dbf-8a16-f7db028b7adb"}, { new Terminator(DelimiterType.LeftParenthesis), "df9b2921-601d-43d6-8168-546a4d5597fc"}, { Terminator.IdentifierTerminator, "5ddf6fe6-d704-41a4-b552-e817ef6f729f"}, { new Terminator(KeywordType.Not), "5332de79-32c9-4e62-a7de-15ee234934bf"}, { new Terminator(OperatorType.Minus), "39df86b4-3963-4ba3-a04e-047cd10c6a62"}, { new Terminator(OperatorType.Plus), "21d97a2b-d0ff-4cf4-b483-728ef20a436f"}, { new Terminator(KeywordType.True), "9dc09043-74f0-4711-8c01-db46e7a63643"}, { new Terminator(KeywordType.False), "7d2d2ab3-6074-49b5-a491-1114292132cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5332de79-32c9-4e62-a7de-15ee234934bf") },
		{ "39df86b4-3963-4ba3-a04e-047cd10c6a62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cda00910-9c85-459a-a5f0-5d2a3733f83a"}, { Terminator.NumberTerminator, "118d5952-60b4-42d8-9773-dd7522338535"}, { new NonTerminator(NonTerminatorType.Variable), "9451c76a-01e8-4dbf-8a16-f7db028b7adb"}, { new Terminator(DelimiterType.LeftParenthesis), "df9b2921-601d-43d6-8168-546a4d5597fc"}, { Terminator.IdentifierTerminator, "5ddf6fe6-d704-41a4-b552-e817ef6f729f"}, { new Terminator(KeywordType.Not), "5332de79-32c9-4e62-a7de-15ee234934bf"}, { new Terminator(OperatorType.Minus), "39df86b4-3963-4ba3-a04e-047cd10c6a62"}, { new Terminator(OperatorType.Plus), "21d97a2b-d0ff-4cf4-b483-728ef20a436f"}, { new Terminator(KeywordType.True), "9dc09043-74f0-4711-8c01-db46e7a63643"}, { new Terminator(KeywordType.False), "7d2d2ab3-6074-49b5-a491-1114292132cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "39df86b4-3963-4ba3-a04e-047cd10c6a62") },
		{ "21d97a2b-d0ff-4cf4-b483-728ef20a436f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d7510c7b-ba89-472a-aa05-64172b3508cb"}, { Terminator.NumberTerminator, "118d5952-60b4-42d8-9773-dd7522338535"}, { new NonTerminator(NonTerminatorType.Variable), "9451c76a-01e8-4dbf-8a16-f7db028b7adb"}, { new Terminator(DelimiterType.LeftParenthesis), "df9b2921-601d-43d6-8168-546a4d5597fc"}, { Terminator.IdentifierTerminator, "5ddf6fe6-d704-41a4-b552-e817ef6f729f"}, { new Terminator(KeywordType.Not), "5332de79-32c9-4e62-a7de-15ee234934bf"}, { new Terminator(OperatorType.Minus), "39df86b4-3963-4ba3-a04e-047cd10c6a62"}, { new Terminator(OperatorType.Plus), "21d97a2b-d0ff-4cf4-b483-728ef20a436f"}, { new Terminator(KeywordType.True), "9dc09043-74f0-4711-8c01-db46e7a63643"}, { new Terminator(KeywordType.False), "7d2d2ab3-6074-49b5-a491-1114292132cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "21d97a2b-d0ff-4cf4-b483-728ef20a436f") },
		{ "9dc09043-74f0-4711-8c01-db46e7a63643", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "9dc09043-74f0-4711-8c01-db46e7a63643") },
		{ "7d2d2ab3-6074-49b5-a491-1114292132cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7d2d2ab3-6074-49b5-a491-1114292132cd") },
		{ "4156b65f-321a-4e96-ae31-0330777bbc33", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "d5e8d0dc-d894-40b9-aaca-e1660241c41b"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "4156b65f-321a-4e96-ae31-0330777bbc33") },
		{ "c1854fed-3d7f-4668-811a-4b82176d4965", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "c1854fed-3d7f-4668-811a-4b82176d4965") },
		{ "7162215e-0834-43ae-94f2-801c43cb7dfd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "7162215e-0834-43ae-94f2-801c43cb7dfd") },
		{ "c0ce220a-c936-4e08-9a81-08b7147ba8b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "c0ce220a-c936-4e08-9a81-08b7147ba8b9") },
		{ "d82c5fc2-22c2-41df-8317-d8a6ac3eba10", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "bdc9c78e-ca83-49a8-92fb-5b91652f3a90"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d82c5fc2-22c2-41df-8317-d8a6ac3eba10") },
		{ "aba957b3-15ec-4dbb-976f-6e8e5f2520b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "c5f04330-7d45-41d8-a5fd-8e2039796fec"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aba957b3-15ec-4dbb-976f-6e8e5f2520b4") },
		{ "90363a8f-7f4a-401f-a5ac-878a034e4453", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "90363a8f-7f4a-401f-a5ac-878a034e4453") },
		{ "469dd03b-8892-47dc-b743-c98e497d66fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "f4d15b21-17bb-4fe0-9913-e3346cecb6a3"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "469dd03b-8892-47dc-b743-c98e497d66fa") },
		{ "14d2a496-27ff-4ba0-b9bf-e5a503cebe29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "7a966d66-2a5e-4eb5-b37e-c8c50be29dd3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "23620ebe-7492-4b56-bf17-d65b352fe28f"}, { new NonTerminator(NonTerminatorType.Term), "885a83d0-10be-4ec3-ac23-7485a698a9cb"}, { new NonTerminator(NonTerminatorType.Factor), "935cddc7-c502-403e-8611-26ec1d618f1f"}, { Terminator.NumberTerminator, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5"}, { new NonTerminator(NonTerminatorType.Variable), "9b08c15a-47f3-41ad-89fc-87a70b94d277"}, { new Terminator(DelimiterType.LeftParenthesis), "a8d9ae7e-7768-428a-8b1b-1b666fbb5907"}, { Terminator.IdentifierTerminator, "2da2fd69-4c2f-4183-8897-8c75052f90ce"}, { new Terminator(KeywordType.Not), "df9d412e-5c51-4948-be23-ed680985f2ae"}, { new Terminator(OperatorType.Minus), "fffb31ab-dc7f-4607-b5a8-f29000182972"}, { new Terminator(OperatorType.Plus), "a1794963-b3a9-4188-a1f8-19513de4a213"}, { new Terminator(KeywordType.True), "37b04a17-928f-4c84-86a4-3fc64cee61f2"}, { new Terminator(KeywordType.False), "87d11209-56f5-4141-93a7-4abaee6c3b44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14d2a496-27ff-4ba0-b9bf-e5a503cebe29") },
		{ "b45eb976-9df0-4f3e-9dbf-bb9dd8fd3c23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "aa12f399-0fac-48f3-b3a5-ddf8ffc7c55b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b45eb976-9df0-4f3e-9dbf-bb9dd8fd3c23") },
		{ "9afb6c66-5cc3-4ddc-8049-27381dafb426", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "d79288f9-5914-4b0d-9b87-cdcc5299788b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9afb6c66-5cc3-4ddc-8049-27381dafb426") },
		{ "77f0e0d9-ee75-4e79-a257-4404d92e8bd5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "35a8919d-3896-442a-8479-fcaa1a3d9787"}, { new NonTerminator(NonTerminatorType.ExpressionList), "296bbf0a-b93e-4cae-9b20-255b667e827c"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "77f0e0d9-ee75-4e79-a257-4404d92e8bd5") },
		{ "ce6da976-60ba-4844-baab-0e3e1b6b397e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "55f2b7d9-8f80-41c0-a650-9fda9cd44433"}, { new Terminator(DelimiterType.Semicolon), "e05f016b-b74f-46b6-a24f-f01a847e7d36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ce6da976-60ba-4844-baab-0e3e1b6b397e") },
		{ "bc837e3b-b5f7-4c23-90cc-ef990a5a98a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "d9fdd74b-1ba6-46f2-bc36-060354a88fbe"}, { new NonTerminator(NonTerminatorType.Factor), "06f68d47-af3e-48cb-ad03-28be91344b7c"}, { Terminator.NumberTerminator, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74"}, { new NonTerminator(NonTerminatorType.Variable), "ee1f29a9-06b3-45fd-824c-a319a2988aa9"}, { new Terminator(DelimiterType.LeftParenthesis), "8abb7440-8069-44c2-ab58-94c4486624d2"}, { Terminator.IdentifierTerminator, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a"}, { new Terminator(KeywordType.Not), "511d04d4-c513-4863-8a48-ab5258a8c6a5"}, { new Terminator(OperatorType.Minus), "71d87b84-da4f-4b6e-b472-04e6910f77d3"}, { new Terminator(OperatorType.Plus), "eaa10f35-d92c-4913-9d3e-66152dc1592e"}, { new Terminator(KeywordType.True), "9067f1a1-acd5-4ef6-b59a-006235d313ba"}, { new Terminator(KeywordType.False), "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc837e3b-b5f7-4c23-90cc-ef990a5a98a7") },
		{ "50c1a67d-c4d7-446f-a505-a5dfb43f5988", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "877c7a73-5b59-4177-ace7-2c573d97f8f5"}, { Terminator.NumberTerminator, "5c064a71-4cd3-434b-b60c-ac24fe1a2c74"}, { new NonTerminator(NonTerminatorType.Variable), "ee1f29a9-06b3-45fd-824c-a319a2988aa9"}, { new Terminator(DelimiterType.LeftParenthesis), "8abb7440-8069-44c2-ab58-94c4486624d2"}, { Terminator.IdentifierTerminator, "8cbc937e-ce5b-49a5-a35c-d07bd2f1d82a"}, { new Terminator(KeywordType.Not), "511d04d4-c513-4863-8a48-ab5258a8c6a5"}, { new Terminator(OperatorType.Minus), "71d87b84-da4f-4b6e-b472-04e6910f77d3"}, { new Terminator(OperatorType.Plus), "eaa10f35-d92c-4913-9d3e-66152dc1592e"}, { new Terminator(KeywordType.True), "9067f1a1-acd5-4ef6-b59a-006235d313ba"}, { new Terminator(KeywordType.False), "019dfdf3-9d25-44e2-89c9-5f5d2e7e7511"},}, new Dictionary<Terminator, ReduceInformation>{ }, "50c1a67d-c4d7-446f-a505-a5dfb43f5988") },
		{ "8b060542-f56f-4a7f-ac56-5c1e5ed27462", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2d9f1790-8c00-4dc9-98de-106aba93f238"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8b060542-f56f-4a7f-ac56-5c1e5ed27462") },
		{ "b66337d9-9864-4821-b361-93996d20046d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "02633a30-0a57-4601-acba-ccc074aab324"}, { new NonTerminator(NonTerminatorType.ExpressionList), "ad1d5fb0-3d35-484e-a64f-68dd96ffc055"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b66337d9-9864-4821-b361-93996d20046d") },
		{ "b5edb048-e007-46d4-be13-b41b1b9bb78e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "b5edb048-e007-46d4-be13-b41b1b9bb78e") },
		{ "953656c4-d3dd-4480-9225-b70bd0ab69f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "fcd89654-b1d0-4449-b0d1-5f0dacb7a415"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "953656c4-d3dd-4480-9225-b70bd0ab69f0") },
		{ "a1415324-197b-43af-959c-6f94ec59d928", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "a1415324-197b-43af-959c-6f94ec59d928") },
		{ "40d75275-c36d-49a3-876e-dd1d26db7c5e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "40d75275-c36d-49a3-876e-dd1d26db7c5e") },
		{ "7b0f787a-85bb-4963-af18-6a9619df90ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7b0f787a-85bb-4963-af18-6a9619df90ae") },
		{ "9535c43f-c365-4de8-9c84-4239752f1bb0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "ac09e1d0-3a6d-44c3-9253-3676c4bf9423"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "9535c43f-c365-4de8-9c84-4239752f1bb0") },
		{ "fe0e57e8-6cb8-4f07-a75a-690c2cfb2a0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "f8ab527d-ec4b-4b2d-959c-7311f736f8d0"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fe0e57e8-6cb8-4f07-a75a-690c2cfb2a0f") },
		{ "caed1736-e1a9-4a08-a996-38057dc9ef38", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "caed1736-e1a9-4a08-a996-38057dc9ef38") },
		{ "2d7a0ca8-d47b-47af-96b1-79a244a8162b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2d7a0ca8-d47b-47af-96b1-79a244a8162b") },
		{ "0203b66a-97cf-49e4-b4a5-44a9240daee1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0203b66a-97cf-49e4-b4a5-44a9240daee1") },
		{ "72ae4a09-129b-4e98-a17f-98fd3df8f433", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ec32e05c-955a-455f-8b69-2b44e3b27f49"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "72ae4a09-129b-4e98-a17f-98fd3df8f433") },
		{ "b3d5f8de-1810-4638-a42c-ed6b741212fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "b59c4582-2588-4859-8276-c159572aa60e"}, { new NonTerminator(NonTerminatorType.IdVarPart), "7558e831-e7c3-41ad-8dc6-04aa53c0e6ec"}, { new Terminator(DelimiterType.LeftSquareBracket), "517f9a1e-26b5-48b0-a234-bd254c2a9aee"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b3d5f8de-1810-4638-a42c-ed6b741212fe") },
		{ "baa9ab37-d944-4c90-83a1-2d243c2b3aec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "557fe2b8-e99b-4367-ac62-3657ec6532db"}, { Terminator.NumberTerminator, "2d7a0ca8-d47b-47af-96b1-79a244a8162b"}, { new NonTerminator(NonTerminatorType.Variable), "0203b66a-97cf-49e4-b4a5-44a9240daee1"}, { new Terminator(DelimiterType.LeftParenthesis), "72ae4a09-129b-4e98-a17f-98fd3df8f433"}, { Terminator.IdentifierTerminator, "b3d5f8de-1810-4638-a42c-ed6b741212fe"}, { new Terminator(KeywordType.Not), "baa9ab37-d944-4c90-83a1-2d243c2b3aec"}, { new Terminator(OperatorType.Minus), "075b0a32-180c-440c-9b7b-6cb315a3b4b8"}, { new Terminator(OperatorType.Plus), "7236e6ff-062d-4114-a692-c106386308ab"}, { new Terminator(KeywordType.True), "bf93e069-f9ae-4e60-9b14-743260b945e8"}, { new Terminator(KeywordType.False), "1cf66947-0ecc-4413-8902-7bfcfdcf86fd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "baa9ab37-d944-4c90-83a1-2d243c2b3aec") },
		{ "075b0a32-180c-440c-9b7b-6cb315a3b4b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "93e778c9-b993-4168-96d5-fc7e0619e1dc"}, { Terminator.NumberTerminator, "2d7a0ca8-d47b-47af-96b1-79a244a8162b"}, { new NonTerminator(NonTerminatorType.Variable), "0203b66a-97cf-49e4-b4a5-44a9240daee1"}, { new Terminator(DelimiterType.LeftParenthesis), "72ae4a09-129b-4e98-a17f-98fd3df8f433"}, { Terminator.IdentifierTerminator, "b3d5f8de-1810-4638-a42c-ed6b741212fe"}, { new Terminator(KeywordType.Not), "baa9ab37-d944-4c90-83a1-2d243c2b3aec"}, { new Terminator(OperatorType.Minus), "075b0a32-180c-440c-9b7b-6cb315a3b4b8"}, { new Terminator(OperatorType.Plus), "7236e6ff-062d-4114-a692-c106386308ab"}, { new Terminator(KeywordType.True), "bf93e069-f9ae-4e60-9b14-743260b945e8"}, { new Terminator(KeywordType.False), "1cf66947-0ecc-4413-8902-7bfcfdcf86fd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "075b0a32-180c-440c-9b7b-6cb315a3b4b8") },
		{ "7236e6ff-062d-4114-a692-c106386308ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e9813adf-695f-4d1d-bf52-5d69439eab3c"}, { Terminator.NumberTerminator, "2d7a0ca8-d47b-47af-96b1-79a244a8162b"}, { new NonTerminator(NonTerminatorType.Variable), "0203b66a-97cf-49e4-b4a5-44a9240daee1"}, { new Terminator(DelimiterType.LeftParenthesis), "72ae4a09-129b-4e98-a17f-98fd3df8f433"}, { Terminator.IdentifierTerminator, "b3d5f8de-1810-4638-a42c-ed6b741212fe"}, { new Terminator(KeywordType.Not), "baa9ab37-d944-4c90-83a1-2d243c2b3aec"}, { new Terminator(OperatorType.Minus), "075b0a32-180c-440c-9b7b-6cb315a3b4b8"}, { new Terminator(OperatorType.Plus), "7236e6ff-062d-4114-a692-c106386308ab"}, { new Terminator(KeywordType.True), "bf93e069-f9ae-4e60-9b14-743260b945e8"}, { new Terminator(KeywordType.False), "1cf66947-0ecc-4413-8902-7bfcfdcf86fd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7236e6ff-062d-4114-a692-c106386308ab") },
		{ "bf93e069-f9ae-4e60-9b14-743260b945e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "bf93e069-f9ae-4e60-9b14-743260b945e8") },
		{ "1cf66947-0ecc-4413-8902-7bfcfdcf86fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1cf66947-0ecc-4413-8902-7bfcfdcf86fd") },
		{ "7c788ce8-7b50-4152-a830-e1177138f469", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "6511841e-e643-4c22-9273-ce5966dc3912"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "7c788ce8-7b50-4152-a830-e1177138f469") },
		{ "3456692d-f93d-496e-b951-a675d4900536", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "3456692d-f93d-496e-b951-a675d4900536") },
		{ "a99c25e5-08b5-4068-9161-4ab4b9da429f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a99c25e5-08b5-4068-9161-4ab4b9da429f") },
		{ "bb72f9c6-09bb-422a-b5d4-9a97ce6faec7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "bb72f9c6-09bb-422a-b5d4-9a97ce6faec7") },
		{ "69da751b-6a02-422e-84dd-e60952edd5b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2dfefd0b-96ac-413e-b04b-b3373f9f0728"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "69da751b-6a02-422e-84dd-e60952edd5b3") },
		{ "cb9b53a1-220a-4cab-9860-11875ffe0efd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d9319c70-461d-4cdb-b7b4-bd77837b2ebd"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cb9b53a1-220a-4cab-9860-11875ffe0efd") },
		{ "ac98e717-18c4-4d33-ab3d-829a849c15f3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "ac98e717-18c4-4d33-ab3d-829a849c15f3") },
		{ "6d854d70-c549-4c54-b4b4-be5115c12be8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "6d854d70-c549-4c54-b4b4-be5115c12be8") },
		{ "31583a25-4eeb-42ff-9dca-52b99e96bca3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f53b614a-51f4-429d-9f35-6a2883307bec"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "00dc0d38-55fe-4e5b-a032-3d01772dd3f9"}, { new NonTerminator(NonTerminatorType.Term), "230281f9-66be-4ca7-8818-f4944e726d65"}, { new NonTerminator(NonTerminatorType.Factor), "a1bdce73-48fa-4f14-b8b6-e88691c025ee"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "31583a25-4eeb-42ff-9dca-52b99e96bca3") },
		{ "51c015e8-a8c2-47da-b993-f196bb938d7e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "5a091b64-1a9d-4c29-9fed-b2a24924254e"}, { new NonTerminator(NonTerminatorType.Term), "cd0c559d-b2da-4607-ba57-4668bdfe7544"}, { new NonTerminator(NonTerminatorType.Factor), "f87c3693-8813-4e52-897a-219e6b03e98b"}, { Terminator.NumberTerminator, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58"}, { new NonTerminator(NonTerminatorType.Variable), "93f2aeb0-544b-4512-a759-4beddb818dab"}, { new Terminator(DelimiterType.LeftParenthesis), "a3f5b2bb-164d-45da-8345-9e903c24c921"}, { Terminator.IdentifierTerminator, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b"}, { new Terminator(KeywordType.Not), "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd"}, { new Terminator(OperatorType.Minus), "7215eafb-c0b0-45ca-8077-0ac97db1201e"}, { new Terminator(OperatorType.Plus), "e4a63316-44d2-48a8-91c7-4b2694815e50"}, { new Terminator(KeywordType.True), "2776d0bd-36a9-4559-ab46-da3f863c2562"}, { new Terminator(KeywordType.False), "524cc89a-598b-468c-89dd-01f958884f6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "51c015e8-a8c2-47da-b993-f196bb938d7e") },
		{ "f928a75d-a4db-487e-a0cd-52d9ca904a8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "4a62591e-a51e-4404-86b4-b3f49d7e97f6"}, { new NonTerminator(NonTerminatorType.Factor), "26bcc0d0-0146-41ca-a19e-298fccaefbd3"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f928a75d-a4db-487e-a0cd-52d9ca904a8d") },
		{ "67e63b46-842a-4cc9-b27f-11b09466b910", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ba66ba2f-fc95-4974-b42d-9d60c92e7250"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "67e63b46-842a-4cc9-b27f-11b09466b910") },
		{ "bc43a728-0e45-4b02-9683-da0cc3fad18b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c99374e6-3e4e-43c5-bbfc-fd89b45f3c51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc43a728-0e45-4b02-9683-da0cc3fad18b") },
		{ "c4037f40-b27c-42f9-b0e9-c6ccedb11233", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3061ae66-4e67-4989-8ec7-759cb172d9e0"}, { new NonTerminator(NonTerminatorType.ExpressionList), "3280389c-08e6-4fc7-99ec-3cc7b4f18216"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c4037f40-b27c-42f9-b0e9-c6ccedb11233") },
		{ "bba10811-fc61-4a40-9f95-ec7cf18ff557", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "bba10811-fc61-4a40-9f95-ec7cf18ff557") },
		{ "3bd290db-5e45-4d9d-b8e4-634d46d260ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "87c4b5c8-40b9-4cee-a72b-204a6fb0827b"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3bd290db-5e45-4d9d-b8e4-634d46d260ec") },
		{ "1f649678-08aa-4a8c-949c-2efd89844328", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1f649678-08aa-4a8c-949c-2efd89844328") },
		{ "431fa655-343c-43a0-a4e7-9e1b0f26514e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "431fa655-343c-43a0-a4e7-9e1b0f26514e") },
		{ "155be5d7-2f4b-4467-b422-87c477a88d29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "155be5d7-2f4b-4467-b422-87c477a88d29") },
		{ "7c8c601d-2a17-4974-8df4-2958b0123787", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "7c8c601d-2a17-4974-8df4-2958b0123787") },
		{ "4c3ffb7d-d412-48bb-862b-6c4c3cc93ca3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "47b8b6fb-5470-4783-a67d-2c2d071c2269"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "4c3ffb7d-d412-48bb-862b-6c4c3cc93ca3") },
		{ "0247a51d-336d-413b-9b5c-c1e7e2bcb3e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "0ff969ac-9895-4912-87e8-8cb5ac13ef7c"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0247a51d-336d-413b-9b5c-c1e7e2bcb3e2") },
		{ "60f20762-8c4c-41dd-bc93-dc8c84db0682", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "60f20762-8c4c-41dd-bc93-dc8c84db0682") },
		{ "f95b34cd-97a5-4318-acca-b4a22e8dc0d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2") },
		{ "f87282db-e5a1-4b5b-b673-d0b76c972837", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f87282db-e5a1-4b5b-b673-d0b76c972837") },
		{ "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "4a70b3ba-5435-463d-af99-71129e0978ef"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e") },
		{ "f8fe3648-13a1-4035-9dce-f436bdbe3ec2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "e3562aec-6d0b-4e86-9143-eaf0ca8d1b56"}, { new NonTerminator(NonTerminatorType.IdVarPart), "58cab06b-9a08-4caa-be04-88fee59d7cb7"}, { new Terminator(DelimiterType.LeftSquareBracket), "7e02ad79-173a-4bf6-a6a1-d5169ec8be71"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2") },
		{ "c79c62fa-c519-4ece-b1bb-eec179b25cb3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "46e33e67-fd2c-456e-9399-cd744ca9ad32"}, { Terminator.NumberTerminator, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2"}, { new NonTerminator(NonTerminatorType.Variable), "f87282db-e5a1-4b5b-b673-d0b76c972837"}, { new Terminator(DelimiterType.LeftParenthesis), "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e"}, { Terminator.IdentifierTerminator, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2"}, { new Terminator(KeywordType.Not), "c79c62fa-c519-4ece-b1bb-eec179b25cb3"}, { new Terminator(OperatorType.Minus), "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87"}, { new Terminator(OperatorType.Plus), "c5db1619-b97d-4ddb-ac28-08eefe9b20ca"}, { new Terminator(KeywordType.True), "c37942f8-2eb1-4cba-8b4a-b60d627d577c"}, { new Terminator(KeywordType.False), "29a323e8-0116-4be6-a550-317cc5e48e9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c79c62fa-c519-4ece-b1bb-eec179b25cb3") },
		{ "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "26a48a34-b8af-47f0-b24a-e0b813db1ab1"}, { Terminator.NumberTerminator, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2"}, { new NonTerminator(NonTerminatorType.Variable), "f87282db-e5a1-4b5b-b673-d0b76c972837"}, { new Terminator(DelimiterType.LeftParenthesis), "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e"}, { Terminator.IdentifierTerminator, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2"}, { new Terminator(KeywordType.Not), "c79c62fa-c519-4ece-b1bb-eec179b25cb3"}, { new Terminator(OperatorType.Minus), "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87"}, { new Terminator(OperatorType.Plus), "c5db1619-b97d-4ddb-ac28-08eefe9b20ca"}, { new Terminator(KeywordType.True), "c37942f8-2eb1-4cba-8b4a-b60d627d577c"}, { new Terminator(KeywordType.False), "29a323e8-0116-4be6-a550-317cc5e48e9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87") },
		{ "c5db1619-b97d-4ddb-ac28-08eefe9b20ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e49ff909-c8e2-426c-81a8-53d3ce2d6883"}, { Terminator.NumberTerminator, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2"}, { new NonTerminator(NonTerminatorType.Variable), "f87282db-e5a1-4b5b-b673-d0b76c972837"}, { new Terminator(DelimiterType.LeftParenthesis), "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e"}, { Terminator.IdentifierTerminator, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2"}, { new Terminator(KeywordType.Not), "c79c62fa-c519-4ece-b1bb-eec179b25cb3"}, { new Terminator(OperatorType.Minus), "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87"}, { new Terminator(OperatorType.Plus), "c5db1619-b97d-4ddb-ac28-08eefe9b20ca"}, { new Terminator(KeywordType.True), "c37942f8-2eb1-4cba-8b4a-b60d627d577c"}, { new Terminator(KeywordType.False), "29a323e8-0116-4be6-a550-317cc5e48e9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c5db1619-b97d-4ddb-ac28-08eefe9b20ca") },
		{ "c37942f8-2eb1-4cba-8b4a-b60d627d577c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c37942f8-2eb1-4cba-8b4a-b60d627d577c") },
		{ "29a323e8-0116-4be6-a550-317cc5e48e9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "29a323e8-0116-4be6-a550-317cc5e48e9c") },
		{ "b13146e4-c202-440c-9493-63dd6ad0c5cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ae62e89b-0b5f-4c6f-be21-981e9671786e"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b13146e4-c202-440c-9493-63dd6ad0c5cb") },
		{ "1f9412ee-dc90-4d46-80fc-9e842389f9cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "1f9412ee-dc90-4d46-80fc-9e842389f9cd") },
		{ "31dc6f79-88bb-47fa-b947-2d3cca5a052a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "31dc6f79-88bb-47fa-b947-2d3cca5a052a") },
		{ "b9eef3c1-e869-48cb-925a-84cc484bb62b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b9eef3c1-e869-48cb-925a-84cc484bb62b") },
		{ "684f23c1-cf3f-4c42-bb65-91a1b2c1cc68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d017c940-5413-484a-a8e0-1c0810a0b616"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "684f23c1-cf3f-4c42-bb65-91a1b2c1cc68") },
		{ "bddabbd1-7bf0-4d45-a29a-42be2f867b43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "84d1bb50-b96c-4bcc-b0af-e128c006712a"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bddabbd1-7bf0-4d45-a29a-42be2f867b43") },
		{ "1e4b0846-09a6-4f32-92b3-6894718eb4e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "1e4b0846-09a6-4f32-92b3-6894718eb4e2") },
		{ "f4fb9eb5-e0fd-49f6-b2cd-d8e71aca20cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "dce69fc9-07f6-48ca-b58e-3af71d531b9b"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "f4fb9eb5-e0fd-49f6-b2cd-d8e71aca20cf") },
		{ "9d74abdd-6566-4fce-844c-b4070177df3b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "bf0c92a6-b193-45a5-a9e7-82089562d19b"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9d74abdd-6566-4fce-844c-b4070177df3b") },
		{ "2267b72e-e179-4cec-853d-630a52b9d22b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "2267b72e-e179-4cec-853d-630a52b9d22b") },
		{ "0d4d8133-eb53-49f2-9341-489db84d615b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0d4d8133-eb53-49f2-9341-489db84d615b") },
		{ "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d") },
		{ "785f53e2-1ff3-414e-a54b-d53f74bb08ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "403543b6-9ce7-4fc6-b837-b5e3350c73fc"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "785f53e2-1ff3-414e-a54b-d53f74bb08ad") },
		{ "a7b75ed5-af36-43c7-bbde-c5f49995d060", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "dac26fb4-0525-4bd1-ae95-d8c90795900f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "6e70dc7e-ec04-46fc-bbe0-3961d966c82f"}, { new Terminator(DelimiterType.LeftSquareBracket), "b1ef3bd2-f9fd-4b75-b162-739b43040b26"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a7b75ed5-af36-43c7-bbde-c5f49995d060") },
		{ "31658312-8d73-4d47-b03e-c3716a9e250e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d6719cba-3358-47a0-806d-b3e3ca912953"}, { Terminator.NumberTerminator, "0d4d8133-eb53-49f2-9341-489db84d615b"}, { new NonTerminator(NonTerminatorType.Variable), "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d"}, { new Terminator(DelimiterType.LeftParenthesis), "785f53e2-1ff3-414e-a54b-d53f74bb08ad"}, { Terminator.IdentifierTerminator, "a7b75ed5-af36-43c7-bbde-c5f49995d060"}, { new Terminator(KeywordType.Not), "31658312-8d73-4d47-b03e-c3716a9e250e"}, { new Terminator(OperatorType.Minus), "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db"}, { new Terminator(OperatorType.Plus), "b46278a2-0b18-40f7-9b32-adb4bd69cb78"}, { new Terminator(KeywordType.True), "58ad9ef1-afb6-48e6-9c6b-9695512df704"}, { new Terminator(KeywordType.False), "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "31658312-8d73-4d47-b03e-c3716a9e250e") },
		{ "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cef46389-9d40-4ce3-94c0-f2f8b3635c18"}, { Terminator.NumberTerminator, "0d4d8133-eb53-49f2-9341-489db84d615b"}, { new NonTerminator(NonTerminatorType.Variable), "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d"}, { new Terminator(DelimiterType.LeftParenthesis), "785f53e2-1ff3-414e-a54b-d53f74bb08ad"}, { Terminator.IdentifierTerminator, "a7b75ed5-af36-43c7-bbde-c5f49995d060"}, { new Terminator(KeywordType.Not), "31658312-8d73-4d47-b03e-c3716a9e250e"}, { new Terminator(OperatorType.Minus), "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db"}, { new Terminator(OperatorType.Plus), "b46278a2-0b18-40f7-9b32-adb4bd69cb78"}, { new Terminator(KeywordType.True), "58ad9ef1-afb6-48e6-9c6b-9695512df704"}, { new Terminator(KeywordType.False), "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db") },
		{ "b46278a2-0b18-40f7-9b32-adb4bd69cb78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4f356464-ae1b-4d1f-a78d-413deba7574e"}, { Terminator.NumberTerminator, "0d4d8133-eb53-49f2-9341-489db84d615b"}, { new NonTerminator(NonTerminatorType.Variable), "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d"}, { new Terminator(DelimiterType.LeftParenthesis), "785f53e2-1ff3-414e-a54b-d53f74bb08ad"}, { Terminator.IdentifierTerminator, "a7b75ed5-af36-43c7-bbde-c5f49995d060"}, { new Terminator(KeywordType.Not), "31658312-8d73-4d47-b03e-c3716a9e250e"}, { new Terminator(OperatorType.Minus), "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db"}, { new Terminator(OperatorType.Plus), "b46278a2-0b18-40f7-9b32-adb4bd69cb78"}, { new Terminator(KeywordType.True), "58ad9ef1-afb6-48e6-9c6b-9695512df704"}, { new Terminator(KeywordType.False), "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b46278a2-0b18-40f7-9b32-adb4bd69cb78") },
		{ "58ad9ef1-afb6-48e6-9c6b-9695512df704", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "58ad9ef1-afb6-48e6-9c6b-9695512df704") },
		{ "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6") },
		{ "b7c1b096-5479-4e49-a673-24cee28834a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "35226c26-fd83-42d7-853c-d29d48133038"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b7c1b096-5479-4e49-a673-24cee28834a8") },
		{ "af5a3457-dde6-4196-9319-59126d6640f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "af5a3457-dde6-4196-9319-59126d6640f7") },
		{ "f06ef383-6f25-4da5-a3c1-322c45213768", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "f06ef383-6f25-4da5-a3c1-322c45213768") },
		{ "f1d69dc3-602b-4da2-810e-9bf78def0058", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "f1d69dc3-602b-4da2-810e-9bf78def0058") },
		{ "d97bdf66-358a-41ba-8e8d-d188bad3e6e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e84a20f6-cc0b-439f-8256-f90af4b8c77f"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d97bdf66-358a-41ba-8e8d-d188bad3e6e4") },
		{ "ebb19376-2bb4-4c80-a6a2-e99ac4f6f626", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "dd782ae4-5aeb-4b84-ae1e-8178918a8213"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ebb19376-2bb4-4c80-a6a2-e99ac4f6f626") },
		{ "9b1ff833-4e6d-470b-80ad-1b5a41902b83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "c0342272-e754-4afb-85c8-1efeaa79434b"}, { new Terminator(DelimiterType.Semicolon), "e05f016b-b74f-46b6-a24f-f01a847e7d36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9b1ff833-4e6d-470b-80ad-1b5a41902b83") },
		{ "63885661-a232-41c3-9b86-8f3acd369238", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "03d461ff-96eb-4d00-8179-71306652b9ee"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "63885661-a232-41c3-9b86-8f3acd369238") },
		{ "b725044e-275e-421c-b156-1deeb8682521", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, }, "b725044e-275e-421c-b156-1deeb8682521") },
		{ "fc5789b9-9d19-44c3-9862-182730b83237", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "020cf9fe-62a8-4b45-8def-978f45933306"}, { new Terminator(DelimiterType.Comma), "1a933aae-c301-4ce5-9e03-9e7ba76073fe"}, { new Terminator(DelimiterType.Colon), "1823fe2a-60d4-4212-8fd4-bf96ffe71d19"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fc5789b9-9d19-44c3-9862-182730b83237") },
		{ "fd8a3538-6475-41b4-a8e1-6311f4af42d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "fd8a3538-6475-41b4-a8e1-6311f4af42d7") },
		{ "2a811ee0-fa89-43e1-866d-5551afa06384", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "2a811ee0-fa89-43e1-866d-5551afa06384") },
		{ "d99a28a1-3151-450e-ba2b-7f4aefd351e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "4491d4b7-84b9-4584-acd6-89210c94c275"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d99a28a1-3151-450e-ba2b-7f4aefd351e6") },
		{ "8e00c7b1-2698-40a2-9d75-c6c231ca214d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "8e00c7b1-2698-40a2-9d75-c6c231ca214d") },
		{ "1af623d2-70f9-4436-8ef1-6a6f38eb66ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "1af623d2-70f9-4436-8ef1-6a6f38eb66ce") },
		{ "c111394e-c196-4e91-9e1f-29f2ca60aae1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "c111394e-c196-4e91-9e1f-29f2ca60aae1") },
		{ "154e611b-b0ac-4ff7-a430-1b750a012404", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "154e611b-b0ac-4ff7-a430-1b750a012404") },
		{ "4aa4f335-86c1-4f4c-9cf4-88313d4247ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "0e625463-5cbc-491f-b72f-b944fa3d0214"}, { new Terminator(KeywordType.Integer), "06bbef92-0d08-4369-a6e1-07308f373494"}, { new Terminator(KeywordType.Real), "141114e5-17fa-4ef5-8833-b7cf274019dc"}, { new Terminator(KeywordType.Boolean), "cfb5a6ab-7bb3-4d92-afae-7235a569a96f"}, { new Terminator(KeywordType.Character), "9ff8a64c-96bc-45f6-8b24-56e3e3dffd24"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4aa4f335-86c1-4f4c-9cf4-88313d4247ba") },
		{ "53af4ff8-86ea-490a-9209-9b45ccf78a2c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, }, "53af4ff8-86ea-490a-9209-9b45ccf78a2c") },
		{ "44f6ccb9-9dfa-46ba-aea9-a510a85c7f9a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "fd2e6f91-4df9-4055-9b0b-c61df2ad0f1c"}, { new NonTerminator(NonTerminatorType.Factor), "2e01c973-a9fc-40ce-8dc1-df110cdb0e61"}, { Terminator.NumberTerminator, "118d5952-60b4-42d8-9773-dd7522338535"}, { new NonTerminator(NonTerminatorType.Variable), "9451c76a-01e8-4dbf-8a16-f7db028b7adb"}, { new Terminator(DelimiterType.LeftParenthesis), "df9b2921-601d-43d6-8168-546a4d5597fc"}, { Terminator.IdentifierTerminator, "5ddf6fe6-d704-41a4-b552-e817ef6f729f"}, { new Terminator(KeywordType.Not), "5332de79-32c9-4e62-a7de-15ee234934bf"}, { new Terminator(OperatorType.Minus), "39df86b4-3963-4ba3-a04e-047cd10c6a62"}, { new Terminator(OperatorType.Plus), "21d97a2b-d0ff-4cf4-b483-728ef20a436f"}, { new Terminator(KeywordType.True), "9dc09043-74f0-4711-8c01-db46e7a63643"}, { new Terminator(KeywordType.False), "7d2d2ab3-6074-49b5-a491-1114292132cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "44f6ccb9-9dfa-46ba-aea9-a510a85c7f9a") },
		{ "fdf2caf6-3dd1-4f8b-941a-e19e7580eb23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "add3b63d-689f-46b6-8d83-bfb1e4001593"}, { Terminator.NumberTerminator, "118d5952-60b4-42d8-9773-dd7522338535"}, { new NonTerminator(NonTerminatorType.Variable), "9451c76a-01e8-4dbf-8a16-f7db028b7adb"}, { new Terminator(DelimiterType.LeftParenthesis), "df9b2921-601d-43d6-8168-546a4d5597fc"}, { Terminator.IdentifierTerminator, "5ddf6fe6-d704-41a4-b552-e817ef6f729f"}, { new Terminator(KeywordType.Not), "5332de79-32c9-4e62-a7de-15ee234934bf"}, { new Terminator(OperatorType.Minus), "39df86b4-3963-4ba3-a04e-047cd10c6a62"}, { new Terminator(OperatorType.Plus), "21d97a2b-d0ff-4cf4-b483-728ef20a436f"}, { new Terminator(KeywordType.True), "9dc09043-74f0-4711-8c01-db46e7a63643"}, { new Terminator(KeywordType.False), "7d2d2ab3-6074-49b5-a491-1114292132cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fdf2caf6-3dd1-4f8b-941a-e19e7580eb23") },
		{ "62577db7-9b09-4ebb-aa6a-005a5c41d199", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "03f109b3-42fd-423f-8938-b6c41163c408"},}, new Dictionary<Terminator, ReduceInformation>{ }, "62577db7-9b09-4ebb-aa6a-005a5c41d199") },
		{ "051b4a7e-be7f-4037-980e-a9cf7376e8dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7d95a0d5-c2cc-4945-a67c-c4d629ed4234"}, { new NonTerminator(NonTerminatorType.ExpressionList), "fe8f6add-7b05-4aea-873b-c3dfc8e10d91"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "051b4a7e-be7f-4037-980e-a9cf7376e8dc") },
		{ "aaa54aa2-8d3a-4d1f-b79e-a1fd06b06a84", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "aaa54aa2-8d3a-4d1f-b79e-a1fd06b06a84") },
		{ "a08db743-71f6-40db-a4c9-6dca4dde579e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "97b05b5a-b32c-4a99-8b72-e46549a57b8f"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a08db743-71f6-40db-a4c9-6dca4dde579e") },
		{ "e124b53f-21e8-4548-97f7-7ffa3342a588", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e124b53f-21e8-4548-97f7-7ffa3342a588") },
		{ "cda00910-9c85-459a-a5f0-5d2a3733f83a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cda00910-9c85-459a-a5f0-5d2a3733f83a") },
		{ "d7510c7b-ba89-472a-aa05-64172b3508cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "d7510c7b-ba89-472a-aa05-64172b3508cb") },
		{ "bdc9c78e-ca83-49a8-92fb-5b91652f3a90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "bdc9c78e-ca83-49a8-92fb-5b91652f3a90") },
		{ "c5f04330-7d45-41d8-a5fd-8e2039796fec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "c5f04330-7d45-41d8-a5fd-8e2039796fec") },
		{ "f4d15b21-17bb-4fe0-9913-e3346cecb6a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "f4d15b21-17bb-4fe0-9913-e3346cecb6a3") },
		{ "7a966d66-2a5e-4eb5-b37e-c8c50be29dd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "7a966d66-2a5e-4eb5-b37e-c8c50be29dd3") },
		{ "23620ebe-7492-4b56-bf17-d65b352fe28f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "68dee5b2-975f-4b7c-bb58-350a9785b25d"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "154f67bc-cccc-4723-bed9-5e543f6e2cb7"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "23620ebe-7492-4b56-bf17-d65b352fe28f") },
		{ "885a83d0-10be-4ec3-ac23-7485a698a9cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "b0abf1fd-c702-4176-8cd8-fd6f47e0a5d6"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "885a83d0-10be-4ec3-ac23-7485a698a9cb") },
		{ "935cddc7-c502-403e-8611-26ec1d618f1f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "935cddc7-c502-403e-8611-26ec1d618f1f") },
		{ "772ab90f-b3db-44e9-bf32-0020a9eb7ea5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5") },
		{ "9b08c15a-47f3-41ad-89fc-87a70b94d277", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "9b08c15a-47f3-41ad-89fc-87a70b94d277") },
		{ "a8d9ae7e-7768-428a-8b1b-1b666fbb5907", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "bf48ac89-48ce-4252-9b13-53f5e789e03e"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a8d9ae7e-7768-428a-8b1b-1b666fbb5907") },
		{ "2da2fd69-4c2f-4183-8897-8c75052f90ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "388cfd08-a97a-4dc2-8141-6f035bd43e05"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a8dbf801-edce-4c8f-acdf-35b47c8cef20"}, { new Terminator(DelimiterType.LeftSquareBracket), "9abea0ee-91b9-4398-b708-edd6e29c31c1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "2da2fd69-4c2f-4183-8897-8c75052f90ce") },
		{ "df9d412e-5c51-4948-be23-ed680985f2ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3c8aaabb-838f-4e22-b80c-aa33b69c3193"}, { Terminator.NumberTerminator, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5"}, { new NonTerminator(NonTerminatorType.Variable), "9b08c15a-47f3-41ad-89fc-87a70b94d277"}, { new Terminator(DelimiterType.LeftParenthesis), "a8d9ae7e-7768-428a-8b1b-1b666fbb5907"}, { Terminator.IdentifierTerminator, "2da2fd69-4c2f-4183-8897-8c75052f90ce"}, { new Terminator(KeywordType.Not), "df9d412e-5c51-4948-be23-ed680985f2ae"}, { new Terminator(OperatorType.Minus), "fffb31ab-dc7f-4607-b5a8-f29000182972"}, { new Terminator(OperatorType.Plus), "a1794963-b3a9-4188-a1f8-19513de4a213"}, { new Terminator(KeywordType.True), "37b04a17-928f-4c84-86a4-3fc64cee61f2"}, { new Terminator(KeywordType.False), "87d11209-56f5-4141-93a7-4abaee6c3b44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df9d412e-5c51-4948-be23-ed680985f2ae") },
		{ "fffb31ab-dc7f-4607-b5a8-f29000182972", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cafe739b-43a6-4499-85a5-b19e187ae9b7"}, { Terminator.NumberTerminator, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5"}, { new NonTerminator(NonTerminatorType.Variable), "9b08c15a-47f3-41ad-89fc-87a70b94d277"}, { new Terminator(DelimiterType.LeftParenthesis), "a8d9ae7e-7768-428a-8b1b-1b666fbb5907"}, { Terminator.IdentifierTerminator, "2da2fd69-4c2f-4183-8897-8c75052f90ce"}, { new Terminator(KeywordType.Not), "df9d412e-5c51-4948-be23-ed680985f2ae"}, { new Terminator(OperatorType.Minus), "fffb31ab-dc7f-4607-b5a8-f29000182972"}, { new Terminator(OperatorType.Plus), "a1794963-b3a9-4188-a1f8-19513de4a213"}, { new Terminator(KeywordType.True), "37b04a17-928f-4c84-86a4-3fc64cee61f2"}, { new Terminator(KeywordType.False), "87d11209-56f5-4141-93a7-4abaee6c3b44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fffb31ab-dc7f-4607-b5a8-f29000182972") },
		{ "a1794963-b3a9-4188-a1f8-19513de4a213", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "09573f55-3a40-4895-89fd-8f19edb93981"}, { Terminator.NumberTerminator, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5"}, { new NonTerminator(NonTerminatorType.Variable), "9b08c15a-47f3-41ad-89fc-87a70b94d277"}, { new Terminator(DelimiterType.LeftParenthesis), "a8d9ae7e-7768-428a-8b1b-1b666fbb5907"}, { Terminator.IdentifierTerminator, "2da2fd69-4c2f-4183-8897-8c75052f90ce"}, { new Terminator(KeywordType.Not), "df9d412e-5c51-4948-be23-ed680985f2ae"}, { new Terminator(OperatorType.Minus), "fffb31ab-dc7f-4607-b5a8-f29000182972"}, { new Terminator(OperatorType.Plus), "a1794963-b3a9-4188-a1f8-19513de4a213"}, { new Terminator(KeywordType.True), "37b04a17-928f-4c84-86a4-3fc64cee61f2"}, { new Terminator(KeywordType.False), "87d11209-56f5-4141-93a7-4abaee6c3b44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a1794963-b3a9-4188-a1f8-19513de4a213") },
		{ "37b04a17-928f-4c84-86a4-3fc64cee61f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "37b04a17-928f-4c84-86a4-3fc64cee61f2") },
		{ "87d11209-56f5-4141-93a7-4abaee6c3b44", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "87d11209-56f5-4141-93a7-4abaee6c3b44") },
		{ "aa12f399-0fac-48f3-b3a5-ddf8ffc7c55b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "5a18bf8f-db46-4e34-bae4-c1c60c13fb82"}, { new NonTerminator(NonTerminatorType.Variable), "02da0c34-6cbd-42b8-a890-cdd4e7f62670"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "10b72ef2-2d50-4e2e-ad5e-9777f87581fc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "4abb7227-aa91-422b-8c07-5cefec0f972c"}, { new Terminator(KeywordType.If), "c9b058f7-972c-4a63-a2f6-4dd0fbda04e4"}, { new Terminator(KeywordType.For), "05a3f0e8-28df-4577-943e-299acda150d7"}, { Terminator.IdentifierTerminator, "c57c40e0-158e-40bf-ba1d-793d854fc4d2"}, { new Terminator(KeywordType.Begin), "6101ca1c-a6a9-41b4-a112-c00138796288"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "aa12f399-0fac-48f3-b3a5-ddf8ffc7c55b") },
		{ "d79288f9-5914-4b0d-9b87-cdcc5299788b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "73b71fa8-981f-483b-a6a7-05702c206c03"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "fdd4eff1-f432-44c3-95cc-04854e0aa40c"}, { new NonTerminator(NonTerminatorType.Term), "0d02e1d4-d78b-4019-a7bd-6b8c6fe08e37"}, { new NonTerminator(NonTerminatorType.Factor), "26bcc0d0-0146-41ca-a19e-298fccaefbd3"}, { Terminator.NumberTerminator, "6f9ef057-1227-46ff-9bc5-a80ced1c1c52"}, { new NonTerminator(NonTerminatorType.Variable), "420e78b1-4a11-4138-bea6-40e52080d87a"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb5d37d-00ba-4b24-89d5-dfdb9d95cb4c"}, { Terminator.IdentifierTerminator, "0d89aa43-37a4-4968-978a-0803faae50e8"}, { new Terminator(KeywordType.Not), "bf131321-b3af-49c5-8027-86396b6872c4"}, { new Terminator(OperatorType.Minus), "16515a1b-acea-49a9-b7ba-a123773961bc"}, { new Terminator(OperatorType.Plus), "ace0221f-53fa-4b1d-935b-5e47e84c6067"}, { new Terminator(KeywordType.True), "9d833b41-f2b4-4ccd-b794-d7d71f2ea5ce"}, { new Terminator(KeywordType.False), "074d0326-2f3c-4044-a9d2-816242c2d63d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d79288f9-5914-4b0d-9b87-cdcc5299788b") },
		{ "35a8919d-3896-442a-8479-fcaa1a3d9787", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "35a8919d-3896-442a-8479-fcaa1a3d9787") },
		{ "296bbf0a-b93e-4cae-9b20-255b667e827c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3bbf8f29-697e-4eaa-8a04-b4112350403a"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "296bbf0a-b93e-4cae-9b20-255b667e827c") },
		{ "55f2b7d9-8f80-41c0-a650-9fda9cd44433", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "55f2b7d9-8f80-41c0-a650-9fda9cd44433") },
		{ "d9fdd74b-1ba6-46f2-bc36-060354a88fbe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "50c1a67d-c4d7-446f-a505-a5dfb43f5988"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d9fdd74b-1ba6-46f2-bc36-060354a88fbe") },
		{ "877c7a73-5b59-4177-ace7-2c573d97f8f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "877c7a73-5b59-4177-ace7-2c573d97f8f5") },
		{ "2d9f1790-8c00-4dc9-98de-106aba93f238", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "2d9f1790-8c00-4dc9-98de-106aba93f238") },
		{ "02633a30-0a57-4601-acba-ccc074aab324", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "02633a30-0a57-4601-acba-ccc074aab324") },
		{ "ad1d5fb0-3d35-484e-a64f-68dd96ffc055", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "80180d73-6020-4107-ba54-9ce90b56a477"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad1d5fb0-3d35-484e-a64f-68dd96ffc055") },
		{ "fcd89654-b1d0-4449-b0d1-5f0dacb7a415", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "b52abd35-2425-480f-b9d6-c5eb60b6a0b9"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fcd89654-b1d0-4449-b0d1-5f0dacb7a415") },
		{ "ac09e1d0-3a6d-44c3-9253-3676c4bf9423", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "90f4aeab-b24d-428b-af8e-8952df2bec83"}, { new NonTerminator(NonTerminatorType.Factor), "caed1736-e1a9-4a08-a996-38057dc9ef38"}, { Terminator.NumberTerminator, "2d7a0ca8-d47b-47af-96b1-79a244a8162b"}, { new NonTerminator(NonTerminatorType.Variable), "0203b66a-97cf-49e4-b4a5-44a9240daee1"}, { new Terminator(DelimiterType.LeftParenthesis), "72ae4a09-129b-4e98-a17f-98fd3df8f433"}, { Terminator.IdentifierTerminator, "b3d5f8de-1810-4638-a42c-ed6b741212fe"}, { new Terminator(KeywordType.Not), "baa9ab37-d944-4c90-83a1-2d243c2b3aec"}, { new Terminator(OperatorType.Minus), "075b0a32-180c-440c-9b7b-6cb315a3b4b8"}, { new Terminator(OperatorType.Plus), "7236e6ff-062d-4114-a692-c106386308ab"}, { new Terminator(KeywordType.True), "bf93e069-f9ae-4e60-9b14-743260b945e8"}, { new Terminator(KeywordType.False), "1cf66947-0ecc-4413-8902-7bfcfdcf86fd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac09e1d0-3a6d-44c3-9253-3676c4bf9423") },
		{ "f8ab527d-ec4b-4b2d-959c-7311f736f8d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9bab83b3-330d-46c1-9b65-e674eff384c7"}, { Terminator.NumberTerminator, "2d7a0ca8-d47b-47af-96b1-79a244a8162b"}, { new NonTerminator(NonTerminatorType.Variable), "0203b66a-97cf-49e4-b4a5-44a9240daee1"}, { new Terminator(DelimiterType.LeftParenthesis), "72ae4a09-129b-4e98-a17f-98fd3df8f433"}, { Terminator.IdentifierTerminator, "b3d5f8de-1810-4638-a42c-ed6b741212fe"}, { new Terminator(KeywordType.Not), "baa9ab37-d944-4c90-83a1-2d243c2b3aec"}, { new Terminator(OperatorType.Minus), "075b0a32-180c-440c-9b7b-6cb315a3b4b8"}, { new Terminator(OperatorType.Plus), "7236e6ff-062d-4114-a692-c106386308ab"}, { new Terminator(KeywordType.True), "bf93e069-f9ae-4e60-9b14-743260b945e8"}, { new Terminator(KeywordType.False), "1cf66947-0ecc-4413-8902-7bfcfdcf86fd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f8ab527d-ec4b-4b2d-959c-7311f736f8d0") },
		{ "ec32e05c-955a-455f-8b69-2b44e3b27f49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2fab3aca-802f-4605-9ed1-a056f58bf07a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ec32e05c-955a-455f-8b69-2b44e3b27f49") },
		{ "b59c4582-2588-4859-8276-c159572aa60e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "fe97f0e4-51d6-4012-a339-a162c088b19c"}, { new NonTerminator(NonTerminatorType.ExpressionList), "439b6494-97ab-4430-966d-392a1b1e2458"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b59c4582-2588-4859-8276-c159572aa60e") },
		{ "7558e831-e7c3-41ad-8dc6-04aa53c0e6ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "7558e831-e7c3-41ad-8dc6-04aa53c0e6ec") },
		{ "517f9a1e-26b5-48b0-a234-bd254c2a9aee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "8069051d-765b-44cc-9df7-b1f11f6f9b26"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "517f9a1e-26b5-48b0-a234-bd254c2a9aee") },
		{ "557fe2b8-e99b-4367-ac62-3657ec6532db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "557fe2b8-e99b-4367-ac62-3657ec6532db") },
		{ "93e778c9-b993-4168-96d5-fc7e0619e1dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "93e778c9-b993-4168-96d5-fc7e0619e1dc") },
		{ "e9813adf-695f-4d1d-bf52-5d69439eab3c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e9813adf-695f-4d1d-bf52-5d69439eab3c") },
		{ "2dfefd0b-96ac-413e-b04b-b3373f9f0728", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "2dfefd0b-96ac-413e-b04b-b3373f9f0728") },
		{ "d9319c70-461d-4cdb-b7b4-bd77837b2ebd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d9319c70-461d-4cdb-b7b4-bd77837b2ebd") },
		{ "f53b614a-51f4-429d-9f35-6a2883307bec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "9e623a6c-c94d-40bd-a9eb-5515ebb72647"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f53b614a-51f4-429d-9f35-6a2883307bec") },
		{ "00dc0d38-55fe-4e5b-a032-3d01772dd3f9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "60f58790-ad24-4f33-b687-7d342bd39725"}, { new Terminator(OperatorType.Equal), "00f9e6b4-d835-4d58-b32a-08c692f2bfd1"}, { new Terminator(OperatorType.NotEqual), "53511e62-9a4d-468d-89f2-6dcc50713b8d"}, { new Terminator(OperatorType.Less), "9cdd2886-f0fb-4a95-87a6-2538abe9f2b2"}, { new Terminator(OperatorType.LessEqual), "e81e43c0-1fea-4c0d-84e7-9a11a11023a6"}, { new Terminator(OperatorType.Greater), "690c772a-40e6-4ba1-8c96-e78b459a3f8c"}, { new Terminator(OperatorType.GreaterEqual), "50760e65-8acd-473f-928e-e9ecce50bc7e"}, { new NonTerminator(NonTerminatorType.AddOperator), "6d0a0e97-3c0e-40b6-906d-873e9a8ed9b5"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "00dc0d38-55fe-4e5b-a032-3d01772dd3f9") },
		{ "230281f9-66be-4ca7-8818-f4944e726d65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "b4711c5d-b620-4264-afbb-b9c0c922a3e7"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "230281f9-66be-4ca7-8818-f4944e726d65") },
		{ "a1bdce73-48fa-4f14-b8b6-e88691c025ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a1bdce73-48fa-4f14-b8b6-e88691c025ee") },
		{ "ce82ae6d-953c-4760-b620-97870425c3b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ce82ae6d-953c-4760-b620-97870425c3b7") },
		{ "49fed5be-89fa-4dbd-a4bd-4be8060782f1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "49fed5be-89fa-4dbd-a4bd-4be8060782f1") },
		{ "97fe7a58-d5a2-43b3-b1d7-060c35326e0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6f55d194-e42b-4943-9670-499fc862e983"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "97fe7a58-d5a2-43b3-b1d7-060c35326e0f") },
		{ "d079309c-3a64-4602-91f0-e28177e72c5b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "3ba1c4d1-1d72-4e43-9f4f-a34bc91886d9"}, { new NonTerminator(NonTerminatorType.IdVarPart), "ce0b0018-36f3-4d4f-b396-4163b6ea705b"}, { new Terminator(DelimiterType.LeftSquareBracket), "3b547878-78dd-42ba-bba6-7d2691f58b69"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d079309c-3a64-4602-91f0-e28177e72c5b") },
		{ "c3448f1d-e951-474f-b420-12116a60e437", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "01e4e9ee-6744-44dc-98af-dc6a589cd245"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c3448f1d-e951-474f-b420-12116a60e437") },
		{ "2303a490-006b-460c-8feb-c73b02ae0178", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0edb7d4d-6279-4c59-899f-50ae49724ba3"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2303a490-006b-460c-8feb-c73b02ae0178") },
		{ "1a520a45-6d3f-4728-b77e-9e555622a698", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a3b4144f-ea3d-405a-963d-d79ad05ce8a0"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1a520a45-6d3f-4728-b77e-9e555622a698") },
		{ "55849214-2a3d-45bf-a815-bab0331cc12e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "55849214-2a3d-45bf-a815-bab0331cc12e") },
		{ "e11106be-6856-4232-808b-7ee9a70194ac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e11106be-6856-4232-808b-7ee9a70194ac") },
		{ "5a091b64-1a9d-4c29-9fed-b2a24924254e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "05e620e9-aba4-45f2-bda9-5cfe3d8079e7"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "5a091b64-1a9d-4c29-9fed-b2a24924254e") },
		{ "cd0c559d-b2da-4607-ba57-4668bdfe7544", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1b1cc606-8208-4949-8e34-222a622c43c3"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "cd0c559d-b2da-4607-ba57-4668bdfe7544") },
		{ "f87c3693-8813-4e52-897a-219e6b03e98b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "f87c3693-8813-4e52-897a-219e6b03e98b") },
		{ "2c17c7a9-cc9f-4190-aa32-5f119dd36c58", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58") },
		{ "93f2aeb0-544b-4512-a759-4beddb818dab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "93f2aeb0-544b-4512-a759-4beddb818dab") },
		{ "a3f5b2bb-164d-45da-8345-9e903c24c921", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "1ac18d5c-2099-4440-96fc-ae62e4029c0f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a3f5b2bb-164d-45da-8345-9e903c24c921") },
		{ "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "0d81be10-baa9-4598-8c36-2a3f340ff37d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "4835c9ca-e702-4058-aa3f-cc5608b618fc"}, { new Terminator(DelimiterType.LeftSquareBracket), "4a542c74-ff11-4004-bd84-dfa72c63d912"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b") },
		{ "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b4e59573-7751-438d-bf59-b7c51e6b104a"}, { Terminator.NumberTerminator, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58"}, { new NonTerminator(NonTerminatorType.Variable), "93f2aeb0-544b-4512-a759-4beddb818dab"}, { new Terminator(DelimiterType.LeftParenthesis), "a3f5b2bb-164d-45da-8345-9e903c24c921"}, { Terminator.IdentifierTerminator, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b"}, { new Terminator(KeywordType.Not), "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd"}, { new Terminator(OperatorType.Minus), "7215eafb-c0b0-45ca-8077-0ac97db1201e"}, { new Terminator(OperatorType.Plus), "e4a63316-44d2-48a8-91c7-4b2694815e50"}, { new Terminator(KeywordType.True), "2776d0bd-36a9-4559-ab46-da3f863c2562"}, { new Terminator(KeywordType.False), "524cc89a-598b-468c-89dd-01f958884f6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd") },
		{ "7215eafb-c0b0-45ca-8077-0ac97db1201e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fa083b73-39dd-4ab4-9c42-0e06ffd3e2e7"}, { Terminator.NumberTerminator, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58"}, { new NonTerminator(NonTerminatorType.Variable), "93f2aeb0-544b-4512-a759-4beddb818dab"}, { new Terminator(DelimiterType.LeftParenthesis), "a3f5b2bb-164d-45da-8345-9e903c24c921"}, { Terminator.IdentifierTerminator, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b"}, { new Terminator(KeywordType.Not), "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd"}, { new Terminator(OperatorType.Minus), "7215eafb-c0b0-45ca-8077-0ac97db1201e"}, { new Terminator(OperatorType.Plus), "e4a63316-44d2-48a8-91c7-4b2694815e50"}, { new Terminator(KeywordType.True), "2776d0bd-36a9-4559-ab46-da3f863c2562"}, { new Terminator(KeywordType.False), "524cc89a-598b-468c-89dd-01f958884f6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7215eafb-c0b0-45ca-8077-0ac97db1201e") },
		{ "e4a63316-44d2-48a8-91c7-4b2694815e50", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8070e97a-ec01-46e2-8113-9de2191f4985"}, { Terminator.NumberTerminator, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58"}, { new NonTerminator(NonTerminatorType.Variable), "93f2aeb0-544b-4512-a759-4beddb818dab"}, { new Terminator(DelimiterType.LeftParenthesis), "a3f5b2bb-164d-45da-8345-9e903c24c921"}, { Terminator.IdentifierTerminator, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b"}, { new Terminator(KeywordType.Not), "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd"}, { new Terminator(OperatorType.Minus), "7215eafb-c0b0-45ca-8077-0ac97db1201e"}, { new Terminator(OperatorType.Plus), "e4a63316-44d2-48a8-91c7-4b2694815e50"}, { new Terminator(KeywordType.True), "2776d0bd-36a9-4559-ab46-da3f863c2562"}, { new Terminator(KeywordType.False), "524cc89a-598b-468c-89dd-01f958884f6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e4a63316-44d2-48a8-91c7-4b2694815e50") },
		{ "2776d0bd-36a9-4559-ab46-da3f863c2562", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2776d0bd-36a9-4559-ab46-da3f863c2562") },
		{ "524cc89a-598b-468c-89dd-01f958884f6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "524cc89a-598b-468c-89dd-01f958884f6d") },
		{ "4a62591e-a51e-4404-86b4-b3f49d7e97f6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "67e63b46-842a-4cc9-b27f-11b09466b910"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "4a62591e-a51e-4404-86b4-b3f49d7e97f6") },
		{ "ba66ba2f-fc95-4974-b42d-9d60c92e7250", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "ba66ba2f-fc95-4974-b42d-9d60c92e7250") },
		{ "c99374e6-3e4e-43c5-bbfc-fd89b45f3c51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "c99374e6-3e4e-43c5-bbfc-fd89b45f3c51") },
		{ "3061ae66-4e67-4989-8ec7-759cb172d9e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "3061ae66-4e67-4989-8ec7-759cb172d9e0") },
		{ "3280389c-08e6-4fc7-99ec-3cc7b4f18216", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0246c493-c5d3-48d5-be72-5a17a9f2e891"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3280389c-08e6-4fc7-99ec-3cc7b4f18216") },
		{ "87c4b5c8-40b9-4cee-a72b-204a6fb0827b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "07df27d3-2cd0-400d-b06f-f49d7c6be440"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "87c4b5c8-40b9-4cee-a72b-204a6fb0827b") },
		{ "47b8b6fb-5470-4783-a67d-2c2d071c2269", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "e21ea7c7-2e0c-44ac-a6a2-af045a590eaf"}, { new NonTerminator(NonTerminatorType.Factor), "60f20762-8c4c-41dd-bc93-dc8c84db0682"}, { Terminator.NumberTerminator, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2"}, { new NonTerminator(NonTerminatorType.Variable), "f87282db-e5a1-4b5b-b673-d0b76c972837"}, { new Terminator(DelimiterType.LeftParenthesis), "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e"}, { Terminator.IdentifierTerminator, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2"}, { new Terminator(KeywordType.Not), "c79c62fa-c519-4ece-b1bb-eec179b25cb3"}, { new Terminator(OperatorType.Minus), "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87"}, { new Terminator(OperatorType.Plus), "c5db1619-b97d-4ddb-ac28-08eefe9b20ca"}, { new Terminator(KeywordType.True), "c37942f8-2eb1-4cba-8b4a-b60d627d577c"}, { new Terminator(KeywordType.False), "29a323e8-0116-4be6-a550-317cc5e48e9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "47b8b6fb-5470-4783-a67d-2c2d071c2269") },
		{ "0ff969ac-9895-4912-87e8-8cb5ac13ef7c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b92da72b-5693-4ad2-baa0-4b33189c57cb"}, { Terminator.NumberTerminator, "f95b34cd-97a5-4318-acca-b4a22e8dc0d2"}, { new NonTerminator(NonTerminatorType.Variable), "f87282db-e5a1-4b5b-b673-d0b76c972837"}, { new Terminator(DelimiterType.LeftParenthesis), "a6ac256b-b802-42f8-8c00-7c8dcaef0b2e"}, { Terminator.IdentifierTerminator, "f8fe3648-13a1-4035-9dce-f436bdbe3ec2"}, { new Terminator(KeywordType.Not), "c79c62fa-c519-4ece-b1bb-eec179b25cb3"}, { new Terminator(OperatorType.Minus), "1a05fef1-e1dc-433e-9a69-cc4cf4ed9f87"}, { new Terminator(OperatorType.Plus), "c5db1619-b97d-4ddb-ac28-08eefe9b20ca"}, { new Terminator(KeywordType.True), "c37942f8-2eb1-4cba-8b4a-b60d627d577c"}, { new Terminator(KeywordType.False), "29a323e8-0116-4be6-a550-317cc5e48e9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0ff969ac-9895-4912-87e8-8cb5ac13ef7c") },
		{ "4a70b3ba-5435-463d-af99-71129e0978ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "910d00ab-97da-45f2-8b22-5395f3b9f185"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4a70b3ba-5435-463d-af99-71129e0978ef") },
		{ "e3562aec-6d0b-4e86-9143-eaf0ca8d1b56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8ab61044-cf25-4c3d-ad18-1f068dc2f95c"}, { new NonTerminator(NonTerminatorType.ExpressionList), "ab2ba06a-573d-48db-989f-29a888cf5455"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e3562aec-6d0b-4e86-9143-eaf0ca8d1b56") },
		{ "58cab06b-9a08-4caa-be04-88fee59d7cb7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "58cab06b-9a08-4caa-be04-88fee59d7cb7") },
		{ "7e02ad79-173a-4bf6-a6a1-d5169ec8be71", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "b5687e2c-0a19-4125-9ff1-ee7f59935094"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7e02ad79-173a-4bf6-a6a1-d5169ec8be71") },
		{ "46e33e67-fd2c-456e-9399-cd744ca9ad32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "46e33e67-fd2c-456e-9399-cd744ca9ad32") },
		{ "26a48a34-b8af-47f0-b24a-e0b813db1ab1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "26a48a34-b8af-47f0-b24a-e0b813db1ab1") },
		{ "e49ff909-c8e2-426c-81a8-53d3ce2d6883", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e49ff909-c8e2-426c-81a8-53d3ce2d6883") },
		{ "d017c940-5413-484a-a8e0-1c0810a0b616", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "d017c940-5413-484a-a8e0-1c0810a0b616") },
		{ "84d1bb50-b96c-4bcc-b0af-e128c006712a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "84d1bb50-b96c-4bcc-b0af-e128c006712a") },
		{ "dce69fc9-07f6-48ca-b58e-3af71d531b9b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "071edd06-de2b-4c6f-aa6e-a9942b99cbba"}, { new NonTerminator(NonTerminatorType.Factor), "2267b72e-e179-4cec-853d-630a52b9d22b"}, { Terminator.NumberTerminator, "0d4d8133-eb53-49f2-9341-489db84d615b"}, { new NonTerminator(NonTerminatorType.Variable), "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d"}, { new Terminator(DelimiterType.LeftParenthesis), "785f53e2-1ff3-414e-a54b-d53f74bb08ad"}, { Terminator.IdentifierTerminator, "a7b75ed5-af36-43c7-bbde-c5f49995d060"}, { new Terminator(KeywordType.Not), "31658312-8d73-4d47-b03e-c3716a9e250e"}, { new Terminator(OperatorType.Minus), "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db"}, { new Terminator(OperatorType.Plus), "b46278a2-0b18-40f7-9b32-adb4bd69cb78"}, { new Terminator(KeywordType.True), "58ad9ef1-afb6-48e6-9c6b-9695512df704"}, { new Terminator(KeywordType.False), "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dce69fc9-07f6-48ca-b58e-3af71d531b9b") },
		{ "bf0c92a6-b193-45a5-a9e7-82089562d19b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1131003e-9223-457e-a46a-d2f02f15d080"}, { Terminator.NumberTerminator, "0d4d8133-eb53-49f2-9341-489db84d615b"}, { new NonTerminator(NonTerminatorType.Variable), "ce2aa889-c2c4-493d-ab03-eeac4d8e7d0d"}, { new Terminator(DelimiterType.LeftParenthesis), "785f53e2-1ff3-414e-a54b-d53f74bb08ad"}, { Terminator.IdentifierTerminator, "a7b75ed5-af36-43c7-bbde-c5f49995d060"}, { new Terminator(KeywordType.Not), "31658312-8d73-4d47-b03e-c3716a9e250e"}, { new Terminator(OperatorType.Minus), "a5d93d7d-022d-41d9-b4fb-01d1f39ea2db"}, { new Terminator(OperatorType.Plus), "b46278a2-0b18-40f7-9b32-adb4bd69cb78"}, { new Terminator(KeywordType.True), "58ad9ef1-afb6-48e6-9c6b-9695512df704"}, { new Terminator(KeywordType.False), "f9cb9fd8-a394-4b16-bdd4-fe4e6ee8e1e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf0c92a6-b193-45a5-a9e7-82089562d19b") },
		{ "403543b6-9ce7-4fc6-b837-b5e3350c73fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ca47ecd1-9d33-49fd-beea-94a4dfc137cb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "403543b6-9ce7-4fc6-b837-b5e3350c73fc") },
		{ "dac26fb4-0525-4bd1-ae95-d8c90795900f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f1b87422-b6de-4d85-a26e-f9a643d5b44a"}, { new NonTerminator(NonTerminatorType.ExpressionList), "3cf93454-7594-4709-834c-bab06d9ddc2e"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dac26fb4-0525-4bd1-ae95-d8c90795900f") },
		{ "6e70dc7e-ec04-46fc-bbe0-3961d966c82f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "6e70dc7e-ec04-46fc-bbe0-3961d966c82f") },
		{ "b1ef3bd2-f9fd-4b75-b162-739b43040b26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "25e15443-d0b9-4489-a179-053bedb3de9e"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b1ef3bd2-f9fd-4b75-b162-739b43040b26") },
		{ "d6719cba-3358-47a0-806d-b3e3ca912953", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "d6719cba-3358-47a0-806d-b3e3ca912953") },
		{ "cef46389-9d40-4ce3-94c0-f2f8b3635c18", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cef46389-9d40-4ce3-94c0-f2f8b3635c18") },
		{ "4f356464-ae1b-4d1f-a78d-413deba7574e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "4f356464-ae1b-4d1f-a78d-413deba7574e") },
		{ "e84a20f6-cc0b-439f-8256-f90af4b8c77f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e84a20f6-cc0b-439f-8256-f90af4b8c77f") },
		{ "dd782ae4-5aeb-4b84-ae1e-8178918a8213", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "dd782ae4-5aeb-4b84-ae1e-8178918a8213") },
		{ "c0342272-e754-4afb-85c8-1efeaa79434b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "c0342272-e754-4afb-85c8-1efeaa79434b") },
		{ "020cf9fe-62a8-4b45-8def-978f45933306", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "020cf9fe-62a8-4b45-8def-978f45933306") },
		{ "4491d4b7-84b9-4584-acd6-89210c94c275", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "9bcbe78b-4cba-4e09-9f5d-203154171914"}, { Terminator.NumberTerminator, "07caf411-dce0-4707-a4d0-d40f3fe89cae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4491d4b7-84b9-4584-acd6-89210c94c275") },
		{ "0e625463-5cbc-491f-b72f-b944fa3d0214", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "0e625463-5cbc-491f-b72f-b944fa3d0214") },
		{ "fd2e6f91-4df9-4055-9b0b-c61df2ad0f1c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fdf2caf6-3dd1-4f8b-941a-e19e7580eb23"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fd2e6f91-4df9-4055-9b0b-c61df2ad0f1c") },
		{ "add3b63d-689f-46b6-8d83-bfb1e4001593", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "add3b63d-689f-46b6-8d83-bfb1e4001593") },
		{ "03f109b3-42fd-423f-8938-b6c41163c408", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "03f109b3-42fd-423f-8938-b6c41163c408") },
		{ "7d95a0d5-c2cc-4945-a67c-c4d629ed4234", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "7d95a0d5-c2cc-4945-a67c-c4d629ed4234") },
		{ "fe8f6add-7b05-4aea-873b-c3dfc8e10d91", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e97fe690-908c-4d94-9ffc-d5f770dfa2f9"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fe8f6add-7b05-4aea-873b-c3dfc8e10d91") },
		{ "97b05b5a-b32c-4a99-8b72-e46549a57b8f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "9a4e082e-4e3b-4aca-8c25-21f444c28e59"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "97b05b5a-b32c-4a99-8b72-e46549a57b8f") },
		{ "68dee5b2-975f-4b7c-bb58-350a9785b25d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "3712148b-11eb-4bdd-b596-f03b1ce52523"}, { new NonTerminator(NonTerminatorType.Term), "4203efb0-6fef-41ef-9747-5bd3db80bc8e"}, { new NonTerminator(NonTerminatorType.Factor), "6e04f9ad-ddf6-4dbc-95c6-d9f74176d822"}, { Terminator.NumberTerminator, "1be01006-2f44-4e8d-97ab-75ca7ca56c58"}, { new NonTerminator(NonTerminatorType.Variable), "a7ba29ce-27b6-4fca-931f-5020c6c27046"}, { new Terminator(DelimiterType.LeftParenthesis), "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2"}, { Terminator.IdentifierTerminator, "b4cc1630-8532-470b-bda4-758795dbae90"}, { new Terminator(KeywordType.Not), "4ec65aff-034b-47b2-a50c-bba5d2a000c0"}, { new Terminator(OperatorType.Minus), "70fd34e2-8883-485e-a64c-f28e4126247f"}, { new Terminator(OperatorType.Plus), "9d2de763-4c51-4609-b174-240c92c8cb47"}, { new Terminator(KeywordType.True), "8349d5b7-2252-438a-8101-a6a3f7409ca4"}, { new Terminator(KeywordType.False), "57f25122-b6dd-4051-8886-5e886abf7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "68dee5b2-975f-4b7c-bb58-350a9785b25d") },
		{ "154f67bc-cccc-4723-bed9-5e543f6e2cb7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "98399ba7-7e1f-4167-8277-1f7574a9e692"}, { new NonTerminator(NonTerminatorType.Factor), "935cddc7-c502-403e-8611-26ec1d618f1f"}, { Terminator.NumberTerminator, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5"}, { new NonTerminator(NonTerminatorType.Variable), "9b08c15a-47f3-41ad-89fc-87a70b94d277"}, { new Terminator(DelimiterType.LeftParenthesis), "a8d9ae7e-7768-428a-8b1b-1b666fbb5907"}, { Terminator.IdentifierTerminator, "2da2fd69-4c2f-4183-8897-8c75052f90ce"}, { new Terminator(KeywordType.Not), "df9d412e-5c51-4948-be23-ed680985f2ae"}, { new Terminator(OperatorType.Minus), "fffb31ab-dc7f-4607-b5a8-f29000182972"}, { new Terminator(OperatorType.Plus), "a1794963-b3a9-4188-a1f8-19513de4a213"}, { new Terminator(KeywordType.True), "37b04a17-928f-4c84-86a4-3fc64cee61f2"}, { new Terminator(KeywordType.False), "87d11209-56f5-4141-93a7-4abaee6c3b44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "154f67bc-cccc-4723-bed9-5e543f6e2cb7") },
		{ "b0abf1fd-c702-4176-8cd8-fd6f47e0a5d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b96fc979-3045-4ff4-9882-16525ac3df9e"}, { Terminator.NumberTerminator, "772ab90f-b3db-44e9-bf32-0020a9eb7ea5"}, { new NonTerminator(NonTerminatorType.Variable), "9b08c15a-47f3-41ad-89fc-87a70b94d277"}, { new Terminator(DelimiterType.LeftParenthesis), "a8d9ae7e-7768-428a-8b1b-1b666fbb5907"}, { Terminator.IdentifierTerminator, "2da2fd69-4c2f-4183-8897-8c75052f90ce"}, { new Terminator(KeywordType.Not), "df9d412e-5c51-4948-be23-ed680985f2ae"}, { new Terminator(OperatorType.Minus), "fffb31ab-dc7f-4607-b5a8-f29000182972"}, { new Terminator(OperatorType.Plus), "a1794963-b3a9-4188-a1f8-19513de4a213"}, { new Terminator(KeywordType.True), "37b04a17-928f-4c84-86a4-3fc64cee61f2"}, { new Terminator(KeywordType.False), "87d11209-56f5-4141-93a7-4abaee6c3b44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b0abf1fd-c702-4176-8cd8-fd6f47e0a5d6") },
		{ "bf48ac89-48ce-4252-9b13-53f5e789e03e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d3e0889d-af27-4e62-9356-7ebe76f9c5b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf48ac89-48ce-4252-9b13-53f5e789e03e") },
		{ "388cfd08-a97a-4dc2-8141-6f035bd43e05", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d71bb00b-a6e1-4cc1-b12a-20bbe428f2c2"}, { new NonTerminator(NonTerminatorType.ExpressionList), "da1071b0-388c-4ef6-ad28-ab6378e694e0"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "388cfd08-a97a-4dc2-8141-6f035bd43e05") },
		{ "a8dbf801-edce-4c8f-acdf-35b47c8cef20", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "a8dbf801-edce-4c8f-acdf-35b47c8cef20") },
		{ "9abea0ee-91b9-4398-b708-edd6e29c31c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "d91ada7e-071f-4520-996a-857581730d17"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9abea0ee-91b9-4398-b708-edd6e29c31c1") },
		{ "3c8aaabb-838f-4e22-b80c-aa33b69c3193", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3c8aaabb-838f-4e22-b80c-aa33b69c3193") },
		{ "cafe739b-43a6-4499-85a5-b19e187ae9b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cafe739b-43a6-4499-85a5-b19e187ae9b7") },
		{ "09573f55-3a40-4895-89fd-8f19edb93981", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "09573f55-3a40-4895-89fd-8f19edb93981") },
		{ "5a18bf8f-db46-4e34-bae4-c1c60c13fb82", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "232bb6eb-fec1-48bc-ad8a-17820f3c5488"}, { new Terminator(KeywordType.Else), "68339181-e2fd-4b8b-bc0e-11dabcfe83d3"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "5a18bf8f-db46-4e34-bae4-c1c60c13fb82") },
		{ "73b71fa8-981f-483b-a6a7-05702c206c03", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "2869bb1d-1f12-4455-a67e-b8b28a4fa9e8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73b71fa8-981f-483b-a6a7-05702c206c03") },
		{ "3bbf8f29-697e-4eaa-8a04-b4112350403a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "3bbf8f29-697e-4eaa-8a04-b4112350403a") },
		{ "80180d73-6020-4107-ba54-9ce90b56a477", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "80180d73-6020-4107-ba54-9ce90b56a477") },
		{ "b52abd35-2425-480f-b9d6-c5eb60b6a0b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b52abd35-2425-480f-b9d6-c5eb60b6a0b9") },
		{ "90f4aeab-b24d-428b-af8e-8952df2bec83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "f8ab527d-ec4b-4b2d-959c-7311f736f8d0"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "90f4aeab-b24d-428b-af8e-8952df2bec83") },
		{ "9bab83b3-330d-46c1-9b65-e674eff384c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "9bab83b3-330d-46c1-9b65-e674eff384c7") },
		{ "2fab3aca-802f-4605-9ed1-a056f58bf07a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "2fab3aca-802f-4605-9ed1-a056f58bf07a") },
		{ "fe97f0e4-51d6-4012-a339-a162c088b19c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "fe97f0e4-51d6-4012-a339-a162c088b19c") },
		{ "439b6494-97ab-4430-966d-392a1b1e2458", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b3ebd13f-bdca-407b-b6fd-da6060e8a026"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "439b6494-97ab-4430-966d-392a1b1e2458") },
		{ "8069051d-765b-44cc-9df7-b1f11f6f9b26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "da133378-83a8-4eef-80ee-67297641b9b3"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8069051d-765b-44cc-9df7-b1f11f6f9b26") },
		{ "9e623a6c-c94d-40bd-a9eb-5515ebb72647", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "069899d6-c8d4-4340-9f5e-8e335220982a"}, { new NonTerminator(NonTerminatorType.Variable), "eb235d1e-80d3-4d8c-ad8c-3fe064787fcb"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "70a9a068-00ac-4fda-afd9-8420a497c944"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "5e1696fb-becd-4945-a48e-278473b63380"}, { new Terminator(KeywordType.If), "5f5cbf98-1028-45dd-99b3-fb92a2c3558f"}, { new Terminator(KeywordType.For), "8d0836c1-fa55-4101-9a86-33aa16e2810b"}, { Terminator.IdentifierTerminator, "b4e2291a-278c-4d06-a8db-b307d1fa6694"}, { new Terminator(KeywordType.Begin), "090bd9b1-fe24-47e8-a6a0-ba484add5b2d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "9e623a6c-c94d-40bd-a9eb-5515ebb72647") },
		{ "60f58790-ad24-4f33-b687-7d342bd39725", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "903b7ee7-018d-4aba-befe-bc7e5a0f3c43"}, { new NonTerminator(NonTerminatorType.Term), "db2cbe8f-6e06-46e5-89e5-957b14a4a2ab"}, { new NonTerminator(NonTerminatorType.Factor), "5b6fd525-664b-4b84-a281-a0e3e94112f5"}, { Terminator.NumberTerminator, "0d3b5204-314f-414c-adff-857477889392"}, { new NonTerminator(NonTerminatorType.Variable), "d54a0897-bdc5-4d3d-b42e-404557419658"}, { new Terminator(DelimiterType.LeftParenthesis), "09065ad9-dc24-4477-a84b-bfc8bd2d6846"}, { Terminator.IdentifierTerminator, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8"}, { new Terminator(KeywordType.Not), "bf895578-c816-4803-8690-36199ce06d1a"}, { new Terminator(OperatorType.Minus), "50d92e97-418e-4dcc-8a0b-0417be94f780"}, { new Terminator(OperatorType.Plus), "c0cb164f-4a72-4001-968c-819a880d7325"}, { new Terminator(KeywordType.True), "913fa60f-50c8-4d05-91f5-5b70ac8d1897"}, { new Terminator(KeywordType.False), "c4f48802-cd68-4fe3-bc32-00801162e89d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "60f58790-ad24-4f33-b687-7d342bd39725") },
		{ "6d0a0e97-3c0e-40b6-906d-873e9a8ed9b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "99cf5aee-d2d7-415c-ae4d-462e014d2906"}, { new NonTerminator(NonTerminatorType.Factor), "a1bdce73-48fa-4f14-b8b6-e88691c025ee"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6d0a0e97-3c0e-40b6-906d-873e9a8ed9b5") },
		{ "b4711c5d-b620-4264-afbb-b9c0c922a3e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b31a52f2-01e2-4fa8-afbc-8e8e8905a826"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b4711c5d-b620-4264-afbb-b9c0c922a3e7") },
		{ "6f55d194-e42b-4943-9670-499fc862e983", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "eb37b301-f1c6-4e88-88da-64a2ef2cf3cf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6f55d194-e42b-4943-9670-499fc862e983") },
		{ "3ba1c4d1-1d72-4e43-9f4f-a34bc91886d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b228072e-f8ae-4d54-b451-29af3cc36d20"}, { new NonTerminator(NonTerminatorType.ExpressionList), "fa802866-a329-4298-be36-d2a1934c570c"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3ba1c4d1-1d72-4e43-9f4f-a34bc91886d9") },
		{ "ce0b0018-36f3-4d4f-b396-4163b6ea705b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "ce0b0018-36f3-4d4f-b396-4163b6ea705b") },
		{ "3b547878-78dd-42ba-bba6-7d2691f58b69", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9dc9f1da-47c5-40a9-b21b-de605e5ba8a7"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3b547878-78dd-42ba-bba6-7d2691f58b69") },
		{ "01e4e9ee-6744-44dc-98af-dc6a589cd245", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "01e4e9ee-6744-44dc-98af-dc6a589cd245") },
		{ "0edb7d4d-6279-4c59-899f-50ae49724ba3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0edb7d4d-6279-4c59-899f-50ae49724ba3") },
		{ "a3b4144f-ea3d-405a-963d-d79ad05ce8a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "a3b4144f-ea3d-405a-963d-d79ad05ce8a0") },
		{ "05e620e9-aba4-45f2-bda9-5cfe3d8079e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "9696a74f-961e-44e5-82b5-28a41b1da496"}, { new NonTerminator(NonTerminatorType.Factor), "f87c3693-8813-4e52-897a-219e6b03e98b"}, { Terminator.NumberTerminator, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58"}, { new NonTerminator(NonTerminatorType.Variable), "93f2aeb0-544b-4512-a759-4beddb818dab"}, { new Terminator(DelimiterType.LeftParenthesis), "a3f5b2bb-164d-45da-8345-9e903c24c921"}, { Terminator.IdentifierTerminator, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b"}, { new Terminator(KeywordType.Not), "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd"}, { new Terminator(OperatorType.Minus), "7215eafb-c0b0-45ca-8077-0ac97db1201e"}, { new Terminator(OperatorType.Plus), "e4a63316-44d2-48a8-91c7-4b2694815e50"}, { new Terminator(KeywordType.True), "2776d0bd-36a9-4559-ab46-da3f863c2562"}, { new Terminator(KeywordType.False), "524cc89a-598b-468c-89dd-01f958884f6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "05e620e9-aba4-45f2-bda9-5cfe3d8079e7") },
		{ "1b1cc606-8208-4949-8e34-222a622c43c3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b6cb8a23-1706-4bd3-92d1-0406492297d7"}, { Terminator.NumberTerminator, "2c17c7a9-cc9f-4190-aa32-5f119dd36c58"}, { new NonTerminator(NonTerminatorType.Variable), "93f2aeb0-544b-4512-a759-4beddb818dab"}, { new Terminator(DelimiterType.LeftParenthesis), "a3f5b2bb-164d-45da-8345-9e903c24c921"}, { Terminator.IdentifierTerminator, "3b8e1200-3889-4aed-95f4-ae4fc78b1f3b"}, { new Terminator(KeywordType.Not), "ccfde1cf-3f1b-41f5-8ee3-86ecbdf030bd"}, { new Terminator(OperatorType.Minus), "7215eafb-c0b0-45ca-8077-0ac97db1201e"}, { new Terminator(OperatorType.Plus), "e4a63316-44d2-48a8-91c7-4b2694815e50"}, { new Terminator(KeywordType.True), "2776d0bd-36a9-4559-ab46-da3f863c2562"}, { new Terminator(KeywordType.False), "524cc89a-598b-468c-89dd-01f958884f6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1b1cc606-8208-4949-8e34-222a622c43c3") },
		{ "1ac18d5c-2099-4440-96fc-ae62e4029c0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "77069d94-e311-4e01-98a5-002aa94ee79e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1ac18d5c-2099-4440-96fc-ae62e4029c0f") },
		{ "0d81be10-baa9-4598-8c36-2a3f340ff37d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f9911432-da07-47ec-9c36-0b9854c4f2f5"}, { new NonTerminator(NonTerminatorType.ExpressionList), "5e179093-4c37-46f9-b0de-b2c56dba9bcc"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0d81be10-baa9-4598-8c36-2a3f340ff37d") },
		{ "4835c9ca-e702-4058-aa3f-cc5608b618fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "4835c9ca-e702-4058-aa3f-cc5608b618fc") },
		{ "4a542c74-ff11-4004-bd84-dfa72c63d912", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "75390dae-a0c4-450d-9947-cee2e9eac9bb"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4a542c74-ff11-4004-bd84-dfa72c63d912") },
		{ "b4e59573-7751-438d-bf59-b7c51e6b104a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "b4e59573-7751-438d-bf59-b7c51e6b104a") },
		{ "fa083b73-39dd-4ab4-9c42-0e06ffd3e2e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "fa083b73-39dd-4ab4-9c42-0e06ffd3e2e7") },
		{ "8070e97a-ec01-46e2-8113-9de2191f4985", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "8070e97a-ec01-46e2-8113-9de2191f4985") },
		{ "0246c493-c5d3-48d5-be72-5a17a9f2e891", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "0246c493-c5d3-48d5-be72-5a17a9f2e891") },
		{ "07df27d3-2cd0-400d-b06f-f49d7c6be440", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "07df27d3-2cd0-400d-b06f-f49d7c6be440") },
		{ "e21ea7c7-2e0c-44ac-a6a2-af045a590eaf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "0ff969ac-9895-4912-87e8-8cb5ac13ef7c"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e21ea7c7-2e0c-44ac-a6a2-af045a590eaf") },
		{ "b92da72b-5693-4ad2-baa0-4b33189c57cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "b92da72b-5693-4ad2-baa0-4b33189c57cb") },
		{ "910d00ab-97da-45f2-8b22-5395f3b9f185", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "910d00ab-97da-45f2-8b22-5395f3b9f185") },
		{ "8ab61044-cf25-4c3d-ad18-1f068dc2f95c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "8ab61044-cf25-4c3d-ad18-1f068dc2f95c") },
		{ "ab2ba06a-573d-48db-989f-29a888cf5455", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "93be189a-2064-4d1d-b49c-73509eca1398"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ab2ba06a-573d-48db-989f-29a888cf5455") },
		{ "b5687e2c-0a19-4125-9ff1-ee7f59935094", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1898a929-8341-45b5-a450-45fad252fe00"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b5687e2c-0a19-4125-9ff1-ee7f59935094") },
		{ "071edd06-de2b-4c6f-aa6e-a9942b99cbba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "bf0c92a6-b193-45a5-a9e7-82089562d19b"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "071edd06-de2b-4c6f-aa6e-a9942b99cbba") },
		{ "1131003e-9223-457e-a46a-d2f02f15d080", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "1131003e-9223-457e-a46a-d2f02f15d080") },
		{ "ca47ecd1-9d33-49fd-beea-94a4dfc137cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "ca47ecd1-9d33-49fd-beea-94a4dfc137cb") },
		{ "f1b87422-b6de-4d85-a26e-f9a643d5b44a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "f1b87422-b6de-4d85-a26e-f9a643d5b44a") },
		{ "3cf93454-7594-4709-834c-bab06d9ddc2e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2394c1c3-e09e-411b-967d-49e899858b63"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3cf93454-7594-4709-834c-bab06d9ddc2e") },
		{ "25e15443-d0b9-4489-a179-053bedb3de9e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "c78c2b19-3353-4170-bacd-94613091bd28"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "25e15443-d0b9-4489-a179-053bedb3de9e") },
		{ "9bcbe78b-4cba-4e09-9f5d-203154171914", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "bce60ed5-192a-4eb0-8ff8-6d0f01727d9c"}, { new Terminator(DelimiterType.Comma), "9a62e1cc-7cdb-4336-94b3-720faa11196b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9bcbe78b-4cba-4e09-9f5d-203154171914") },
		{ "e97fe690-908c-4d94-9ffc-d5f770dfa2f9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e97fe690-908c-4d94-9ffc-d5f770dfa2f9") },
		{ "9a4e082e-4e3b-4aca-8c25-21f444c28e59", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "9a4e082e-4e3b-4aca-8c25-21f444c28e59") },
		{ "3712148b-11eb-4bdd-b596-f03b1ce52523", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "63ab766c-87fb-4de4-93f5-d1bd7b84f62d"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "3712148b-11eb-4bdd-b596-f03b1ce52523") },
		{ "4203efb0-6fef-41ef-9747-5bd3db80bc8e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "3fbcca5a-0862-4697-9ded-bd9623639886"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "4203efb0-6fef-41ef-9747-5bd3db80bc8e") },
		{ "6e04f9ad-ddf6-4dbc-95c6-d9f74176d822", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "6e04f9ad-ddf6-4dbc-95c6-d9f74176d822") },
		{ "1be01006-2f44-4e8d-97ab-75ca7ca56c58", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1be01006-2f44-4e8d-97ab-75ca7ca56c58") },
		{ "a7ba29ce-27b6-4fca-931f-5020c6c27046", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a7ba29ce-27b6-4fca-931f-5020c6c27046") },
		{ "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "90876872-747a-439a-b3ff-def962c2d693"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2") },
		{ "b4cc1630-8532-470b-bda4-758795dbae90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "5350b6f1-e47d-4158-bcf9-f7c6a3c29fe3"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c1fb4fca-dca1-4456-9305-48576a6e2fd4"}, { new Terminator(DelimiterType.LeftSquareBracket), "cf33cbf3-072c-49f3-a3a9-00b65ef7b19e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b4cc1630-8532-470b-bda4-758795dbae90") },
		{ "4ec65aff-034b-47b2-a50c-bba5d2a000c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "879e84e7-daf9-4329-a78f-c609f45fc65c"}, { Terminator.NumberTerminator, "1be01006-2f44-4e8d-97ab-75ca7ca56c58"}, { new NonTerminator(NonTerminatorType.Variable), "a7ba29ce-27b6-4fca-931f-5020c6c27046"}, { new Terminator(DelimiterType.LeftParenthesis), "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2"}, { Terminator.IdentifierTerminator, "b4cc1630-8532-470b-bda4-758795dbae90"}, { new Terminator(KeywordType.Not), "4ec65aff-034b-47b2-a50c-bba5d2a000c0"}, { new Terminator(OperatorType.Minus), "70fd34e2-8883-485e-a64c-f28e4126247f"}, { new Terminator(OperatorType.Plus), "9d2de763-4c51-4609-b174-240c92c8cb47"}, { new Terminator(KeywordType.True), "8349d5b7-2252-438a-8101-a6a3f7409ca4"}, { new Terminator(KeywordType.False), "57f25122-b6dd-4051-8886-5e886abf7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4ec65aff-034b-47b2-a50c-bba5d2a000c0") },
		{ "70fd34e2-8883-485e-a64c-f28e4126247f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "27ce5e63-ff9a-4899-b9a5-5934ec0c93d6"}, { Terminator.NumberTerminator, "1be01006-2f44-4e8d-97ab-75ca7ca56c58"}, { new NonTerminator(NonTerminatorType.Variable), "a7ba29ce-27b6-4fca-931f-5020c6c27046"}, { new Terminator(DelimiterType.LeftParenthesis), "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2"}, { Terminator.IdentifierTerminator, "b4cc1630-8532-470b-bda4-758795dbae90"}, { new Terminator(KeywordType.Not), "4ec65aff-034b-47b2-a50c-bba5d2a000c0"}, { new Terminator(OperatorType.Minus), "70fd34e2-8883-485e-a64c-f28e4126247f"}, { new Terminator(OperatorType.Plus), "9d2de763-4c51-4609-b174-240c92c8cb47"}, { new Terminator(KeywordType.True), "8349d5b7-2252-438a-8101-a6a3f7409ca4"}, { new Terminator(KeywordType.False), "57f25122-b6dd-4051-8886-5e886abf7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "70fd34e2-8883-485e-a64c-f28e4126247f") },
		{ "9d2de763-4c51-4609-b174-240c92c8cb47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e1d9e863-493d-4821-a7a9-dddff8f4515e"}, { Terminator.NumberTerminator, "1be01006-2f44-4e8d-97ab-75ca7ca56c58"}, { new NonTerminator(NonTerminatorType.Variable), "a7ba29ce-27b6-4fca-931f-5020c6c27046"}, { new Terminator(DelimiterType.LeftParenthesis), "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2"}, { Terminator.IdentifierTerminator, "b4cc1630-8532-470b-bda4-758795dbae90"}, { new Terminator(KeywordType.Not), "4ec65aff-034b-47b2-a50c-bba5d2a000c0"}, { new Terminator(OperatorType.Minus), "70fd34e2-8883-485e-a64c-f28e4126247f"}, { new Terminator(OperatorType.Plus), "9d2de763-4c51-4609-b174-240c92c8cb47"}, { new Terminator(KeywordType.True), "8349d5b7-2252-438a-8101-a6a3f7409ca4"}, { new Terminator(KeywordType.False), "57f25122-b6dd-4051-8886-5e886abf7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9d2de763-4c51-4609-b174-240c92c8cb47") },
		{ "8349d5b7-2252-438a-8101-a6a3f7409ca4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "8349d5b7-2252-438a-8101-a6a3f7409ca4") },
		{ "57f25122-b6dd-4051-8886-5e886abf7f0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "57f25122-b6dd-4051-8886-5e886abf7f0a") },
		{ "98399ba7-7e1f-4167-8277-1f7574a9e692", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "b0abf1fd-c702-4176-8cd8-fd6f47e0a5d6"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "98399ba7-7e1f-4167-8277-1f7574a9e692") },
		{ "b96fc979-3045-4ff4-9882-16525ac3df9e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "b96fc979-3045-4ff4-9882-16525ac3df9e") },
		{ "d3e0889d-af27-4e62-9356-7ebe76f9c5b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d3e0889d-af27-4e62-9356-7ebe76f9c5b1") },
		{ "d71bb00b-a6e1-4cc1-b12a-20bbe428f2c2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d71bb00b-a6e1-4cc1-b12a-20bbe428f2c2") },
		{ "da1071b0-388c-4ef6-ad28-ab6378e694e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "dc3117b5-f322-4887-88ae-7e2704de97ca"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "da1071b0-388c-4ef6-ad28-ab6378e694e0") },
		{ "d91ada7e-071f-4520-996a-857581730d17", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5fcfb5bc-42b8-4581-bafc-5bbf53cc725b"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d91ada7e-071f-4520-996a-857581730d17") },
		{ "232bb6eb-fec1-48bc-ad8a-17820f3c5488", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "232bb6eb-fec1-48bc-ad8a-17820f3c5488") },
		{ "68339181-e2fd-4b8b-bc0e-11dabcfe83d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "e2fcccd1-d7ce-4eed-84ec-722c0ca0fcfc"}, { new NonTerminator(NonTerminatorType.Variable), "02da0c34-6cbd-42b8-a890-cdd4e7f62670"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "10b72ef2-2d50-4e2e-ad5e-9777f87581fc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "4abb7227-aa91-422b-8c07-5cefec0f972c"}, { new Terminator(KeywordType.If), "c9b058f7-972c-4a63-a2f6-4dd0fbda04e4"}, { new Terminator(KeywordType.For), "05a3f0e8-28df-4577-943e-299acda150d7"}, { Terminator.IdentifierTerminator, "c57c40e0-158e-40bf-ba1d-793d854fc4d2"}, { new Terminator(KeywordType.Begin), "6101ca1c-a6a9-41b4-a112-c00138796288"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "68339181-e2fd-4b8b-bc0e-11dabcfe83d3") },
		{ "2869bb1d-1f12-4455-a67e-b8b28a4fa9e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ac432171-fe62-48be-99b8-b82c8d9d8f49"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "00dc0d38-55fe-4e5b-a032-3d01772dd3f9"}, { new NonTerminator(NonTerminatorType.Term), "230281f9-66be-4ca7-8818-f4944e726d65"}, { new NonTerminator(NonTerminatorType.Factor), "a1bdce73-48fa-4f14-b8b6-e88691c025ee"}, { Terminator.NumberTerminator, "ce82ae6d-953c-4760-b620-97870425c3b7"}, { new NonTerminator(NonTerminatorType.Variable), "49fed5be-89fa-4dbd-a4bd-4be8060782f1"}, { new Terminator(DelimiterType.LeftParenthesis), "97fe7a58-d5a2-43b3-b1d7-060c35326e0f"}, { Terminator.IdentifierTerminator, "d079309c-3a64-4602-91f0-e28177e72c5b"}, { new Terminator(KeywordType.Not), "c3448f1d-e951-474f-b420-12116a60e437"}, { new Terminator(OperatorType.Minus), "2303a490-006b-460c-8feb-c73b02ae0178"}, { new Terminator(OperatorType.Plus), "1a520a45-6d3f-4728-b77e-9e555622a698"}, { new Terminator(KeywordType.True), "55849214-2a3d-45bf-a815-bab0331cc12e"}, { new Terminator(KeywordType.False), "e11106be-6856-4232-808b-7ee9a70194ac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2869bb1d-1f12-4455-a67e-b8b28a4fa9e8") },
		{ "b3ebd13f-bdca-407b-b6fd-da6060e8a026", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "b3ebd13f-bdca-407b-b6fd-da6060e8a026") },
		{ "da133378-83a8-4eef-80ee-67297641b9b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "da133378-83a8-4eef-80ee-67297641b9b3") },
		{ "069899d6-c8d4-4340-9f5e-8e335220982a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "069899d6-c8d4-4340-9f5e-8e335220982a") },
		{ "903b7ee7-018d-4aba-befe-bc7e5a0f3c43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "d55fd873-6fb4-416a-baa5-11e00fd3c6ff"}, { new Terminator(OperatorType.Plus), "ec492a38-9608-4c47-b622-384820e1e7b8"}, { new Terminator(OperatorType.Minus), "8aee6cb0-9499-46fa-bd1d-354d7710722d"}, { new Terminator(KeywordType.Or), "f2449077-15a8-451d-8e69-f734c89a16ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "903b7ee7-018d-4aba-befe-bc7e5a0f3c43") },
		{ "db2cbe8f-6e06-46e5-89e5-957b14a4a2ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2175173b-f979-413c-8da7-c50ac7121f68"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "db2cbe8f-6e06-46e5-89e5-957b14a4a2ab") },
		{ "5b6fd525-664b-4b84-a281-a0e3e94112f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "5b6fd525-664b-4b84-a281-a0e3e94112f5") },
		{ "0d3b5204-314f-414c-adff-857477889392", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0d3b5204-314f-414c-adff-857477889392") },
		{ "d54a0897-bdc5-4d3d-b42e-404557419658", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d54a0897-bdc5-4d3d-b42e-404557419658") },
		{ "09065ad9-dc24-4477-a84b-bfc8bd2d6846", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "86f8719c-dd82-4a75-834e-d0dd01617281"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ab682e97-3b78-4d58-a255-118650e13d96"}, { new NonTerminator(NonTerminatorType.Term), "611cfdfb-5856-4399-80d2-095291ebb774"}, { new NonTerminator(NonTerminatorType.Factor), "95b52b9a-255c-47d1-b6b0-636ef908452e"}, { Terminator.NumberTerminator, "3d50648a-761a-4c45-add9-026f9631c92d"}, { new NonTerminator(NonTerminatorType.Variable), "50eb06a8-d7a4-4d5c-9a01-8841b61df175"}, { new Terminator(DelimiterType.LeftParenthesis), "7929fa15-03d5-4eb6-84c2-eba74372cf0f"}, { Terminator.IdentifierTerminator, "e8e3cd92-00c6-4331-b572-7d900e34d3fe"}, { new Terminator(KeywordType.Not), "e77e5574-028a-4eb0-9cfd-23b040ac8e58"}, { new Terminator(OperatorType.Minus), "322c45a4-8a7b-4f0c-b684-3c705ea1166f"}, { new Terminator(OperatorType.Plus), "6b4c5e8e-245b-4018-8fe2-deb6646e8dcc"}, { new Terminator(KeywordType.True), "8a5210d5-c688-431b-a671-85ba652c83b5"}, { new Terminator(KeywordType.False), "0a01f9a5-406b-4299-92c7-88a7d2ece198"},}, new Dictionary<Terminator, ReduceInformation>{ }, "09065ad9-dc24-4477-a84b-bfc8bd2d6846") },
		{ "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "8f110252-27ff-41bd-926f-a0aeb3cac968"}, { new NonTerminator(NonTerminatorType.IdVarPart), "bbfc0c11-9ff3-41eb-8014-cf919907a1b8"}, { new Terminator(DelimiterType.LeftSquareBracket), "cac807ed-93d5-470c-9bfb-b38584252c57"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8") },
		{ "bf895578-c816-4803-8690-36199ce06d1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c85af13d-5ae3-4c2f-8de4-6311ce0f897c"}, { Terminator.NumberTerminator, "0d3b5204-314f-414c-adff-857477889392"}, { new NonTerminator(NonTerminatorType.Variable), "d54a0897-bdc5-4d3d-b42e-404557419658"}, { new Terminator(DelimiterType.LeftParenthesis), "09065ad9-dc24-4477-a84b-bfc8bd2d6846"}, { Terminator.IdentifierTerminator, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8"}, { new Terminator(KeywordType.Not), "bf895578-c816-4803-8690-36199ce06d1a"}, { new Terminator(OperatorType.Minus), "50d92e97-418e-4dcc-8a0b-0417be94f780"}, { new Terminator(OperatorType.Plus), "c0cb164f-4a72-4001-968c-819a880d7325"}, { new Terminator(KeywordType.True), "913fa60f-50c8-4d05-91f5-5b70ac8d1897"}, { new Terminator(KeywordType.False), "c4f48802-cd68-4fe3-bc32-00801162e89d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf895578-c816-4803-8690-36199ce06d1a") },
		{ "50d92e97-418e-4dcc-8a0b-0417be94f780", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "333a4eb3-0494-4eb3-981a-e5f2e82bde13"}, { Terminator.NumberTerminator, "0d3b5204-314f-414c-adff-857477889392"}, { new NonTerminator(NonTerminatorType.Variable), "d54a0897-bdc5-4d3d-b42e-404557419658"}, { new Terminator(DelimiterType.LeftParenthesis), "09065ad9-dc24-4477-a84b-bfc8bd2d6846"}, { Terminator.IdentifierTerminator, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8"}, { new Terminator(KeywordType.Not), "bf895578-c816-4803-8690-36199ce06d1a"}, { new Terminator(OperatorType.Minus), "50d92e97-418e-4dcc-8a0b-0417be94f780"}, { new Terminator(OperatorType.Plus), "c0cb164f-4a72-4001-968c-819a880d7325"}, { new Terminator(KeywordType.True), "913fa60f-50c8-4d05-91f5-5b70ac8d1897"}, { new Terminator(KeywordType.False), "c4f48802-cd68-4fe3-bc32-00801162e89d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "50d92e97-418e-4dcc-8a0b-0417be94f780") },
		{ "c0cb164f-4a72-4001-968c-819a880d7325", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f21739ef-3ff4-4f17-b5c8-a4134572c66f"}, { Terminator.NumberTerminator, "0d3b5204-314f-414c-adff-857477889392"}, { new NonTerminator(NonTerminatorType.Variable), "d54a0897-bdc5-4d3d-b42e-404557419658"}, { new Terminator(DelimiterType.LeftParenthesis), "09065ad9-dc24-4477-a84b-bfc8bd2d6846"}, { Terminator.IdentifierTerminator, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8"}, { new Terminator(KeywordType.Not), "bf895578-c816-4803-8690-36199ce06d1a"}, { new Terminator(OperatorType.Minus), "50d92e97-418e-4dcc-8a0b-0417be94f780"}, { new Terminator(OperatorType.Plus), "c0cb164f-4a72-4001-968c-819a880d7325"}, { new Terminator(KeywordType.True), "913fa60f-50c8-4d05-91f5-5b70ac8d1897"}, { new Terminator(KeywordType.False), "c4f48802-cd68-4fe3-bc32-00801162e89d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c0cb164f-4a72-4001-968c-819a880d7325") },
		{ "913fa60f-50c8-4d05-91f5-5b70ac8d1897", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "913fa60f-50c8-4d05-91f5-5b70ac8d1897") },
		{ "c4f48802-cd68-4fe3-bc32-00801162e89d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c4f48802-cd68-4fe3-bc32-00801162e89d") },
		{ "99cf5aee-d2d7-415c-ae4d-462e014d2906", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "b4711c5d-b620-4264-afbb-b9c0c922a3e7"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "99cf5aee-d2d7-415c-ae4d-462e014d2906") },
		{ "b31a52f2-01e2-4fa8-afbc-8e8e8905a826", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "b31a52f2-01e2-4fa8-afbc-8e8e8905a826") },
		{ "eb37b301-f1c6-4e88-88da-64a2ef2cf3cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "eb37b301-f1c6-4e88-88da-64a2ef2cf3cf") },
		{ "b228072e-f8ae-4d54-b451-29af3cc36d20", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b228072e-f8ae-4d54-b451-29af3cc36d20") },
		{ "fa802866-a329-4298-be36-d2a1934c570c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "fcad1619-4999-4a32-ace3-e835fdcdb47a"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fa802866-a329-4298-be36-d2a1934c570c") },
		{ "9dc9f1da-47c5-40a9-b21b-de605e5ba8a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "4638bb94-54eb-426b-89f9-53b4520bae92"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9dc9f1da-47c5-40a9-b21b-de605e5ba8a7") },
		{ "9696a74f-961e-44e5-82b5-28a41b1da496", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1b1cc606-8208-4949-8e34-222a622c43c3"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9696a74f-961e-44e5-82b5-28a41b1da496") },
		{ "b6cb8a23-1706-4bd3-92d1-0406492297d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "b6cb8a23-1706-4bd3-92d1-0406492297d7") },
		{ "77069d94-e311-4e01-98a5-002aa94ee79e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "77069d94-e311-4e01-98a5-002aa94ee79e") },
		{ "f9911432-da07-47ec-9c36-0b9854c4f2f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "f9911432-da07-47ec-9c36-0b9854c4f2f5") },
		{ "5e179093-4c37-46f9-b0de-b2c56dba9bcc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "86e571b7-c676-4732-a1a3-0992a508e59d"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5e179093-4c37-46f9-b0de-b2c56dba9bcc") },
		{ "75390dae-a0c4-450d-9947-cee2e9eac9bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "022fcc80-0225-48bb-b486-27943ce817fd"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "75390dae-a0c4-450d-9947-cee2e9eac9bb") },
		{ "93be189a-2064-4d1d-b49c-73509eca1398", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "93be189a-2064-4d1d-b49c-73509eca1398") },
		{ "1898a929-8341-45b5-a450-45fad252fe00", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1898a929-8341-45b5-a450-45fad252fe00") },
		{ "2394c1c3-e09e-411b-967d-49e899858b63", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "2394c1c3-e09e-411b-967d-49e899858b63") },
		{ "c78c2b19-3353-4170-bacd-94613091bd28", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "c78c2b19-3353-4170-bacd-94613091bd28") },
		{ "bce60ed5-192a-4eb0-8ff8-6d0f01727d9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "a00864ef-1e3f-4633-9049-57681b0cfdfb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bce60ed5-192a-4eb0-8ff8-6d0f01727d9c") },
		{ "63ab766c-87fb-4de4-93f5-d1bd7b84f62d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "7ca6cb9f-7d55-46ba-b411-8fe6c9d7cf04"}, { new NonTerminator(NonTerminatorType.Factor), "6e04f9ad-ddf6-4dbc-95c6-d9f74176d822"}, { Terminator.NumberTerminator, "1be01006-2f44-4e8d-97ab-75ca7ca56c58"}, { new NonTerminator(NonTerminatorType.Variable), "a7ba29ce-27b6-4fca-931f-5020c6c27046"}, { new Terminator(DelimiterType.LeftParenthesis), "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2"}, { Terminator.IdentifierTerminator, "b4cc1630-8532-470b-bda4-758795dbae90"}, { new Terminator(KeywordType.Not), "4ec65aff-034b-47b2-a50c-bba5d2a000c0"}, { new Terminator(OperatorType.Minus), "70fd34e2-8883-485e-a64c-f28e4126247f"}, { new Terminator(OperatorType.Plus), "9d2de763-4c51-4609-b174-240c92c8cb47"}, { new Terminator(KeywordType.True), "8349d5b7-2252-438a-8101-a6a3f7409ca4"}, { new Terminator(KeywordType.False), "57f25122-b6dd-4051-8886-5e886abf7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "63ab766c-87fb-4de4-93f5-d1bd7b84f62d") },
		{ "3fbcca5a-0862-4697-9ded-bd9623639886", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fa9cb35d-3258-46fd-b5ed-13e287d5f8d7"}, { Terminator.NumberTerminator, "1be01006-2f44-4e8d-97ab-75ca7ca56c58"}, { new NonTerminator(NonTerminatorType.Variable), "a7ba29ce-27b6-4fca-931f-5020c6c27046"}, { new Terminator(DelimiterType.LeftParenthesis), "e9e5ce3f-8f2b-43ad-80ff-54048d8fecf2"}, { Terminator.IdentifierTerminator, "b4cc1630-8532-470b-bda4-758795dbae90"}, { new Terminator(KeywordType.Not), "4ec65aff-034b-47b2-a50c-bba5d2a000c0"}, { new Terminator(OperatorType.Minus), "70fd34e2-8883-485e-a64c-f28e4126247f"}, { new Terminator(OperatorType.Plus), "9d2de763-4c51-4609-b174-240c92c8cb47"}, { new Terminator(KeywordType.True), "8349d5b7-2252-438a-8101-a6a3f7409ca4"}, { new Terminator(KeywordType.False), "57f25122-b6dd-4051-8886-5e886abf7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3fbcca5a-0862-4697-9ded-bd9623639886") },
		{ "90876872-747a-439a-b3ff-def962c2d693", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "88b049a4-0a8c-44ec-9e78-4271804319b5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "90876872-747a-439a-b3ff-def962c2d693") },
		{ "5350b6f1-e47d-4158-bcf9-f7c6a3c29fe3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d8aa5076-033b-4b51-85d5-2c748e30b229"}, { new NonTerminator(NonTerminatorType.ExpressionList), "c6660a37-d7e5-4943-835c-2245ebe5f753"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5350b6f1-e47d-4158-bcf9-f7c6a3c29fe3") },
		{ "c1fb4fca-dca1-4456-9305-48576a6e2fd4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c1fb4fca-dca1-4456-9305-48576a6e2fd4") },
		{ "cf33cbf3-072c-49f3-a3a9-00b65ef7b19e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "c6dfb59d-52a5-4741-9682-426591d0df9e"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cf33cbf3-072c-49f3-a3a9-00b65ef7b19e") },
		{ "879e84e7-daf9-4329-a78f-c609f45fc65c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "879e84e7-daf9-4329-a78f-c609f45fc65c") },
		{ "27ce5e63-ff9a-4899-b9a5-5934ec0c93d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "27ce5e63-ff9a-4899-b9a5-5934ec0c93d6") },
		{ "e1d9e863-493d-4821-a7a9-dddff8f4515e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e1d9e863-493d-4821-a7a9-dddff8f4515e") },
		{ "dc3117b5-f322-4887-88ae-7e2704de97ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "dc3117b5-f322-4887-88ae-7e2704de97ca") },
		{ "5fcfb5bc-42b8-4581-bafc-5bbf53cc725b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5fcfb5bc-42b8-4581-bafc-5bbf53cc725b") },
		{ "e2fcccd1-d7ce-4eed-84ec-722c0ca0fcfc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "e2fcccd1-d7ce-4eed-84ec-722c0ca0fcfc") },
		{ "ac432171-fe62-48be-99b8-b82c8d9d8f49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "c83a1530-4708-4b0d-b68f-7f816044f528"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac432171-fe62-48be-99b8-b82c8d9d8f49") },
		{ "d55fd873-6fb4-416a-baa5-11e00fd3c6ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2863d6b1-81d4-4a37-8f04-b70f62fb623b"}, { new NonTerminator(NonTerminatorType.Factor), "5b6fd525-664b-4b84-a281-a0e3e94112f5"}, { Terminator.NumberTerminator, "0d3b5204-314f-414c-adff-857477889392"}, { new NonTerminator(NonTerminatorType.Variable), "d54a0897-bdc5-4d3d-b42e-404557419658"}, { new Terminator(DelimiterType.LeftParenthesis), "09065ad9-dc24-4477-a84b-bfc8bd2d6846"}, { Terminator.IdentifierTerminator, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8"}, { new Terminator(KeywordType.Not), "bf895578-c816-4803-8690-36199ce06d1a"}, { new Terminator(OperatorType.Minus), "50d92e97-418e-4dcc-8a0b-0417be94f780"}, { new Terminator(OperatorType.Plus), "c0cb164f-4a72-4001-968c-819a880d7325"}, { new Terminator(KeywordType.True), "913fa60f-50c8-4d05-91f5-5b70ac8d1897"}, { new Terminator(KeywordType.False), "c4f48802-cd68-4fe3-bc32-00801162e89d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d55fd873-6fb4-416a-baa5-11e00fd3c6ff") },
		{ "2175173b-f979-413c-8da7-c50ac7121f68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "63e8b2cf-fdd8-48f8-9e94-a09d68688121"}, { Terminator.NumberTerminator, "0d3b5204-314f-414c-adff-857477889392"}, { new NonTerminator(NonTerminatorType.Variable), "d54a0897-bdc5-4d3d-b42e-404557419658"}, { new Terminator(DelimiterType.LeftParenthesis), "09065ad9-dc24-4477-a84b-bfc8bd2d6846"}, { Terminator.IdentifierTerminator, "7a7bdaad-8aad-492c-b265-4d1d2f79e7c8"}, { new Terminator(KeywordType.Not), "bf895578-c816-4803-8690-36199ce06d1a"}, { new Terminator(OperatorType.Minus), "50d92e97-418e-4dcc-8a0b-0417be94f780"}, { new Terminator(OperatorType.Plus), "c0cb164f-4a72-4001-968c-819a880d7325"}, { new Terminator(KeywordType.True), "913fa60f-50c8-4d05-91f5-5b70ac8d1897"}, { new Terminator(KeywordType.False), "c4f48802-cd68-4fe3-bc32-00801162e89d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2175173b-f979-413c-8da7-c50ac7121f68") },
		{ "86f8719c-dd82-4a75-834e-d0dd01617281", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3fde1f0b-25de-49be-87b3-3de2b87019da"},}, new Dictionary<Terminator, ReduceInformation>{ }, "86f8719c-dd82-4a75-834e-d0dd01617281") },
		{ "8f110252-27ff-41bd-926f-a0aeb3cac968", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9d1c5510-3a56-4606-844f-98148266eb96"}, { new NonTerminator(NonTerminatorType.ExpressionList), "e634905c-e196-46e5-9e03-c283aaf25d4e"}, { new NonTerminator(NonTerminatorType.Expression), "3dd8fdbf-966e-4e50-814e-e42e9d0fa8cb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "76f0d1ad-c34a-466d-8253-559dda913cad"}, { new NonTerminator(NonTerminatorType.Term), "e43ff673-e432-4ca6-a673-1610a617a04c"}, { new NonTerminator(NonTerminatorType.Factor), "c533182a-88ca-4990-ba69-8cad415533c8"}, { Terminator.NumberTerminator, "5dff7871-fd3e-4474-a5d1-6999b6b73dcd"}, { new NonTerminator(NonTerminatorType.Variable), "0835bf10-0ed5-4eb6-8e7a-c18eb1a6a79b"}, { new Terminator(DelimiterType.LeftParenthesis), "8ad03c91-d8a2-43f3-a9b9-785c90530517"}, { Terminator.IdentifierTerminator, "7cee472a-26f5-4b06-b5a6-8cd88e3d9f50"}, { new Terminator(KeywordType.Not), "713922ed-8e68-4d95-83db-f8544e1587eb"}, { new Terminator(OperatorType.Minus), "cb641776-84bd-40c8-91e7-ed34b5560acf"}, { new Terminator(OperatorType.Plus), "7cc341e9-b0ae-4558-a46f-75f72b3dbe0a"}, { new Terminator(KeywordType.True), "6db98640-0b70-4753-92a5-b25fad75bb99"}, { new Terminator(KeywordType.False), "85c68c56-8c61-4219-969e-b531eba59f49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8f110252-27ff-41bd-926f-a0aeb3cac968") },
		{ "bbfc0c11-9ff3-41eb-8014-cf919907a1b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "bbfc0c11-9ff3-41eb-8014-cf919907a1b8") },
		{ "cac807ed-93d5-470c-9bfb-b38584252c57", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9339cf4b-19a9-4e9f-94e4-49ddedf26162"}, { new NonTerminator(NonTerminatorType.Expression), "448c905c-5cc0-4646-85aa-112043d970e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b190a4d5-1ed9-4785-a10c-7dcab76321a6"}, { new NonTerminator(NonTerminatorType.Term), "fe52cbda-3e9c-491e-b578-970cd3929ef0"}, { new NonTerminator(NonTerminatorType.Factor), "2dc1ec98-133e-4375-8295-8aa394bf8492"}, { Terminator.NumberTerminator, "fd7f3201-4476-4775-b6f0-65eeaa91478e"}, { new NonTerminator(NonTerminatorType.Variable), "967d95dd-847f-420a-b578-41cebd118238"}, { new Terminator(DelimiterType.LeftParenthesis), "189a808f-b459-4a2d-b40a-9a3ee1d80ec4"}, { Terminator.IdentifierTerminator, "afba1d4b-61d5-499f-8c1d-b0ad46898948"}, { new Terminator(KeywordType.Not), "7e1993fc-adab-4dcc-a9a3-efb5af7b4db0"}, { new Terminator(OperatorType.Minus), "cb04fa57-83f1-42b6-9acc-e1a68e9d6b50"}, { new Terminator(OperatorType.Plus), "2d6df5a4-ccc0-458e-8452-ba5a9ad598ef"}, { new Terminator(KeywordType.True), "9ca05cd7-eb2e-4720-a786-80e30d2d2331"}, { new Terminator(KeywordType.False), "2cd9c8ee-2538-4d77-9adf-f3d3172025cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cac807ed-93d5-470c-9bfb-b38584252c57") },
		{ "c85af13d-5ae3-4c2f-8de4-6311ce0f897c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c85af13d-5ae3-4c2f-8de4-6311ce0f897c") },
		{ "333a4eb3-0494-4eb3-981a-e5f2e82bde13", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "333a4eb3-0494-4eb3-981a-e5f2e82bde13") },
		{ "f21739ef-3ff4-4f17-b5c8-a4134572c66f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f21739ef-3ff4-4f17-b5c8-a4134572c66f") },
		{ "fcad1619-4999-4a32-ace3-e835fdcdb47a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "fcad1619-4999-4a32-ace3-e835fdcdb47a") },
		{ "4638bb94-54eb-426b-89f9-53b4520bae92", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "4638bb94-54eb-426b-89f9-53b4520bae92") },
		{ "86e571b7-c676-4732-a1a3-0992a508e59d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "86e571b7-c676-4732-a1a3-0992a508e59d") },
		{ "022fcc80-0225-48bb-b486-27943ce817fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "022fcc80-0225-48bb-b486-27943ce817fd") },
		{ "a00864ef-1e3f-4633-9049-57681b0cfdfb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "b6bbd660-afa3-4959-bae2-7c1b6ea73416"}, { new Terminator(KeywordType.Integer), "8e00c7b1-2698-40a2-9d75-c6c231ca214d"}, { new Terminator(KeywordType.Real), "1af623d2-70f9-4436-8ef1-6a6f38eb66ce"}, { new Terminator(KeywordType.Boolean), "c111394e-c196-4e91-9e1f-29f2ca60aae1"}, { new Terminator(KeywordType.Character), "154e611b-b0ac-4ff7-a430-1b750a012404"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a00864ef-1e3f-4633-9049-57681b0cfdfb") },
		{ "7ca6cb9f-7d55-46ba-b411-8fe6c9d7cf04", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "3fbcca5a-0862-4697-9ded-bd9623639886"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "7ca6cb9f-7d55-46ba-b411-8fe6c9d7cf04") },
		{ "fa9cb35d-3258-46fd-b5ed-13e287d5f8d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "fa9cb35d-3258-46fd-b5ed-13e287d5f8d7") },
		{ "88b049a4-0a8c-44ec-9e78-4271804319b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "88b049a4-0a8c-44ec-9e78-4271804319b5") },
		{ "d8aa5076-033b-4b51-85d5-2c748e30b229", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d8aa5076-033b-4b51-85d5-2c748e30b229") },
		{ "c6660a37-d7e5-4943-835c-2245ebe5f753", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "87a85571-c510-4d6e-bf31-4c8198caf909"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c6660a37-d7e5-4943-835c-2245ebe5f753") },
		{ "c6dfb59d-52a5-4741-9682-426591d0df9e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5b8d9d36-d417-4c78-8f70-ede4fd4a87b9"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c6dfb59d-52a5-4741-9682-426591d0df9e") },
		{ "c83a1530-4708-4b0d-b68f-7f816044f528", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "9e78e346-70ce-4f30-866f-539c3be41f0b"}, { new NonTerminator(NonTerminatorType.Variable), "02da0c34-6cbd-42b8-a890-cdd4e7f62670"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "10b72ef2-2d50-4e2e-ad5e-9777f87581fc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "4abb7227-aa91-422b-8c07-5cefec0f972c"}, { new Terminator(KeywordType.If), "c9b058f7-972c-4a63-a2f6-4dd0fbda04e4"}, { new Terminator(KeywordType.For), "05a3f0e8-28df-4577-943e-299acda150d7"}, { Terminator.IdentifierTerminator, "c57c40e0-158e-40bf-ba1d-793d854fc4d2"}, { new Terminator(KeywordType.Begin), "6101ca1c-a6a9-41b4-a112-c00138796288"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "c83a1530-4708-4b0d-b68f-7f816044f528") },
		{ "2863d6b1-81d4-4a37-8f04-b70f62fb623b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2175173b-f979-413c-8da7-c50ac7121f68"}, { new Terminator(OperatorType.Multiply), "b478e88c-8ab6-4b13-a8bb-59838af033b0"}, { new Terminator(OperatorType.Divide), "eade5b89-478e-431f-8b2c-a336366766ef"}, { new Terminator(KeywordType.Divide), "cac88815-70df-444a-a247-aa4328ff8b44"}, { new Terminator(KeywordType.Mod), "fe820fb4-bb24-498c-bbbb-a204e3bf646e"}, { new Terminator(KeywordType.And), "c143f543-403a-456f-8173-33073fd62586"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2863d6b1-81d4-4a37-8f04-b70f62fb623b") },
		{ "63e8b2cf-fdd8-48f8-9e94-a09d68688121", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "63e8b2cf-fdd8-48f8-9e94-a09d68688121") },
		{ "3fde1f0b-25de-49be-87b3-3de2b87019da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "3fde1f0b-25de-49be-87b3-3de2b87019da") },
		{ "9d1c5510-3a56-4606-844f-98148266eb96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "9d1c5510-3a56-4606-844f-98148266eb96") },
		{ "e634905c-e196-46e5-9e03-c283aaf25d4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c2b6704d-3860-458d-b5e2-a4d9e19f4a18"}, { new Terminator(DelimiterType.Comma), "93da2c69-ca85-444c-8720-a81732bebb2a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e634905c-e196-46e5-9e03-c283aaf25d4e") },
		{ "9339cf4b-19a9-4e9f-94e4-49ddedf26162", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "c23226a9-227e-45aa-b400-15dd4930c6f3"}, { new Terminator(DelimiterType.Comma), "5beec4cf-85ae-4849-9f2e-842ea74d7200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9339cf4b-19a9-4e9f-94e4-49ddedf26162") },
		{ "b6bbd660-afa3-4959-bae2-7c1b6ea73416", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "b6bbd660-afa3-4959-bae2-7c1b6ea73416") },
		{ "87a85571-c510-4d6e-bf31-4c8198caf909", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "87a85571-c510-4d6e-bf31-4c8198caf909") },
		{ "5b8d9d36-d417-4c78-8f70-ede4fd4a87b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5b8d9d36-d417-4c78-8f70-ede4fd4a87b9") },
		{ "9e78e346-70ce-4f30-866f-539c3be41f0b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "9e78e346-70ce-4f30-866f-539c3be41f0b") },
		{ "c2b6704d-3860-458d-b5e2-a4d9e19f4a18", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "c2b6704d-3860-458d-b5e2-a4d9e19f4a18") },
		{ "c23226a9-227e-45aa-b400-15dd4930c6f3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "c23226a9-227e-45aa-b400-15dd4930c6f3") },
	};

    private GeneratedGrammarParser()
    {
        foreach(GeneratedTransformer transformer in s_transformers.Values)
        {
            transformer.ConstructShiftTable(s_transformers);
        }
    }

    private static GeneratedGrammarParser s_instance = new GeneratedGrammarParser();

    public static GeneratedGrammarParser Instance => s_instance;

	public ITransformer BeginTransformer => s_transformers["0f0898d1-ab59-485e-8cb8-e1f9f76c9fed"];
	public NonTerminator Begin => new NonTerminator(NonTerminatorType.StartNonTerminator);
}
